@@ -1,8 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+# TEST_UNICODE_LITERALS
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 from ...extern import six
 
+import copy
+
 from .. import Constant
 from ...units import Quantity as Q
 from ...tests.helper import pytest
@@ -87,6 +92,18 @@ def test_g0():
     assert g0.unit.physical_type == 'acceleration'
 
 
+def test_b_wien():
+    """b_wien should give the correct peak wavelength for
+    given blackbody temperature. The Sun is used in this test.
+
+    """
+    from .. import b_wien
+    from ... import units as u
+    t = 5778 * u.K
+    w = (b_wien / t).to(u.nm)
+    assert round(w.value) == 502
+
+
 def test_unit():
 
     from ... import units as u
@@ -99,3 +116,12 @@ def test_unit():
             # that none of the constants defined in astropy have
             # invalid unit.
             assert not isinstance(val.unit, u.UnrecognizedUnit)
+
+
+def test_copy():
+    from ... import constants as const
+    cc = copy.deepcopy(const.c)
+    assert cc == const.c
+
+    cc = copy.copy(const.c)
+    assert cc == const.c
