@@ -8,30 +8,11 @@ latex.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-##
-## Redistribution and use in source and binary forms, with or without
-## modification, are permitted provided that the following conditions are met:
-##     * Redistributions of source code must retain the above copyright
-##       notice, this list of conditions and the following disclaimer.
-##     * Redistributions in binary form must reproduce the above copyright
-##       notice, this list of conditions and the following disclaimer in the
-##       documentation and/or other materials provided with the distribution.
-##     * Neither the name of the Smithsonian Astrophysical Observatory nor the
-##       names of its contributors may be used to endorse or promote products
-##       derived from this software without specific prior written permission.
-##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-## DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-## DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-## (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-## LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-## ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+from __future__ import absolute_import, division, print_function
 
 import re
+
+from ...extern import six
 from . import core
 
 latexdicts = {'AA':  {'tabletype': 'table',
@@ -41,6 +22,7 @@ latexdicts = {'AA':  {'tabletype': 'table',
                               'header_start': r'\hline \hline', 'header_end': r'\hline\hline',
                               'data_end': r'\hline\hline'},
               'template': {'tabletype': 'tabletype', 'caption': 'caption',
+                           'tablealign': 'tablealign',
                            'col_align': 'col_align', 'preamble': 'preamble',
                            'header_start': 'header_start',
                            'header_end': 'header_end', 'data_start': 'data_start',
@@ -56,8 +38,8 @@ def add_dictval_to_list(adict, key, alist):
     :param key: key of value
     :param list: list where value should be added
     '''
-    if key in adict.keys():
-        if type(adict[key]) == str:
+    if key in adict:
+        if isinstance(adict[key], six.string_types):
             alist.append(adict[key])
         else:
             alist.extend(adict[key])
@@ -89,17 +71,20 @@ class LatexHeader(core.BaseHeader):
             return None
 
     def write(self, lines):
-        if not 'col_align' in self.latex.keys():
+        if not 'col_align' in self.latex:
             self.latex['col_align'] = len(self.cols) * 'c'
-
-        lines.append(r'\begin{' + self.latex['tabletype'] + r'}')
+        if 'tablealign' in self.latex:
+            align = '[' + self.latex['tablealign'] + ']'
+        else:
+            align = ''
+        lines.append(r'\begin{' + self.latex['tabletype'] + r'}' + align)
         add_dictval_to_list(self.latex, 'preamble', lines)
-        if 'caption' in self.latex.keys():
+        if 'caption' in self.latex:
             lines.append(r'\caption{' + self.latex['caption'] + '}')
         lines.append(self.header_start + r'{' + self.latex['col_align'] + r'}')
         add_dictval_to_list(self.latex, 'header_start', lines)
         lines.append(self.splitter.join([x.name for x in self.cols]))
-        if 'units' in self.latex.keys():
+        if 'units' in self.latex:
             lines.append(self.splitter.join([self.latex['units'].get(x.name, ' ')
                                              for x in self.cols]))
         add_dictval_to_list(self.latex, 'header_end', lines)
@@ -197,6 +182,10 @@ class Latex(core.BaseReader):
                 ascii.write(data, sys.stdout, Writer = ascii.Latex,
                             latexdict = {'tabletype': 'table*'})
 
+        * tablealign : positioning of table in text.
+            The default is not to specifiy a position preference in the text.
+            If, e.g. the alignment is ``ht``, then the LaTeX will be ``\\begin{table}[ht]``.
+
         * col_align : Alignment of columns
             If not present all columns will be centered.
 
@@ -222,8 +211,8 @@ class Latex(core.BaseReader):
               ascii.write(data, Writer=ascii.Latex,
                                latexdict = {'units': {'mass': 'kg', 'speed': 'km/h'}})
 
-            If the column has no entry in the `units` dictionary, it defaults
-            to `' '`.
+            If the column has no entry in the ``units`` dictionary, it defaults
+            to ``' '``.
 
         Run the following code to see where each element of the
         dictionary is inserted in the LaTeX table::
@@ -298,7 +287,9 @@ class Latex(core.BaseReader):
 
 
 class AASTexHeader(LatexHeader):
-    '''In a `deluxetable` some header keywords differ from standard LaTeX.
+    '''In a `deluxetable
+    <http://fits.gsfc.nasa.gov/standard30/deluxetable.sty>`_ some header
+    keywords differ from standard LaTeX.
 
     This header is modified to take that into account.
     '''
@@ -308,22 +299,26 @@ class AASTexHeader(LatexHeader):
         return find_latex_line(lines, r'\tablehead')
 
     def write(self, lines):
-        if not 'col_align' in self.latex.keys():
+        if not 'col_align' in self.latex:
             self.latex['col_align'] = len(self.cols) * 'c'
-
-        lines.append(r'\begin{' + self.latex['tabletype'] + r'}{' + self.latex['col_align'] + r'}')
+        if 'tablealign' in self.latex:
+            align = '[' + self.latex['tablealign'] + ']'
+        else:
+            align = ''
+        lines.append(r'\begin{' + self.latex['tabletype'] + r'}{' + self.latex['col_align'] + r'}'
+                       + align)
         add_dictval_to_list(self.latex, 'preamble', lines)
-        if 'caption' in self.latex.keys():
+        if 'caption' in self.latex:
             lines.append(r'\tablecaption{' + self.latex['caption'] + '}')
         tablehead = ' & '.join([r'\colhead{' + x.name + '}' for x in self.cols])
-        if 'units' in self.latex.keys():
+        if 'units' in self.latex:
             tablehead += r'\\ ' + (self.splitter.join([self.latex[
                                    'units'].get(x.name, ' ') for x in self.cols]))
         lines.append(r'\tablehead{' + tablehead + '}')
 
 
 class AASTexData(LatexData):
-    '''In a `deluxetable` the data is enclosed in `\startdata` and `\enddata`
+    '''In a `deluxetable`_ the data is enclosed in `\startdata` and `\enddata`
     '''
     data_start = r'\startdata'
     data_end = r'\enddata'
@@ -340,7 +335,7 @@ class AASTexData(LatexData):
 
 
 class AASTexHeaderSplitter(LatexSplitter):
-    '''extract column names from a `deluxetable`
+    '''Extract column names from a `deluxetable`_.
 
     This splitter expects the following LaTeX code **in a single line**:
 
