@@ -7,9 +7,9 @@ Convenience functions
 The functions in this module provide shortcuts for some of the most basic
 operations on FITS files, such as reading and updating the header.  They are
 included directly in the 'astropy.io.fits' namespace so that they can be used
-like:
+like::
 
-    >>> astropy.io.fits.getheader(...)
+    astropy.io.fits.getheader(...)
 
 These functions are primarily for convenience when working with FITS files in
 the command-line interpreter.  If performing several operations on the same
@@ -30,22 +30,22 @@ explanation of all the different formats.
     arguments.  This is to avoid ambiguity and conflicts with the
     extension arguments.  For example, to set NAXIS=1 on the Primary HDU:
 
-    Wrong:
+    Wrong::
 
-        >>> astropy.io.fits.setval('myimage.fits', 'NAXIS', 1)
+        astropy.io.fits.setval('myimage.fits', 'NAXIS', 1)
 
     The above example will try to set the NAXIS value on the first extension
     HDU to blank.  That is, the argument '1' is assumed to specify an extension
     HDU.
 
-    Right:
+    Right::
 
-        >>> astropy.io.fits.setval('myimage.fits', 'NAXIS', value=1)
+        astropy.io.fits.setval('myimage.fits', 'NAXIS', value=1)
 
     This will set the NAXIS keyword to 1 on the primary HDU (the default).  To
-    specify the first extension HDU use:
+    specify the first extension HDU use::
 
-        >>> astropy.io.fits.setval('myimage.fits', 'NAXIS', value=1, ext=1)
+        astropy.io.fits.setval('myimage.fits', 'NAXIS', value=1, ext=1)
 
     This complexity arises out of the attempt to simultaneously support
     multiple argument formats that were used in past versions of PyFITS.
@@ -55,18 +55,19 @@ explanation of all the different formats.
 """
 
 
-import gzip
 import os
 
 import numpy as np
 
-from .file import PYTHON_MODES, _File
+from .file import FILE_MODES, _File
 from .hdu.base import _BaseHDU, _ValidHDU
 from .hdu.hdulist import fitsopen
 from .hdu.image import PrimaryHDU, ImageHDU
 from .hdu.table import BinTableHDU
 from .header import Header
-from .util import fileobj_closed, fileobj_name, isfile, _is_int
+from .util import (fileobj_closed, fileobj_name, fileobj_mode,
+                   fileobj_closed, _is_int)
+from ...extern.six import string_types
 from ...utils import deprecated
 
 
@@ -123,43 +124,43 @@ def getdata(filename, *args, **kwargs):
 
         No extra arguments implies the primary header::
 
-            >>> getdata('in.fits')
+            getdata('in.fits')
 
         By extension number::
 
-            >>> getdata('in.fits', 0)    # the primary header
-            >>> getdata('in.fits', 2)    # the second extension
-            >>> getdata('in.fits', ext=2) # the second extension
+            getdata('in.fits', 0)      # the primary header
+            getdata('in.fits', 2)      # the second extension
+            getdata('in.fits', ext=2)  # the second extension
 
         By name, i.e., ``EXTNAME`` value (if unique)::
 
-            >>> getdata('in.fits', 'sci')
-            >>> getdata('in.fits', extname='sci') # equivalent
+            getdata('in.fits', 'sci')
+            getdata('in.fits', extname='sci')  # equivalent
 
         Note ``EXTNAME`` values are not case sensitive
 
         By combination of ``EXTNAME`` and EXTVER`` as separate
         arguments or as a tuple::
 
-            >>> getdata('in.fits', 'sci', 2) # EXTNAME='SCI' & EXTVER=2
-            >>> getdata('in.fits', extname='sci', extver=2) # equivalent
-            >>> getdata('in.fits', ('sci', 2)) # equivalent
+            getdata('in.fits', 'sci', 2)  # EXTNAME='SCI' & EXTVER=2
+            getdata('in.fits', extname='sci', extver=2)  # equivalent
+            getdata('in.fits', ('sci', 2))  # equivalent
 
         Ambiguous or conflicting specifications will raise an exception::
 
-            >>> getdata('in.fits', ext=('sci',1), extname='err', extver=2)
+            getdata('in.fits', ext=('sci',1), extname='err', extver=2)
 
-    header : bool (optional)
+    header : bool, optional
         If `True`, return the data and the header of the specified HDU as a
         tuple.
 
-    lower, upper : bool (optional)
-        If `lower` or `upper` are `True`, the field names in the
+    lower, upper : bool, optional
+        If ``lower`` or ``upper`` are `True`, the field names in the
         returned data object will be converted to lower or upper case,
         respectively.
 
-    view : ndarray (optional)
-        When given, the data will be turned wrapped in the given ndarray
+    view : ndarray, optional
+        When given, the data will be returned wrapped in the given ndarray
         subclass by calling::
 
            data.view(view)
@@ -173,8 +174,8 @@ def getdata(filename, *args, **kwargs):
     array : array, record array or groups data object
         Type depends on the type of the extension being referenced.
 
-        If the optional keyword `header` is set to `True`, this
-        function will return a (`data`, `header`) tuple.
+        If the optional keyword ``header`` is set to `True`, this
+        function will return a (``data``, ``header``) tuple.
     """
 
     mode, closed = _get_file_mode(filename)
@@ -250,7 +251,7 @@ def getval(filename, keyword, *args, **kwargs):
 
     Returns
     -------
-    keyword value : string, integer, or float
+    keyword value : str, int, or float
     """
 
     if 'do_not_scale_image_data' not in kwargs:
@@ -266,8 +267,8 @@ def setval(filename, keyword, *args, **kwargs):
 
     If the keyword already exists, it's value/comment will be updated.
     If it does not exist, a new card will be created and it will be
-    placed before or after the specified location.  If no `before` or
-    `after` is specified, it will be appended at the end.
+    placed before or after the specified location.  If no ``before`` or
+    ``after`` is specified, it will be appended at the end.
 
     When updating more than one keyword in a file, this convenience
     function is a much less efficient approach compared with opening
@@ -276,33 +277,33 @@ def setval(filename, keyword, *args, **kwargs):
     Parameters
     ----------
     filename : file path, file object, or file like object
-        Name of the FITS file, or file object If opened, mode must be
-        update (rb+).  An opened file object or `GzipFile` object will
-        be closed upon return.
+        Name of the FITS file, or file object If opened, mode must be update
+        (rb+).  An opened file object or `~gzip.GzipFile` object will be closed
+        upon return.
 
     keyword : str
         Keyword name
 
-    value : str, int, float (optional)
+    value : str, int, float, optional
         Keyword value (default: `None`, meaning don't modify)
 
-    comment : str (optional)
+    comment : str, optional
         Keyword comment, (default: `None`, meaning don't modify)
 
-    before : str, int (optional)
+    before : str, int, optional
         Name of the keyword, or index of the card before which the new card
-        will be placed.  The argument `before` takes precedence over `after` if
-        both are specified (default: `None`).
+        will be placed.  The argument ``before`` takes precedence over
+        ``after`` if both are specified (default: `None`).
 
-    after : str, int (optional)
+    after : str, int, optional
         Name of the keyword, or index of the card after which the new card will
         be placed. (default: `None`).
 
-    savecomment : bool (optional)
+    savecomment : bool, optional
         When `True`, preserve the current comment for an existing keyword.  The
-        argument `savecomment` takes precedence over `comment` if both
-        specified.  If `comment` is not specified then the current comment will
-        automatically be preserved  (default: `False`).
+        argument ``savecomment`` takes precedence over ``comment`` if both
+        specified.  If ``comment`` is not specified then the current comment
+        will automatically be preserved  (default: `False`).
 
     ext, extname, extver
         The rest of the arguments are for extension specification.
@@ -340,9 +341,9 @@ def delval(filename, keyword, *args, **kwargs):
     ----------
 
     filename : file path, file object, or file like object
-        Name of the FITS file, or file object If opened, mode must be
-        update (rb+).  An opened file object or `GzipFile` object will
-        be closed upon return.
+        Name of the FITS file, or file object If opened, mode must be update
+        (rb+).  An opened file object or `~gzip.GzipFile` object will be closed
+        upon return.
 
     keyword : str, int
         Keyword name or index
@@ -375,26 +376,29 @@ def writeto(filename, data, header=None, output_verify='exception',
     Parameters
     ----------
     filename : file path, file object, or file like object
-        File to write to.  If opened, must be opened for append (ab+).
+        File to write to.  If opened, must be opened in a writeable binary
+        mode such as 'wb' or 'ab+'.
 
     data : array, record array, or groups data object
         data to write to the new file
 
-    header : `Header` object (optional)
-        the header associated with `data`. If `None`, a header
+    header : `Header` object, optional
+        the header associated with ``data``. If `None`, a header
         of the appropriate type is created for the supplied data. This
         argument is optional.
 
     output_verify : str
-        Output verification option.  Must be one of ``"fix"``,
-        ``"silentfix"``, ``"ignore"``, ``"warn"``, or
-        ``"exception"``.  See :ref:`verify` for more info.
+        Output verification option.  Must be one of ``"fix"``, ``"silentfix"``,
+        ``"ignore"``, ``"warn"``, or ``"exception"``.  May also be any
+        combination of ``"fix"`` or ``"silentfix"`` with ``"+ignore"``,
+        ``+warn``, or ``+exception" (e.g. ``"fix+warn"``).  See :ref:`verify`
+        for more info.
 
-    clobber : bool (optional)
+    clobber : bool, optional
         If `True`, and if filename already exists, it will overwrite
         the file.  Default is `False`.
 
-    checksum : bool (optional)
+    checksum : bool, optional
         If `True`, adds both ``DATASUM`` and ``CHECKSUM`` cards to the
         headers of all HDU's written to the file.
     """
@@ -410,32 +414,31 @@ def append(filename, data, header=None, checksum=False, verify=True, **kwargs):
     """
     Append the header/data to FITS file if filename exists, create if not.
 
-    If only `data` is supplied, a minimal header is created.
+    If only ``data`` is supplied, a minimal header is created.
 
     Parameters
     ----------
     filename : file path, file object, or file like object
-        File to write to.  If opened, must be opened for update (rb+)
-        unless it is a new file, then it must be opened for append
-        (ab+).  A file or `GzipFile` object opened for update will be
-        closed after return.
+        File to write to.  If opened, must be opened for update (rb+) unless it
+        is a new file, then it must be opened for append (ab+).  A file or
+        `~gzip.GzipFile` object opened for update will be closed after return.
 
     data : array, table, or group data object
         the new data used for appending
 
-    header : `Header` object (optional)
-        The header associated with `data`.  If `None`, an appropriate
-        header will be created for the data object supplied.
+    header : `Header` object, optional
+        The header associated with ``data``.  If `None`, an appropriate header
+        will be created for the data object supplied.
 
-    checksum : bool (optional)
-        When `True` adds both ``DATASUM`` and ``CHECKSUM`` cards to
-        the header of the HDU when written to the file.
+    checksum : bool, optional
+        When `True` adds both ``DATASUM`` and ``CHECKSUM`` cards to the header
+        of the HDU when written to the file.
 
-    verify: bool (optional)
-        When `True`, the existing FITS file will be read in to verify
-        it for correctness before appending.  When `False`, content is
-        simply appended to the end of the file.  Setting *verify* to
-        `False` can be much faster.
+    verify : bool, optional
+        When `True`, the existing FITS file will be read in to verify it for
+        correctness before appending.  When `False`, content is simply appended
+        to the end of the file.  Setting ``verify`` to `False` can be much
+        faster.
 
     kwargs
         Any additional keyword arguments to be passed to
@@ -479,30 +482,29 @@ def update(filename, data, *args, **kwargs):
     Parameters
     ----------
     filename : file path, file object, or file like object
-        File to update.  If opened, mode must be update (rb+).  An
-        opened file object or `GzipFile` object will be closed upon
-        return.
+        File to update.  If opened, mode must be update (rb+).  An opened file
+        object or `~gzip.GzipFile` object will be closed upon return.
 
     data : array, table, or group data object
         the new data used for updating
 
-    header : `Header` object (optional)
-        The header associated with `data`.  If `None`, an appropriate
-        header will be created for the data object supplied.
+    header : `Header` object, optional
+        The header associated with ``data``.  If `None`, an appropriate header
+        will be created for the data object supplied.
 
     ext, extname, extver
-        The rest of the arguments are flexible: the 3rd argument can
-        be the header associated with the data.  If the 3rd argument
-        is not a `Header`, it (and other positional arguments) are
-        assumed to be the extension specification(s).  Header and
-        extension specs can also be keyword arguments.  For example::
+        The rest of the arguments are flexible: the 3rd argument can be the
+        header associated with the data.  If the 3rd argument is not a
+        `Header`, it (and other positional arguments) are assumed to be the
+        extension specification(s).  Header and extension specs can also be
+        keyword arguments.  For example::
 
-            >>> update(file, dat, hdr, 'sci')  # update the 'sci' extension
-            >>> update(file, dat, 3)  # update the 3rd extension
-            >>> update(file, dat, hdr, 3)  # update the 3rd extension
-            >>> update(file, dat, 'sci', 2)  # update the 2nd SCI extension
-            >>> update(file, dat, 3, header=hdr)  # update the 3rd extension
-            >>> update(file, dat, header=hdr, ext=5) # update the 5th extension
+            update(file, dat, hdr, 'sci')  # update the 'sci' extension
+            update(file, dat, 3)  # update the 3rd extension
+            update(file, dat, hdr, 3)  # update the 3rd extension
+            update(file, dat, 'sci', 2)  # update the 2nd SCI extension
+            update(file, dat, 3, header=hdr)  # update the 3rd extension
+            update(file, dat, header=hdr, ext=5)  # update the 5th extension
 
     kwargs
         Any additional keyword arguments to be passed to
@@ -542,11 +544,12 @@ def info(filename, output=None, **kwargs):
     ----------
     filename : file path, file object, or file like object
         FITS file to obtain info from.  If opened, mode must be one of
-        the following: rb, rb+, or ab+.
+        the following: rb, rb+, or ab+ (i.e. the file must be readable).
 
-    output : file (optional)
-        File-like object to output the HDU info to.  Outputs to stdout by
-        default.
+    output : file, bool, optional
+        A file-like object to write the output to.  If ``False``, does not
+        output to a file and instead returns a list of tuples representing the
+        HDU info.  Writes to ``sys.stdout`` by default.
 
     kwargs
         Any additional keyword arguments to be passed to
@@ -554,7 +557,7 @@ def info(filename, output=None, **kwargs):
         *Note:* This function sets ``ignore_missing_end=True`` by default.
     """
 
-    mode, closed = _get_file_mode(filename, default='copyonwrite')
+    mode, closed = _get_file_mode(filename, default='readonly')
     # Set the default value for the ignore_missing_end parameter
     if not 'ignore_missing_end' in kwargs:
         kwargs['ignore_missing_end'] = True
@@ -580,16 +583,16 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
     filename : file path, file object or file-like object
         Input fits file.
 
-    datafile : file path, file object or file-like object (optional)
+    datafile : file path, file object or file-like object, optional
         Output data file.  The default is the root name of the input
         fits file appended with an underscore, followed by the
         extension number (ext), followed by the extension ``.txt``.
 
-    cdfile : file path, file object or file-like object (optional)
+    cdfile : file path, file object or file-like object, optional
         Output column definitions file.  The default is `None`,
         no column definitions output is produced.
 
-    hfile : file path, file object or file-like object (optional)
+    hfile : file path, file object or file-like object, optional
         Output header parameters file.  The default is `None`,
         no header parameters output is produced.
 
@@ -604,7 +607,7 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
     -----
     The primary use for the `tabledump` function is to allow editing in a
     standard text editor of the table data and parameters.  The
-    `tcreate` function can be used to reassemble the table from the
+    ``tcreate`` function can be used to reassemble the table from the
     three ASCII files.
     """
 
@@ -612,12 +615,14 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
     # and leave the file in the same state (opened or closed) as when
     # the function was called
 
-    mode, closed = _get_file_mode(filename, default='copyonwrite')
+    mode, closed = _get_file_mode(filename, default='readonly')
     f = fitsopen(filename, mode=mode)
 
     # Create the default data file name if one was not provided
 
     if not datafile:
+        # TODO: Really need to provide a better way to access the name of any
+        # files underlying an HDU
         root, tail = os.path.splitext(f._HDUList__file.name)
         datafile = root + '_' + repr(ext) + '.txt'
 
@@ -629,7 +634,7 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
 tabledump.__doc__ += BinTableHDU._tdump_file_format.replace('\n', '\n    ')
 
 
-@deprecated('3.1', alternative=':func:`tabledump`')
+@deprecated('0.1', alternative=':func:`tabledump`')
 def tdump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
           clobber=False):
     tabledump(filename, datafile, cdfile, hfile, ext, clobber)
@@ -654,7 +659,7 @@ def tableload(datafile, cdfile, hfile=None):
         dimensions, undefined values, scale factors, and offsets
         associated with the columns in the table.
 
-    hfile : file path, file object or file-like object (optional)
+    hfile : file path, file object or file-like object, optional
         Input parameter definition file containing the header
         parameter definitions to be associated with the table.
         If `None`, a minimal header is constructed.
@@ -671,7 +676,7 @@ def tableload(datafile, cdfile, hfile=None):
 tableload.__doc__ += BinTableHDU._tdump_file_format.replace('\n', '\n    ')
 
 
-@deprecated('3.1', alternative=':func:`tableload`')
+@deprecated('0.1', alternative=':func:`tableload`')
 def tcreate(datafile, cdfile, hfile=None):
     return tableload(datafile, cdfile, hfile)
 
@@ -702,7 +707,7 @@ def _getext(filename, mode, *args, **kwargs):
             if ext is not None or extname is not None or extver is not None:
                 raise TypeError(err_msg)
             ext = args[0]
-        elif isinstance(args[0], basestring):
+        elif isinstance(args[0], string_types):
             # The first arg is an extension name; it could still be valid
             # to provide an extver kwarg
             if ext is not None or extname is not None:
@@ -722,13 +727,13 @@ def _getext(filename, mode, *args, **kwargs):
         raise TypeError('Too many positional arguments.')
 
     if (ext is not None and
-        not (_is_int(ext) or
-             (isinstance(ext, tuple) and len(ext) == 2 and
-              isinstance(ext[0], basestring) and _is_int(ext[1])))):
-            raise ValueError(
-                'The ext keyword must be either an extension number '
-                '(zero-indexed) or a (extname, extver) tuple.')
-    if extname is not None and not isinstance(extname, basestring):
+            not (_is_int(ext) or
+                 (isinstance(ext, tuple) and len(ext) == 2 and
+                  isinstance(ext[0], string_types) and _is_int(ext[1])))):
+        raise ValueError(
+            'The ext keyword must be either an extension number '
+            '(zero-indexed) or a (extname, extver) tuple.')
+    if extname is not None and not isinstance(extname, string_types):
         raise ValueError('The extname argument must be a string.')
     if extver is not None and not _is_int(extver):
         raise ValueError('The extver argument must be an integer.')
@@ -784,8 +789,7 @@ def _stat_filename_or_fileobj(filename):
     return name, closed, noexist_or_empty
 
 
-# TODO: Replace this with fileobj_mode
-def _get_file_mode(filename, default='copyonwrite'):
+def _get_file_mode(filename, default='readonly'):
     """
     Allow file object to already be opened in any of the valid modes and
     and leave the file in the same state (opened or closed) as when
@@ -793,23 +797,14 @@ def _get_file_mode(filename, default='copyonwrite'):
     """
 
     mode = default
-    closed = True
+    closed = fileobj_closed(filename)
 
-    if hasattr(filename, 'closed'):
-        closed = filename.closed
-    elif hasattr(filename, 'fileobj') and filename.fileobj is not None:
-        closed = filename.fileobj.closed
-
-    if (isfile(filename) or
-        isinstance(filename, gzip.GzipFile) and not closed):
-        if isinstance(filename, gzip.GzipFile):
-            file_mode = filename.fileobj.mode
-        else:
-            file_mode = filename.mode
-
-        for key, val in PYTHON_MODES.iteritems():
-            if val == file_mode:
-                mode = key
-                break
+    fmode = fileobj_mode(filename)
+    if fmode is not None:
+        mode = FILE_MODES.get(fmode)
+        if mode is None:
+            raise IOError(
+                "File mode of the input file object (%r) cannot be used to "
+                "read/write FITS files." % fmode)
 
     return mode, closed
