@@ -2,20 +2,24 @@
 
 from __future__ import division, with_statement
 
-import gzip
+from ...utils.compat import gzip as _astropy_gzip
+import gzip as _system_gzip
 import mmap
 import os
 import tempfile
-import urllib
 import warnings
 import zipfile
 
+from functools import reduce
+
 import numpy as np
 from numpy import memmap as Memmap
 
 from .util import (isreadable, iswritable, isfile, fileobj_open, fileobj_name,
                    fileobj_closed, fileobj_mode, _array_from_file,
-                   _array_to_file, _write_string, b)
+                   _array_to_file, _write_string, encode_ascii)
+from ...extern.six import b, string_types
+from ...extern.six.moves import urllib
 from ...utils.exceptions import AstropyUserWarning
 
 
@@ -67,6 +71,7 @@ MEMMAP_MODES = {'readonly': 'c', 'copyonwrite': 'c', 'update': 'r+',
 GZIP_MAGIC = b('\x1f\x8b\x08')
 PKZIP_MAGIC = b('\x50\x4b\x03\x04')
 
+_GZIP_FILE_TYPES = (_astropy_gzip, _system_gzip)
 
 class _File(object):
     """
@@ -103,7 +108,7 @@ class _File(object):
         if mode not in PYFITS_MODES:
             raise ValueError("Mode '%s' not recognized" % mode)
 
-        if (isinstance(fileobj, basestring) and
+        if (isinstance(fileobj, string_types) and
                 mode not in ('ostream', 'append') and
                 not os.path.exists(fileobj)):
 
@@ -114,7 +119,7 @@ class _File(object):
                 if not os.path.splitdrive(fileobj)[0]:
                     # Basically if the filename (on Windows anyways) doesn't
                     # have a drive letter try to open it as a URL
-                    self.name, _ = urllib.urlretrieve(fileobj)
+                    self.name, _ = urllib.request.urlretrieve(fileobj)
                 else:
                     # Otherwise the file was already not found so just raise
                     # a ValueError
@@ -122,7 +127,7 @@ class _File(object):
             except (TypeError, ValueError, IOError):
                 # A couple different exceptions can occur here when passing a
                 # filename into urlretrieve in Python 3
-                raise IOError('File does not exist: %r' % fileobj)
+                raise IOError('File does not exist: {0!r}'.format(fileobj))
         else:
             self.name = fileobj_name(fileobj)
 
@@ -142,12 +147,14 @@ class _File(object):
         # Initialize the internal self.__file object
         if _is_random_access_file_backed(fileobj):
             self._open_fileobj(fileobj, mode, clobber)
-        elif isinstance(fileobj, basestring):
+        elif isinstance(fileobj, string_types):
             self._open_filename(fileobj, mode, clobber)
         else:
             self._open_filelike(fileobj, mode, clobber)
 
-        if isinstance(fileobj, gzip.GzipFile):
+        self.fileobj_mode = fileobj_mode(self.__file)
+
+        if isinstance(fileobj, (_astropy_gzip.GzipFile, _system_gzip.GzipFile)):
             self.compression = 'gzip'
         elif isinstance(fileobj, zipfile.ZipFile):
             # Reading from zip files is supported but not writing (yet)
@@ -292,7 +299,7 @@ class _File(object):
         # present, we implement our own support for it here
         if not hasattr(self.__file, 'seek'):
             return
-        if isinstance(self.__file, gzip.GzipFile):
+        if isinstance(self.__file, (_astropy_gzip.GzipFile, _system_gzip.GzipFile)):
             if whence:
                 if whence == 1:
                     offset = self.__file.offset + offset
@@ -378,7 +385,7 @@ class _File(object):
         elif isfile(fileobj):
             self.__file = fileobj_open(self.name, PYFITS_MODES[mode])
         else:
-            self.__file = gzip.open(self.name, PYFITS_MODES[mode])
+            self.__file = _astropy_gzip.open(self.name, PYFITS_MODES[mode])
 
         if fmode == 'ab+':
             # Return to the beginning of the file--in Python 3 when opening in
@@ -442,7 +449,7 @@ class _File(object):
 
         if ext == '.gz' or magic.startswith(GZIP_MAGIC):
             # Handle gzip files
-            self.__file = gzip.open(self.name, PYFITS_MODES[mode])
+            self.__file = _astropy_gzip.open(self.name, PYFITS_MODES[mode])
             self.compression = 'gzip'
         elif ext == '.zip' or magic.startswith(PKZIP_MAGIC):
             # Handle zip files
@@ -475,6 +482,7 @@ class _File(object):
                 warnings.warn('Failed to create mmap: %s; mmap use will be '
                               'disabled' % str(e), AstropyUserWarning)
                 _File._mmap_available = False
+                del exc
                 return False
             try:
                 mm.flush()
@@ -532,4 +540,4 @@ def _is_random_access_file_backed(fileobj):
     from an already opened `zipfile.ZipFile` object.
     """
 
-    return isfile(fileobj) or isinstance(fileobj, gzip.GzipFile)
+    return isfile(fileobj) or isinstance(fileobj, (_astropy_gzip.GzipFile, _system_gzip.GzipFile))
