@@ -10,16 +10,18 @@ import warnings
 
 import numpy as np
 
-from .. import EXTENSION_NAME_CASE_SENSITIVE
+from .. import conf
 from ..card import Card
 from ..file import _File
 from ..header import Header, HEADER_END_RE, _pad_length
 from ..util import (_is_int, _is_pseudo_unsigned, _unsigned_zero,
                     itersubclasses, encode_ascii, decode_ascii,
-                    _get_array_mmap, _array_to_file)
+                    _get_array_mmap, _array_to_file, first)
 from ..verify import _Verify, _ErrList
 
+from ....extern.six import string_types, next
 from ....utils import lazyproperty, deprecated
+from ....utils.compat import ignored
 from ....utils.exceptions import AstropyUserWarning
 
 
@@ -56,12 +58,15 @@ def _hdu_class_from_header(cls, header):
                     break
             except NotImplementedError:
                 continue
-            except Exception as e:
+            except Exception as exc:
                 warnings.warn(
                     'An exception occurred matching an HDU header to the '
-                    'appropriate HDU type: %s' % unicode(e), AstropyUserWarning)
-                warnings.warn('The HDU will be treated as corrupted.', AstropyUserWarning)
+                    'appropriate HDU type: {0}'.format(exc),
+                    AstropyUserWarning)
+                warnings.warn('The HDU will be treated as corrupted.',
+                              AstropyUserWarning)
                 klass = _CorruptedHDU
+                del exc
                 break
 
     return klass
@@ -139,9 +144,9 @@ class _BaseHDU(object):
 
     @name.setter
     def name(self, value):
-        if not isinstance(value, basestring):
+        if not isinstance(value, string_types):
             raise TypeError("'name' attribute must be a string")
-        if not EXTENSION_NAME_CASE_SENSITIVE():
+        if not conf.extension_name_case_sensitive:
             value = value.upper()
         if 'EXTNAME' in self._header:
             self._header['EXTNAME'] = value
@@ -192,7 +197,7 @@ class _BaseHDU(object):
         return self._data_loaded and self.data is not None
 
     @property
-    @deprecated('3.2', alternative='the `._header_offset` attribute',
+    @deprecated('0.3', alternative='the `._header_offset` attribute',
                 pending=True)
     def _hdrLoc(self):
         """The byte offset of this HDU's header in the file it came from;
@@ -202,13 +207,13 @@ class _BaseHDU(object):
         return self._header_offset
 
     @_hdrLoc.setter
-    @deprecated('3.2', alternative='the `._header_offset` attribute',
+    @deprecated('0.3', alternative='the `._header_offset` attribute',
                 pending=True)
     def _hdrLoc(self, value):
         self._header_offset = value
 
     @property
-    @deprecated('3.2', alternative='the `._data_offset` attribute',
+    @deprecated('0.3', alternative='the `._data_offset` attribute',
                 pending=True)
     def _datLoc(self):
         """The byte offset of this HDU's data portion in the file it came from;
@@ -218,13 +223,13 @@ class _BaseHDU(object):
         return self._data_offset
 
     @_datLoc.setter
-    @deprecated('3.2', alternative='the `._data_offset` attribute',
+    @deprecated('0.3', alternative='the `._data_offset` attribute',
                 pending=True)
     def _datLoc(self, value):
         self._data_offset = value
 
     @property
-    @deprecated('3.2', alternative='the `._data_size` attribute',
+    @deprecated('0.3', alternative='the `._data_size` attribute',
                 pending=True)
     def _datSpan(self):
         """The byte size of this HDU's data portion in the file it came from;
@@ -234,13 +239,13 @@ class _BaseHDU(object):
         return self._data_size
 
     @_datSpan.setter
-    @deprecated('3.2', alternative='the `._data_size` attribute',
+    @deprecated('0.3', alternative='the `._data_size` attribute',
                 pending=True)
     def _datSpan(self, value):
         self._data_size = value
 
     @property
-    @deprecated('3.2', alternative='the `._header_offset` attribute',
+    @deprecated('0.3', alternative='the `._header_offset` attribute',
                 pending=True)
     def _hdrLoc(self):
         """The byte offset of this HDU's header in the file it came from;
@@ -250,13 +255,13 @@ class _BaseHDU(object):
         return self._header_offset
 
     @_hdrLoc.setter
-    @deprecated('3.2', alternative='the `._header_offset` attribute',
+    @deprecated('0.3', alternative='the `._header_offset` attribute',
                 pending=True)
     def _hdrLoc(self, value):
         self._header_offset = value
 
     @property
-    @deprecated('3.2', alternative='the `._data_offset` attribute',
+    @deprecated('0.3', alternative='the `._data_offset` attribute',
                 pending=True)
     def _datLoc(self):
         """The byte offset of this HDU's data portion in the file it came from;
@@ -266,13 +271,13 @@ class _BaseHDU(object):
         return self._data_offset
 
     @_datLoc.setter
-    @deprecated('3.2', alternative='the `._data_offset` attribute',
+    @deprecated('0.3', alternative='the `._data_offset` attribute',
                 pending=True)
     def _datLoc(self, value):
         self._data_offset = value
 
     @property
-    @deprecated('3.2', alternative='the `._data_size` attribute',
+    @deprecated('0.3', alternative='the `._data_size` attribute',
                 pending=True)
     def _datSpan(self):
         """The byte size of this HDU's data portion in the file it came from;
@@ -282,7 +287,7 @@ class _BaseHDU(object):
         return self._data_size
 
     @_datSpan.setter
-    @deprecated('3.2', alternative='the `._data_size` attribute',
+    @deprecated('0.3', alternative='the `._data_size` attribute',
                 pending=True)
     def _datSpan(self, value):
         self._data_size = value
@@ -300,12 +305,9 @@ class _BaseHDU(object):
     def match_header(cls, header):
         raise NotImplementedError
 
-    # TODO: This method is a bit kludgy, especially in how it still usually
-    # works with a fileobj, and how the first argument may or may not contain
-    # HDU data.  This should be rethought.
     @classmethod
-    def fromstring(cls, data, fileobj=None, offset=0, checksum=False,
-                   ignore_missing_end=False, **kwargs):
+    def fromstring(cls, data, checksum=False, ignore_missing_end=False,
+                   **kwargs):
         """
         Creates a new HDU object of the appropriate type from a string
         containing the HDU's entire header and, optionally, its data.
@@ -318,114 +320,37 @@ class _BaseHDU(object):
         Parameters
         ----------
         data : str, bytearray, memoryview, ndarray
-           A byte string contining the HDU's header and, optionally, its data.
-           If `fileobj` is not specified, and the length of `data` extends
-           beyond the header, then the trailing data is taken to be the HDU's
-           data.  If `fileobj` is specified then the trailing data is ignored.
+           A byte string contining the HDU's header and data.
 
-        fileobj : file (optional)
-           The file-like object that this HDU was read from.
-
-        offset : int (optional)
-           If `fileobj` is specified, the offset into the file-like object at
-           which this HDU begins.
-
-        checksum : bool (optional)
+        checksum : bool, optional
            Check the HDU's checksum and/or datasum.
 
-        ignore_missing_end : bool (optional)
-           Ignore a missing end card in the header data.  Note that without
-           the end card the end of the header can't be found, so the entire
-           data is just assumed to be the header.
+        ignore_missing_end : bool, optional
+           Ignore a missing end card in the header data.  Note that without the
+           end card the end of the header may be ambiguous and resulted in a
+           corrupt HDU.  In this case the assumption is that the first 2880
+           block that does not begin with valid FITS header data is the
+           beginning of the data.
 
-        kwargs : (optional)
-           May contain additional keyword arguments specific to an HDU type.
-           Any unrecognized kwargs are simply ignored.
+        kwargs : optional
+           May consist of additional keyword arguments specific to an HDU
+           type--these correspond to keywords recognized by the constructors of
+           different HDU classes such as `PrimaryHDU`, `ImageHDU`, or
+           `BinTableHDU`.  Any unrecognized keyword arguments are simply
+           ignored.
         """
 
-        if isinstance(data, Header):
-            header = data
-            if (not len(header) or
-                    header.keys()[0] not in ('SIMPLE', 'XTENSION')):
-                raise ValueError('Block does not begin with SIMPLE or '
-                                 'XTENSION')
-        else:
-            try:
-                # Test that the given object supports the buffer interface by
-                # ensuring an ndarray can be created from it
-                np.ndarray((), dtype='ubyte', buffer=data)
-            except TypeError:
-                raise TypeError(
-                    'The provided object %r does not contain an underlying '
-                    'memory buffer.  fromstring() requires an object that '
-                    'supports the buffer interface such as bytes, str '
-                    '(in Python 2.x but not in 3.x), buffer, memoryview, '
-                    'ndarray, etc.' % data)
-
-            if data[:8] not in [encode_ascii('SIMPLE  '),
-                                encode_ascii('XTENSION')]:
-                raise ValueError('Block does not begin with SIMPLE or '
-                                 'XTENSION')
-
-            # Make sure the end card is present
-            for match in HEADER_END_RE.finditer(data):
-                endpos = match.start()
-                if endpos % Card.length == 0:
-                    hdrlen = endpos + len(match.group())
-                    hdrlen += _pad_length(hdrlen)
-                    break
-            else:
-                if ignore_missing_end:
-                    hdrlen = len(data)
-                else:
-                    raise ValueError('Header missing END card.')
-
-            header = Header.fromstring(decode_ascii(data[:hdrlen]))
-        # Determine the appropriate arguments to pass to the constructor from
-        # self._kwargs.  self._kwargs contains any number of optional arguments
-        # that may or may not be valid depending on the HDU type
-        cls = _hdu_class_from_header(cls, header)
-        args, varargs, varkwargs, defaults = inspect.getargspec(cls.__init__)
-        new_kwargs = kwargs.copy()
-        if not varkwargs:
-            # If __init__ accepts arbitrary keyword arguments, then we can go
-            # ahead and pass all keyword arguments; otherwise we need to delete
-            # any that are invalid
-            for key in kwargs:
-                if key not in args:
-                    del new_kwargs[key]
-
-        hdu = cls(data=DELAYED, header=header, **new_kwargs)
-
-        hdu._file = fileobj
-
-        if not fileobj and len(data) > hdrlen:
-            # Provide an underlying buffer to read the data from
-            hdu._buffer = data
-
-        hdu._header_offset = offset            # beginning of the header area
-        if fileobj:
-            hdu._data_offset = fileobj.tell()  # beginning of the data area
-        else:
-            hdu._data_offset = hdrlen
-
-        # data area size, including padding
-        size = hdu.size
-        hdu._data_size = size + _pad_length(size)
-
-        # Checksums are not checked on invalid HDU types
-        if checksum and checksum != 'remove' and isinstance(hdu, _ValidHDU):
-            hdu._verify_checksum_datasum(checksum)
-
-        return hdu
+        return cls._readfrom_internal(data, checksum=checksum,
+                                      ignore_missing_end=ignore_missing_end,
+                                      **kwargs)
 
     @classmethod
     def readfrom(cls, fileobj, checksum=False, ignore_missing_end=False,
                  **kwargs):
         """
         Read the HDU from a file.  Normally an HDU should be opened with
-        `fitsopen()` which reads the entire HDU list in a FITS file.  But this
-        method is still provided for symmetry with `writeto()`.
+        :func:`open` which reads the entire HDU list in a FITS file.  But this
+        method is still provided for symmetry with :func:`writeto`.
 
         Parameters
         ----------
@@ -434,13 +359,13 @@ class _BaseHDU(object):
             beginning of the HDU.
 
         checksum : bool
-            If `True`, verifies that both ``DATASUM`` and
-            ``CHECKSUM`` card values (when present in the HDU header)
-            match the header and data of all HDU's in the file.
+            If `True`, verifies that both ``DATASUM`` and ``CHECKSUM`` card
+            values (when present in the HDU header) match the header and data
+            of all HDU's in the file.
 
         ignore_missing_end : bool
-            Do not issue an exception when opening a file that is
-            missing an ``END`` card in the last header.
+            Do not issue an exception when opening a file that is missing an
+            ``END`` card in the last header.
         """
 
         # TODO: Figure out a way to make it possible for the _File
@@ -448,15 +373,13 @@ class _BaseHDU(object):
         if not isinstance(fileobj, _File):
             fileobj = _File(fileobj)
 
-        hdr_offset = fileobj.tell()
-        hdr = Header.fromfile(fileobj, endcard=not ignore_missing_end)
 
-        hdu = cls.fromstring(hdr, fileobj=fileobj, offset=hdr_offset,
-                             checksum=checksum,
-                             ignore_missing_end=ignore_missing_end, **kwargs)
+        hdu = cls._readfrom_internal(fileobj, checksum=checksum,
+                                     ignore_missing_end=ignore_missing_end,
+                                     **kwargs)
 
         # If the checksum had to be checked the data may have already been read
-        # from the file, in which case we don't want to see relative
+        # from the file, in which case we don't want to seek relative
         fileobj.seek(hdu._data_offset + hdu._data_size, os.SEEK_SET)
         return hdu
 
@@ -476,7 +399,9 @@ class _BaseHDU(object):
         output_verify : str
             Output verification option.  Must be one of ``"fix"``,
             ``"silentfix"``, ``"ignore"``, ``"warn"``, or
-            ``"exception"``.  See :ref:`verify` for more info.
+            ``"exception"``.  May also be any combination of ``"fix"`` or
+            ``"silentfix"`` with ``"+ignore"``, ``+warn``, or ``+exception"
+            (e.g. ``"fix+warn"``).  See :ref:`verify` for more info.
 
         clobber : bool
             Overwrite the output file if exists.
@@ -492,6 +417,97 @@ class _BaseHDU(object):
         hdulist.writeto(name, output_verify, clobber=clobber,
                         checksum=checksum)
 
+    @classmethod
+    def _readfrom_internal(cls, data, header=None, checksum=False,
+                           ignore_missing_end=False, **kwargs):
+        """
+        Provides the bulk of the internal implementation for readfrom and
+        fromstring.
+
+        For some special cases, supports using a header that was already
+        created, and just using the input data for the actual array data.
+        """
+
+        hdu_buffer = None
+        hdu_fileobj = None
+        header_offset = 0
+
+        if isinstance(data, _File):
+            from_file = True
+            if header is None:
+                header_offset = data.tell()
+                header = Header.fromfile(data, endcard=not ignore_missing_end)
+            hdu_fileobj = data
+            data_offset = data.tell()  # *after* reading the header
+        else:
+            from_file = False
+            try:
+                # Test that the given object supports the buffer interface by
+                # ensuring an ndarray can be created from it
+                np.ndarray((), dtype='ubyte', buffer=data)
+            except TypeError:
+                raise TypeError(
+                    'The provided object %r does not contain an underlying '
+                    'memory buffer.  fromstring() requires an object that '
+                    'supports the buffer interface such as bytes, str '
+                    '(in Python 2.x but not in 3.x), buffer, memoryview, '
+                    'ndarray, etc.  This restriction is to ensure that '
+                    'efficient access to the array/table data is possible.'
+                    % data)
+
+            if header is None:
+                def block_iter(nbytes):
+                    idx = 0
+                    while idx < len(data):
+                        yield data[idx:idx + nbytes]
+                        idx += nbytes
+
+                header_str, header = Header._from_blocks(
+                    block_iter, True, '', not ignore_missing_end, True)
+
+                if len(data) > len(header_str):
+                    hdu_buffer = data
+            elif data:
+                hdu_buffer = data
+
+            header_offset = 0
+            data_offset = len(header_str)
+
+        # Determine the appropriate arguments to pass to the constructor from
+        # self._kwargs.  self._kwargs contains any number of optional arguments
+        # that may or may not be valid depending on the HDU type
+        cls = _hdu_class_from_header(cls, header)
+        args, varargs, varkwargs, defaults = inspect.getargspec(cls.__init__)
+        new_kwargs = kwargs.copy()
+        if not varkwargs:
+            # If __init__ accepts arbitrary keyword arguments, then we can go
+            # ahead and pass all keyword arguments; otherwise we need to delete
+            # any that are invalid
+            for key in kwargs:
+                if key not in args:
+                    del new_kwargs[key]
+
+        hdu = cls(data=DELAYED, header=header, **new_kwargs)
+
+        # One of these may be None, depending on whether the data came from a
+        # file or a string buffer--later this will be further abstracted
+        hdu._file = hdu_fileobj
+        hdu._buffer = hdu_buffer
+
+        hdu._header_offset = header_offset     # beginning of the header area
+        hdu._data_offset = data_offset         # beginning of the data area
+
+        # data area size, including padding
+        size = hdu.size
+        hdu._data_size = size + _pad_length(size)
+
+        # Checksums are not checked on invalid HDU types
+        if checksum and checksum != 'remove' and isinstance(hdu, _ValidHDU):
+            hdu._verify_checksum_datasum(checksum)
+
+        return hdu
+
+
     def _get_raw_data(self, shape, code, offset):
         """
         Return raw array from either the HDU's memory buffer or underlying
@@ -511,6 +527,8 @@ class _BaseHDU(object):
 
     # TODO: Rework checksum handling so that it's not necessary to add a
     # checksum argument here
+    # TODO: The BaseHDU class shouldn't even handle checksums since they're
+    # only implemented on _ValidHDU...
     def _prewriteto(self, checksum=False, inplace=False):
         self._update_uint_scale_keywords()
 
@@ -551,7 +569,9 @@ class _BaseHDU(object):
 
             if datasum_keyword in self._header:
                 del self._header[datasum_keyword]
-        elif modified or self._new:
+        elif (modified or self._new or
+                (checksum and ('CHECKSUM' not in self._header or
+                               'DATASUM' not in self._header))):
             if checksum == 'datasum':
                 self.add_datasum(datasum_keyword=datasum_keyword)
             elif checksum == 'nonstandard_datasum':
@@ -578,18 +598,14 @@ class _BaseHDU(object):
         if (self._has_data and self._standard and
                 _is_pseudo_unsigned(self.data.dtype)):
             for keyword in ('BSCALE', 'BZERO'):
-                try:
+                with ignored(KeyError):
                     del self._header[keyword]
-                except KeyError:
-                    pass
 
     def _writeheader(self, fileobj):
         offset = 0
         if not fileobj.simulateonly:
-            try:
+            with ignored(AttributeError, IOError):
                 offset = fileobj.tell()
-            except (AttributeError, IOError):
-                pass
 
             self._header.tofile(fileobj)
 
@@ -600,9 +616,6 @@ class _BaseHDU(object):
         else:
             size = len(str(self._header))
 
-        # Update hdrLoc with the new offset
-        self._header_offset = offset
-
         return offset, size
 
     def _writedata(self, fileobj):
@@ -641,10 +654,6 @@ class _BaseHDU(object):
         if not fileobj.simulateonly:
             fileobj.flush()
 
-        # Update datLoc with the new offset
-        self._data_offset = offset
-        self._data_size = size
-
         # return both the location and the size of the data area
         return offset, size
 
@@ -686,8 +695,14 @@ class _BaseHDU(object):
     def _writeto(self, fileobj, inplace=False, copy=False):
         # For now fileobj is assumed to be a _File object
         if not inplace or self._new:
-            self._writeheader(fileobj)
-            self._writedata(fileobj)
+            header_offset, _ = self._writeheader(fileobj)
+            data_offset, data_size = self._writedata(fileobj)
+
+            # Set the various data location attributes on newly-written HDUs
+            if self._new:
+                self._header_offset = header_offset
+                self._data_offset = data_offset
+                self._data_size = data_size
             return
 
         hdrloc = self._header_offset
@@ -840,7 +855,7 @@ class _NonstandardHDU(_BaseHDU, _Verify):
 
     def _writedata(self, fileobj):
         """
-        Differs from the base class `_writedata()` in that it doesn't
+        Differs from the base class :class:`_writedata` in that it doesn't
         automatically add padding, and treats the data as a string of raw bytes
         instead of an array.
         """
@@ -907,7 +922,7 @@ class _ValidHDU(_BaseHDU, _Verify):
         case?  Not sure...
         """
 
-        return header.keys()[0] not in ('SIMPLE', 'XTENSION')
+        return first(header.keys()) not in ('SIMPLE', 'XTENSION')
 
     @property
     def size(self):
@@ -931,14 +946,6 @@ class _ValidHDU(_BaseHDU, _Verify):
         """
         Calculates and returns the number of bytes that this HDU will write to
         a file.
-
-        Parameters
-        ----------
-        None
-
-        Returns
-        -------
-        Number of bytes
         """
 
         f = _File()
@@ -952,13 +959,9 @@ class _ValidHDU(_BaseHDU, _Verify):
         valid after a read or write of the associated file with no
         intervening changes to the `HDUList`.
 
-        Parameters
-        ----------
-        None
-
         Returns
         -------
-        dictionary or None
+        dict or None
 
            The dictionary details information about the locations of
            this HDU within an associated file.  Returns `None` when
@@ -996,7 +999,7 @@ class _ValidHDU(_BaseHDU, _Verify):
             data = None
         return self.__class__(data=data, header=self._header.copy())
 
-    @deprecated('3.2', alternative='the ``.name`` attribute or `Header.set`',
+    @deprecated('0.3', alternative='the ``.name`` attribute or `Header.set`',
                 pending=True)
     def update_ext_name(self, value, comment=None, before=None,
                         after=None, savecomment=False):
@@ -1006,30 +1009,30 @@ class _ValidHDU(_BaseHDU, _Verify):
         If the keyword already exists in the Header, it's value and/or comment
         will be updated.  If it does not exist, a new card will be created
         and it will be placed before or after the specified location.
-        If no `before` or `after` is specified, it will be appended at
+        If no ``before`` or ``after`` is specified, it will be appended at
         the end.
 
         Parameters
         ----------
         value : str
-            value to be used for the new extension name
+            Value to be used for the new extension name
 
         comment : str, optional
-            to be used for updating, default=None.
+            To be used for updating, default=None.
 
         before : str or int, optional
-            name of the keyword, or index of the `Card` before which
-            the new card will be placed in the Header.  The argument
-            `before` takes precedence over `after` if both specified.
+            Name of the keyword, or index of the `Card` before which the new
+            card will be placed in the Header.  The argument ``before`` takes
+            precedence over ``after`` if both are specified.
 
         after : str or int, optional
-            name of the keyword, or index of the `Card` after which
-            the new card will be placed in the Header.
+            Name of the keyword, or index of the `Card` after which
+            the new card will be placed in the Header
 
         savecomment : bool, optional
             When `True`, preserve the current comment for an existing
-            keyword.  The argument `savecomment` takes precedence over
-            `comment` if both specified.  If `comment` is not
+            keyword.  The argument ``savecomment`` takes precedence over
+            ``comment`` if both specified.  If ``comment`` is not
             specified then the current comment will automatically be
             preserved.
         """
@@ -1042,10 +1045,10 @@ class _ValidHDU(_BaseHDU, _Verify):
         # handles anyone who might use the before/after keywords to set the
         # position of the EXTNAME keyword.  Setting self.name = name does some
         # additional processing on the value such as handling
-        # EXTENSION_NAME_CASE_SENSITIVE
+        # conf.extension_name_case_sensitive
         self.name = value
 
-    @deprecated('3.2', alternative='the `.ver` attribute or `Header.set`',
+    @deprecated('0.3', alternative='the ``.ver`` attribute or `Header.set`',
                 pending=True)
     def update_ext_version(self, value, comment=None, before=None,
                            after=None, savecomment=False):
@@ -1055,30 +1058,30 @@ class _ValidHDU(_BaseHDU, _Verify):
         If the keyword already exists in the Header, it's value and/or comment
         will be updated.  If it does not exist, a new card will be created
         and it will be placed before or after the specified location.
-        If no `before` or `after` is specified, it will be appended at
+        If no ``before`` or ``after`` is specified, it will be appended at
         the end.
 
         Parameters
         ----------
         value : str
-            value to be used for the new extension version
+            Value to be used for the new extension version
 
         comment : str, optional
-            to be used for updating, default=None.
+            To be used for updating; default=None.
 
         before : str or int, optional
-            name of the keyword, or index of the `Card` before which
-            the new card will be placed in the Header.  The argument
-            `before` takes precedence over `after` if both specified.
+            Name of the keyword, or index of the `Card` before which the new
+            card will be placed in the Header.  The argument ``before`` takes
+            precedence over ``after`` if both are specified.
 
         after : str or int, optional
-            name of the keyword, or index of the `Card` after which
+            Name of the keyword, or index of the `Card` after which
             the new card will be placed in the Header.
 
         savecomment : bool, optional
             When `True`, preserve the current comment for an existing
-            keyword.  The argument `savecomment` takes precedence over
-            `comment` if both specified.  If `comment` is not
+            keyword.  The argument ``savecomment`` takes precedence over
+            ``comment`` if both specified.  If ``comment`` is not
             specified then the current comment will automatically be
             preserved.
         """
@@ -1137,7 +1140,7 @@ class _ValidHDU(_BaseHDU, _Verify):
 
         # Verify that the EXTNAME keyword exists and is a string
         if 'EXTNAME' in self._header:
-            if not isinstance(self._header['EXTNAME'], basestring):
+            if not isinstance(self._header['EXTNAME'], string_types):
                 err_text = 'The EXTNAME keyword must have a string value.'
                 fix_text = 'Converted the EXTNAME keyword to a string value.'
 
@@ -1159,11 +1162,51 @@ class _ValidHDU(_BaseHDU, _Verify):
         """
         Check the existence, location, and value of a required `Card`.
 
-        TODO: Write about parameters
+        Parameters
+        ----------
+        keyword : str
+            The keyword to validate
 
-        If `pos` = `None`, it can be anywhere.  If the card does not exist,
-        the new card will have the `fix_value` as its value when created.
-        Also check the card's value by using the `test` argument.
+        pos : int, callable
+            If an ``int``, this specifies the exact location this card should
+            have in the header.  Remember that Python is zero-indexed, so this
+            means ``pos=0`` requires the card to be the first card in the
+            header.  If given a callable, it should take one argument--the
+            actual position of the keyword--and return `True` or `False`.  This
+            can be used for custom evaluation.  For example if
+            ``pos=lambda idx: idx > 10`` this will check that the keyword's
+            index is greater than 10.
+
+        test : callable
+            This should be a callable (generally a function) that is passed the
+            value of the given keyword and returns `True` or `False`.  This can
+            be used to validate the value associated with the given keyword.
+
+        fix_value : str, int, float, complex, bool, None
+            A valid value for a FITS keyword to to use if the given ``test``
+            fails to replace an invalid value.  In other words, this provides
+            a default value to use as a replacement if the keyword's current
+            value is invalid.  If `None`, there is no replacement value and the
+            keyword is unfixable.
+
+        option : str
+            Output verification option.  Must be one of ``"fix"``,
+            ``"silentfix"``, ``"ignore"``, ``"warn"``, or
+            ``"exception"``.  May also be any combination of ``"fix"`` or
+            ``"silentfix"`` with ``"+ignore"``, ``+warn``, or ``+exception"
+            (e.g. ``"fix+warn"``).  See :ref:`verify` for more info.
+
+        errlist : list
+            A list of validation errors already found in the FITS file; this is
+            used primarily for the validation system to collect errors across
+            multiple HDUs and multiple calls to `req_cards`.
+
+        Notes
+        -----
+        If ``pos=None``, the card can be anywhere in the header.  If the card
+        does not exist, the new card will have the ``fix_value`` as its value
+        when created.  Also check the card's value by using the ``test``
+        argument.
         """
 
         errs = errlist
@@ -1249,7 +1292,7 @@ class _ValidHDU(_BaseHDU, _Verify):
             "standard" or "nonstandard", compute sum 2880 bytes at a time, or
             not
 
-        datasum_keyword: str, optional
+        datasum_keyword : str, optional
             The name of the header keyword to store the datasum value in;
             this is typically 'DATASUM' per convention, but there exist
             use cases in which a different keyword should be used
@@ -1261,10 +1304,9 @@ class _ValidHDU(_BaseHDU, _Verify):
 
         Notes
         -----
-        For testing purposes, provide a `when` argument to enable the
-        comment value in the card to remain consistent.  This will
-        enable the generation of a ``CHECKSUM`` card with a consistent
-        value.
+        For testing purposes, provide a ``when`` argument to enable the comment
+        value in the card to remain consistent.  This will enable the
+        generation of a ``CHECKSUM`` card with a consistent value.
         """
 
         cs = self._calculate_datasum(blocking)
@@ -1297,21 +1339,21 @@ class _ValidHDU(_BaseHDU, _Verify):
             "standard" or "nonstandard", compute sum 2880 bytes at a time, or
             not
 
-        checksum_keyword: str, optional
+        checksum_keyword : str, optional
             The name of the header keyword to store the checksum value in; this
             is typically 'CHECKSUM' per convention, but there exist use cases
             in which a different keyword should be used
 
-        datasum_keyword: str, optional
+        datasum_keyword : str, optional
             See ``checksum_keyword``
 
         Notes
         -----
-        For testing purposes, first call `add_datasum` with a `when`
-        argument, then call `add_checksum` with a `when` argument and
-        `override_datasum` set to `True`.  This will provide
-        consistent comments for both cards and enable the generation
-        of a ``CHECKSUM`` card with a consistent value.
+        For testing purposes, first call `add_datasum` with a ``when``
+        argument, then call `add_checksum` with a ``when`` argument and
+        ``override_datasum`` set to `True`.  This will provide consistent
+        comments for both cards and enable the generation of a ``CHECKSUM``
+        card with a consistent value.
         """
 
         if not override_datasum:
@@ -1414,9 +1456,9 @@ class _ValidHDU(_BaseHDU, _Verify):
             self._checksum = self._header['CHECKSUM']
             self._checksum_comment = self._header.comments['CHECKSUM']
             if not self.verify_checksum(blocking):
-                warnings.warn('Checksum verification failed for HDU %s.\n' %
-                              ((self.name, self.ver),), AstropyUserWarning)
-            del self._header['CHECKSUM']
+                warnings.warn(
+                    'Checksum verification failed for HDU {0}.\n'.format(
+                        (self.name, self.ver)), AstropyUserWarning)
         else:
             self._checksum = None
             self._checksum_comment = None
@@ -1426,9 +1468,9 @@ class _ValidHDU(_BaseHDU, _Verify):
             self._datasum_comment = self._header.comments['DATASUM']
 
             if not self.verify_datasum(blocking):
-                warnings.warn('Datasum verification failed for HDU %s.\n' %
-                              ((self.name, self.ver),), AstropyUserWarning)
-            del self._header['DATASUM']
+                warnings.warn(
+                    'Datasum verification failed for HDU {0}.\n'.format(
+                        (self.name, self.ver)), AstropyUserWarning)
         else:
             self._checksum = None
             self._checksum_comment = None
@@ -1490,13 +1532,13 @@ class _ValidHDU(_BaseHDU, _Verify):
 
         return s
 
-    def _compute_checksum(self, bytes, sum32=0, blocking="standard"):
+    def _compute_checksum(self, data, sum32=0, blocking="standard"):
         """
         Compute the ones-complement checksum of a sequence of bytes.
 
         Parameters
         ----------
-        bytes
+        data
             a memory region to checksum
 
         sum32
@@ -1516,17 +1558,17 @@ class _ValidHDU(_BaseHDU, _Verify):
         """
 
         blocklen = {'standard': 2880,
-                    'nonstandard': len(bytes),
+                    'nonstandard': len(data),
                     'either': 2880,  # do standard first
                     True: 2880}[blocking]
 
         sum32 = np.uint32(sum32)
-        for i in range(0, len(bytes), blocklen):
-            length = min(blocklen, len(bytes) - i)   # ????
-            sum32 = self._compute_hdu_checksum(bytes[i:i + length], sum32)
+        for i in range(0, len(data), blocklen):
+            length = min(blocklen, len(data) - i)   # ????
+            sum32 = self._compute_hdu_checksum(data[i:i + length], sum32)
         return sum32
 
-    def _compute_hdu_checksum(self, bytes, sum32=0):
+    def _compute_hdu_checksum(self, data, sum32=0):
         """
         Translated from FITS Checksum Proposal by Seaman, Pence, and Rots.
         Use uint32 literals as a hedge against type promotion to int64.
@@ -1541,20 +1583,20 @@ class _ValidHDU(_BaseHDU, _Verify):
         u16 = np.uint32(16)
         uFFFF = np.uint32(0xFFFF)
 
-        if bytes.nbytes % 2:
-            last = bytes[-1]
-            bytes = bytes[:-1]
+        if data.nbytes % 2:
+            last = data[-1]
+            data = data[:-1]
         else:
             last = np.uint32(0)
 
-        bytes = bytes.view('>u2')
+        data = data.view('>u2')
 
         hi = sum32 >> u16
         lo = sum32 & uFFFF
-        hi += np.add.reduce(bytes[0::2])
-        lo += np.add.reduce(bytes[1::2])
+        hi += np.add.reduce(data[0::2], dtype=np.uint64)
+        lo += np.add.reduce(data[1::2], dtype=np.uint64)
 
-        if (bytes.nbytes // 2) % 2:
+        if (data.nbytes // 2) % 2:
             lo += last << u8
         else:
             hi += last << u8
@@ -1605,7 +1647,7 @@ class _ValidHDU(_BaseHDU, _Verify):
 
     def _char_encode(self, value):
         """
-        Encodes the checksum `value` using the algorithm described
+        Encodes the checksum ``value`` using the algorithm described
         in SPR section A.7.2 and returns it as a 16 character string.
 
         Parameters
@@ -1710,7 +1752,7 @@ class NonstandardExtHDU(ExtensionHDU):
 
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         # A3DTABLE is not really considered a 'standard' extension, as it was
         # sort of the prototype for BINTABLE; however, since our BINTABLE
