@@ -204,6 +204,11 @@ class GroupData(FITS_rec):
 
     @property
     def data(self):
+        """
+        The raw group data represented as a multi-dimensional `numpy.ndarray`
+        array.
+        """
+
         # The last column in the coldefs is the data portion of the group
         return self.field(self._coldefs.names[-1])
 
@@ -235,16 +240,15 @@ class GroupData(FITS_rec):
 class GroupsHDU(PrimaryHDU, _TableLikeHDU):
     """
     FITS Random Groups HDU class.
+
+    See the :ref:`random-groups` section in the PyFITS documentation for more
+    details on working with this type of HDU.
     """
 
     _width2format = {8: 'B', 16: 'I', 32: 'J', 64: 'K', -32: 'E', -64: 'D'}
     _data_type = GroupData
 
     def __init__(self, data=None, header=None):
-        """
-        TODO: Write me
-        """
-
         super(GroupsHDU, self).__init__(data=data, header=header)
 
         # The name of the table record array field that will contain the group
@@ -509,7 +513,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
                 byteswapped = False
                 d = self.data
 
-            cs = self._compute_checksum(np.fromstring(d, dtype='ubyte'),
+            cs = self._compute_checksum(d.flatten().view(np.uint8),
                                         blocking=blocking)
 
             # If the data was byteswapped in this method then return it to
