@@ -13,16 +13,20 @@ import numpy as np
 from numpy import char as chararray
 
 from .base import DELAYED, _ValidHDU, ExtensionHDU
+# This module may have many dependencies on pyfits.column, but pyfits.column
+# has fewer dependencies overall, so it's easier to keep table/column-related
+# utilities in pyfits.column
 from ..column import (FITS2NUMPY, KEYWORD_NAMES, KEYWORD_ATTRIBUTES, TDEF_RE,
-                      Column, ColDefs, _AsciiColDefs, _FormatX, _FormatP,
-                      _FormatQ, _makep, _VLF, _parse_tformat,
-                      _scalar_to_format, _convert_format, _cmp_recformats,
-                      _get_index)
+                      Column, ColDefs, _AsciiColDefs, _FormatP, _FormatQ,
+                      _makep, _VLF, _parse_tformat, _scalar_to_format,
+                      _convert_format, _cmp_recformats, _get_index)
 from ..fitsrec import FITS_rec
 from ..header import Header, _pad_length
 from ..util import _is_int, _str_to_num
 
+from ....extern.six import string_types
 from ....utils import deprecated, lazyproperty
+from ....utils.compat import ignored
 from ....utils.exceptions import AstropyUserWarning
 
 
@@ -62,8 +66,67 @@ class _TableLikeHDU(_ValidHDU):
 
         raise NotImplementedError
 
+    @classmethod
+    def from_columns(cls, columns, header=None, nrows=0, fill=False,
+                     **kwargs):
+        """
+        Given either a `ColDefs` object, a sequence of `Column` objects,
+        or another table HDU or table data (a `FITS_rec` or multi-field
+        `numpy.ndarray` or `numpy.recarray` object, return a new table HDU of
+        the class this method was called on using the column definition from
+        the input.
+
+        This is an alternative to the now deprecated `new_table` function,
+        and otherwise accepts the same arguments.  See also
+        `FITS_rec.from_columns`.
+
+        Parameters
+        ----------
+        columns : sequence of `Column`, `ColDefs`, or other
+            The columns from which to create the table data, or an object with
+            a column-like structure from which a `ColDefs` can be instantiated.
+            This includes an existing `BinTableHDU` or `TableHDU`, or a
+            `numpy.recarray` to give some examples.
+
+            If these columns have data arrays attached that data may be used in
+            initializing the new table.  Otherwise the input columns will be
+            used as a template for a new table with the requested number of
+            rows.
+
+        header : `Header`
+            An optional `Header` object to instantiate the new HDU yet.  Header
+            keywords specifically related to defining the table structure (such
+            as the "TXXXn" keywords like TTYPEn) will be overridden by the
+            supplied column definitions, but all other informational and data
+            model-specific keywords are kept.
+
+        nrows : int
+            Number of rows in the new table.  If the input columns have data
+            associated with them, the size of the largest input column is used.
+            Otherwise the default is 0.
+
+        fill : bool
+            If `True`, will fill all cells with zeros or blanks.  If `False`,
+            copy the data from input, undefined cells will still be filled with
+            zeros/blanks.
+
+        Notes
+        -----
+
+        Any additional keyword arguments accepted by the HDU class's
+        ``__init__`` may also be passed in as keyword arguments.
+        """
+
+        coldefs = cls._columns_type(columns)
+        data = FITS_rec.from_columns(coldefs, nrows=nrows, fill=fill)
+        return cls(data=data, header=header, **kwargs)
+
     @lazyproperty
     def columns(self):
+        """
+        The :class:`ColDefs` objects describing the columns in this table.
+        """
+
         # The base class doesn't make any assumptions about where the column
         # definitions come from, so just return an empty ColDefs
         return ColDefs([])
@@ -124,6 +187,16 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
     FITS table extension base HDU class.
     """
 
+    _manages_own_heap = False
+    """
+    This flag implies that when writing VLA tables (P/Q format) the heap
+    pointers that go into P/Q table columns should not be reordered or
+    rearranged in any way by the default heap management code.
+
+    This is included primarily as an optimization for compressed image HDUs
+    which perform their own heap maintenance.
+    """
+
     def __init__(self, data=None, header=None, name=None, uint=False):
         """
         Parameters
@@ -138,7 +211,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
             name to be populated in ``EXTNAME`` keyword
 
         uint : bool, optional
-            set to ``True`` if the table contains unsigned integer columns.
+            set to `True` if the table contains unsigned integer columns.
         """
 
         super(_TableBaseHDU, self).__init__(data=data, header=header,
@@ -228,7 +301,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                 self.columns = self.data._coldefs
                 self.update()
 
-                try:
+                with ignored(TypeError, AttributeError):
                    # Make the ndarrays in the Column objects of the ColDefs
                    # object of the HDU reference the same ndarray as the HDU's
                    # FITS_rec object.
@@ -238,16 +311,12 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                     # Delete the _arrays attribute so that it is recreated to
                     # point to the new data placed in the column objects above
                     del self.columns._arrays
-                except (TypeError, AttributeError) as e:
-                    # This shouldn't happen as long as self.columns._arrays
-                    # is a lazyproperty
-                    pass
             elif data is None:
                 pass
             else:
                 raise TypeError('Table data has incorrect type.')
 
-        if not (isinstance(self._header[0], basestring) and
+        if not (isinstance(self._header[0], string_types) and
                 self._header[0].rstrip() == self._extension):
             self._header[0] = (self._extension, self._ext_comment)
 
@@ -268,6 +337,10 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
 
     @lazyproperty
     def columns(self):
+        """
+        The :class:`ColDefs` objects describing the columns in this table.
+        """
+
         if self._has_data and hasattr(self.data, '_coldefs'):
             return self.data._coldefs
         return self._columns_type(self)
@@ -318,7 +391,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
             self.columns = self.data.columns
             self.update()
 
-            try:
+            with ignored(TypeError, AttributeError):
                # Make the ndarrays in the Column objects of the ColDefs
                # object of the HDU reference the same ndarray as the HDU's
                # FITS_rec object.
@@ -328,10 +401,6 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                 # Delete the _arrays attribute so that it is recreated to
                 # point to the new data placed in the column objects above
                 del self.columns._arrays
-            except (TypeError, AttributeError):
-                # This shouldn't happen as long as self.columns._arrays
-                # is a lazyproperty
-                pass
         elif data is None:
             pass
         else:
@@ -369,23 +438,23 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
         # touch the data, so it's defined (in the case of reading from a
         # FITS file)
         self.data
-        return new_table(self.columns, header=self._header,
-                         tbtype=self.columns._tbtype)
+        return new_table(self.columns, header=self._header)
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._has_data:
-            self.data._scale_back()
+            self.data._scale_back(
+                update_heap_pointers=not self._manages_own_heap)
             # check TFIELDS and NAXIS2
             self._header['TFIELDS'] = len(self.data._coldefs)
             self._header['NAXIS2'] = self.data.shape[0]
 
             # calculate PCOUNT, for variable length tables
-            tbsize = self.header['NAXIS1'] * self.header['NAXIS2']
-            heapstart = self.header.get('THEAP', tbsize)
+            tbsize = self._header['NAXIS1'] * self._header['NAXIS2']
+            heapstart = self._header.get('THEAP', tbsize)
             self.data._gap = heapstart - tbsize
             pcount = self.data._heapsize + self.data._gap
             if pcount > 0:
-                self.header['PCOUNT'] = pcount
+                self._header['PCOUNT'] = pcount
 
             # update the other T****n keywords
             self._populate_table_keywords()
@@ -497,7 +566,7 @@ class TableHDU(_TableBaseHDU):
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return card.keyword == 'XTENSION' and xtension == cls._extension
 
@@ -579,52 +648,44 @@ class BinTableHDU(_TableBaseHDU):
     _extension = 'BINTABLE'
     _ext_comment = 'binary table extension'
 
+
     @classmethod
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return (card.keyword == 'XTENSION' and
                 xtension in (cls._extension, 'A3DTABLE'))
 
-    def _calculate_datasum_from_data(self, data, blocking):
+    def _calculate_datasum_with_heap(self, blocking):
         """
         Calculate the value for the ``DATASUM`` card given the input data
         """
 
-        # Check the byte order of the data.  If it is little endian we
-        # must swap it before calculating the datasum.
-        for i in range(data._nfields):
-            coldata = data.field(i)
+        swapped = self._binary_table_byte_swap()
+        try:
+            dout = self.data.view(dtype='ubyte')
+            csum = self._compute_checksum(dout, blocking=blocking)
 
-            if not isinstance(coldata, chararray.chararray):
-                if isinstance(coldata, _VLF):
-                    for j, d in enumerate(coldata):
-                        if not isinstance(d, chararray.chararray):
-                            if d.itemsize > 1:
-                                if d.dtype.str[0] != '>':
-                                    d[:] = d.byteswap()
-                                    d.dtype = d.dtype.newbyteorder('>')
-                        field = np.rec.recarray.field(data, i)[j:j + 1]
-                        if field.dtype.str[0] != '>':
-                            field.byteswap(True)
-                else:
-                    if coldata.itemsize > 1:
-                        if data.field(i).dtype.str[0] != '>':
-                            data.field(i)[:] = data.field(i).byteswap()
-        data.dtype = data.dtype.newbyteorder('>')
+            # Now add in the heap data to the checksum (we can skip any gap
+            # between the table and the heap since it's all zeros and doesn't
+            # contribute to the checksum
+            for idx in range(self.data._nfields):
+                if isinstance(self.data.columns._recformats[idx], _FormatP):
+                    for coldata in self.data.field(idx):
+                        # coldata should already be byteswapped from the call
+                        # to _binary_table_byte_swap
+                        if not len(coldata):
+                            continue
 
-        dout = data.view(dtype='ubyte')
+                        csum = self._compute_checksum(coldata, csum,
+                                                      blocking=blocking)
 
-        for i in range(data._nfields):
-            if isinstance(data._coldefs._recformats[i], _FormatP):
-                for coldata in data.field(i):
-                    if len(coldata) > 0:
-                        dout = np.append(dout, coldata.view(dtype='ubyte'))
-
-        cs = self._compute_checksum(dout, blocking=blocking)
-        return cs
+            return csum
+        finally:
+            for arr in swapped:
+                arr.byteswap(True)
 
     def _calculate_datasum(self, blocking):
         """
@@ -632,8 +693,10 @@ class BinTableHDU(_TableBaseHDU):
         """
 
         if self._has_data:
-            # We have the data to be used.
-            return self._calculate_datasum_from_data(self.data, blocking)
+            # This method calculates the datasum while incorporating any
+            # heap data, which is obviously not handled from the base
+            # _calculate_datasum
+            return self._calculate_datasum_with_heap(blocking)
         else:
             # This is the case where the data has not been read from the file
             # yet.  We can handle that in a generic manner so we do it in the
@@ -645,12 +708,48 @@ class BinTableHDU(_TableBaseHDU):
         size = 0
 
         if self.data is not None:
-            size += self._binary_table_byte_swap(fileobj)
+            swapped = self._binary_table_byte_swap()
+            try:
+                fileobj.writearray(self.data)
+                # write out the heap of variable length array columns this has
+                # to be done after the "regular" data is written (above)
+                fileobj.write((self.data._gap * '\0').encode('ascii'))
+
+                nbytes = self.data._gap
+
+                if not self._manages_own_heap:
+                    # Write the heap data one column at a time, in the order
+                    # that the data pointers appear in the column (regardless
+                    # if that data pointer has a different, previous heap
+                    # offset listed)
+                    for idx in range(self.data._nfields):
+                        if not isinstance(self.data.columns._recformats[idx],
+                                          _FormatP):
+                            continue
+
+                        field = self.data.field(idx)
+                        for row in field:
+                            if len(row) > 0:
+                                nbytes += row.nbytes
+                                if not fileobj.simulateonly:
+                                    fileobj.writearray(row)
+                else:
+                    heap_data = self.data._get_heap_data()
+                    if len(heap_data) > 0:
+                        nbytes += len(heap_data)
+                        if not fileobj.simulateonly:
+                            fileobj.writearray(heap_data)
+
+                self.data._heapsize = nbytes - self.data._gap
+                size += nbytes
+            finally:
+                for arr in swapped:
+                    arr.byteswap(True)
             size += self.data.size * self.data.itemsize
 
         return size
 
-    def _binary_table_byte_swap(self, fileobj):
+    def _binary_table_byte_swap(self):
         """Prepares data in the native FITS format and writes the raw bytes
         out to the given file object.  This handles byte swapping from native
         to big endian (if necessary).  In addition, however, this also handles
@@ -659,86 +758,34 @@ class BinTableHDU(_TableBaseHDU):
         """
 
         to_swap = []
-        swapped = []
-        nbytes = 0
+
         if sys.byteorder == 'little':
             swap_types = ('<', '=')
         else:
             swap_types = ('<',)
-        try:
-            if not fileobj.simulateonly:
-                for idx in range(self.data._nfields):
-                    field = np.rec.recarray.field(self.data, idx)
-                    if isinstance(field, chararray.chararray):
-                        continue
-                    recformat = self.data.columns._recformats[idx]
-                    # only swap unswapped
-                    if field.itemsize > 1 and field.dtype.str[0] in swap_types:
-                        to_swap.append(field)
-                    # deal with var length table
-                    if isinstance(recformat, _FormatP):
-                        coldata = self.data.field(idx)
-                        for c in coldata:
-                            if (not isinstance(c, chararray.chararray) and
-                                c.itemsize > 1 and
-                                    c.dtype.str[0] in swap_types):
-                                to_swap.append(c)
 
-                while to_swap:
-                    obj = to_swap.pop()
-                    obj.byteswap(True)
-                    swapped.append(obj)
+        for idx in range(self.data._nfields):
+            field = np.rec.recarray.field(self.data, idx)
+            if isinstance(field, chararray.chararray):
+                continue
 
-                fileobj.writearray(self.data)
+            # only swap unswapped
+            if field.itemsize > 1 and field.dtype.str[0] in swap_types:
+                to_swap.append(field)
 
-                # write out the heap of variable length array
-                # columns this has to be done after the
-                # "regular" data is written (above)
-                fileobj.write((self.data._gap * '\0').encode('ascii'))
+            # deal with var length table
+            recformat = self.data.columns._recformats[idx]
+            if isinstance(recformat, _FormatP):
+                coldata = self.data.field(idx)
+                for c in coldata:
+                    if (not isinstance(c, chararray.chararray) and
+                            c.itemsize > 1 and c.dtype.str[0] in swap_types):
+                        to_swap.append(c)
 
-            nbytes = self.data._gap
+        for arr in reversed(to_swap):
+            arr.byteswap(True)
 
-            for idx in range(self.data._nfields):
-                if isinstance(self.data.columns._recformats[idx], _FormatP):
-                    field = self.data.field(idx)
-                    for row in field:
-                        if len(row) > 0:
-                            nbytes += row.nbytes
-                            if not fileobj.simulateonly:
-                                fileobj.writearray(row)
-
-            self.data._heapsize = nbytes - self.data._gap
-        finally:
-            for obj in swapped:
-                obj.byteswap(True)
-
-        return nbytes
-
-    def __populate_table_keywords(self):
-        """Populate the new table definition keywords from the header."""
-
-        cols = self.columns
-
-        for idx in range(len(cols)):
-            for attr, keyword in zip(KEYWORD_ATTRIBUTES, KEYWORD_NAMES):
-                val = getattr(cols, attr + 's')[idx]
-                if val:
-                    keyword = keyword + str(idx + 1)
-                    if attr == 'format':
-                        val = cols._recformats[idx]
-                        if isinstance(val, (_FormatX, _FormatP)):
-                            val = val.tform
-                        else:
-                            # There are some cases where the original TFORM and
-                            # the one generated from the recformat can have the
-                            # same meaning but different string representation;
-                            # make sure to use the original representation in
-                            # this case
-                            orig_val = cols.formats[idx]
-                            val = _convert_format(val, reverse=True)
-                            if _parse_tformat(orig_val) == _parse_tformat(val):
-                                val = orig_val
-                    self._header[keyword] = val
+        return to_swap
 
     _tdump_file_format = textwrap.dedent("""
 
@@ -767,9 +814,9 @@ class BinTableHDU(_TableBaseHDU):
           .. note::
 
               This format does *not* support variable length arrays using the
-              ('Q' format) due difficult to overcome ambiguities. What
-              this means is that this file format cannot support VLA columns
-              in tables stored in files that are over 2 GB in size.
+              ('Q' format) due to difficult to overcome ambiguities. What this
+              means is that this file format cannot support VLA columns in
+              tables stored in files that are over 2 GB in size.
 
           For column data representing a bit field ('X' format), each bit
           value in the field is output right-justified in a 21-character field
@@ -825,18 +872,17 @@ class BinTableHDU(_TableBaseHDU):
         plain text (ASCII) files.
         """
 
-        # TODO: This is looking pretty long and complicated--might be a few
-        # places we can break this up into smaller functions
-
         # check if the output files already exist
         exist = []
         files = [datafile, cdfile, hfile]
 
         for f in files:
-            if isinstance(f, basestring):
+            if isinstance(f, string_types):
                 if os.path.exists(f) and os.path.getsize(f) != 0:
                     if clobber:
-                        warnings.warn("Overwriting existing file '%s'." % f, AstropyUserWarning)
+                        warnings.warn("Overwriting existing file '%s'." % f,
+                                      AstropyUserWarning)
+                        os.remove(f)
                     else:
                         exist.append(f)
 
@@ -857,7 +903,7 @@ class BinTableHDU(_TableBaseHDU):
 
     dump.__doc__ += _tdump_file_format.replace('\n', '\n        ')
 
-    @deprecated('3.1', alternative=':meth:`dump`')
+    @deprecated('0.1', alternative=':meth:`dump`')
     def tdump(self, datafile=None, cdfile=None, hfile=None, clobber=False):
         self.dump(datafile, cdfile, hfile, clobber)
 
@@ -901,7 +947,7 @@ class BinTableHDU(_TableBaseHDU):
             When the cdfile and hfile are missing, use this Header object in
             the creation of the new table and HDU.  Otherwise this Header
             supercedes the keywords from hfile, which is only used to update
-            values not present in this Header, unless replace=True in which
+            values not present in this Header, unless ``replace=True`` in which
             this Header's values are completely replaced with the values from
             hfile.
 
@@ -943,7 +989,7 @@ class BinTableHDU(_TableBaseHDU):
     # Have to create a classmethod from this here instead of as a decorator;
     # otherwise we can't update __doc__
 
-    @deprecated('3.1', alternative=':meth:`load`')
+    @deprecated('0.1', alternative=':meth:`load`')
     @classmethod
     def tcreate(cls, datafile, cdfile=None, hfile=None, replace=False,
                 header=None):
@@ -961,7 +1007,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'w')
             close_file = True
 
@@ -1007,9 +1053,17 @@ class BinTableHDU(_TableBaseHDU):
                     # The column data is a single element
                     dtype = self.data.dtype.fields[column.name][0]
                     array_format = dtype.char
+                    if array_format == 'V':
+                        array_format = dtype.base.char
                     if array_format == 'S':
                         array_format += str(dtype.itemsize)
-                    line.append(format_value(row[column.name], array_format))
+
+                    if dtype.char == 'V':
+                        for value in row[column.name].flat:
+                            line.append(format_value(value, array_format))
+                    else:
+                        line.append(format_value(row[column.name],
+                                    array_format))
             linewriter.writerow(line)
         if close_file:
             fileobj.close()
@@ -1022,7 +1076,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'w')
             close_file = True
 
@@ -1047,7 +1101,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'r')
             close_file = True
 
@@ -1113,8 +1167,8 @@ class BinTableHDU(_TableBaseHDU):
         # can convert from this format to an actual FITS file on disk without
         # needing enough physical memory to hold the entire thing at once;
         # new_table() could use a similar feature.
-        hdu = new_table(np.recarray(shape=1, dtype=dtype), nrows=nrows,
-                        fill=True)
+        hdu = BinTableHDU.from_columns(np.recarray(shape=1, dtype=dtype),
+                                       nrows=nrows, fill=True)
         data = hdu.data
         for idx, length in enumerate(vla_lengths):
             if length is not None:
@@ -1129,6 +1183,19 @@ class BinTableHDU(_TableBaseHDU):
                 data.columns._recformats[idx] = recformats[idx]
                 data._convert[idx] = _makep(arr, arr, recformats[idx])
 
+        def format_value(col, val):
+            # Special formatting for a couple particular data types
+            if recformats[col] == FITS2NUMPY['L']:
+                return bool(int(val))
+            elif recformats[col] == FITS2NUMPY['M']:
+                # For some reason, in arrays/fields where numpy expects a
+                # complex it's not happy to take a string representation
+                # (though it's happy to do that in other contexts), so we have
+                # to convert the string representation for it:
+                return complex(val)
+            else:
+                return val
+
         # Jump back to the start of the data and create a new line reader
         fileobj.seek(initialpos)
         linereader = csv.reader(fileobj, dialect=FITSTableDumpDialect)
@@ -1139,23 +1206,25 @@ class BinTableHDU(_TableBaseHDU):
                 if line[idx] == 'VLA_Length=':
                     vla_len = vla_lengths[col]
                     idx += 2
+                    slice_ = slice(idx, idx + vla_len)
                     data[row][col][:] = line[idx:idx + vla_len]
                     idx += vla_len
+                elif dtype[col].shape:
+                    # This is an array column
+                    array_size = int(np.multiply.reduce(dtype[col].shape))
+                    slice_ = slice(idx, idx + array_size)
+                    idx += array_size
                 else:
-                    # TODO: This won't work for complex-valued types; fix this
-                    # Kind of silly special handling for bools
-                    val = line[idx]
-                    if recformats[col] == FITS2NUMPY['L']:
-                        val = bool(int(val))
-                    elif recformats[col] == FITS2NUMPY['M']:
-                        # For some reason, in arrays/fields where numpy expects
-                        # a complex it's not happy to take a string
-                        # representation (though it's happy to do that in other
-                        # contexts), so we have to convert the string
-                        # representation for it:
-                        val = complex(val)
-                    data[row][col] = val
+                    slice_ = None
+
+                if slice_ is None:
+                    # This is a scalar row element
+                    data[row][col] = format_value(col, line[idx])
                     idx += 1
+                else:
+                    data[row][col].flat[:] = [format_value(col, val)
+                                              for val in line[slice_]]
+
                 col += 1
 
         if close_file:
@@ -1172,7 +1241,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'r')
             close_file = True
 
@@ -1197,12 +1266,10 @@ class BinTableHDU(_TableBaseHDU):
         return ColDefs(columns)
 
 
-@deprecated('3.2',
-            alternative=':meth:`FITS_rec.from_columns` to create a new '
-                        ':class:`FITS_rec` data object from the input '
-                        'columns to pass into the constructor for '
-                        ':class:`BinTableHDU` or :class:`TableHDU`',
-            pending=True)
+@deprecated('0.4',
+            alternative=':meth:`BinTableHDU.from_columns` for new BINARY '
+                        'tables or :meth:`TableHDU.from_columns` for new '
+                        'ASCII tables')
 def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
     """
     Create a new table from the input column definitions.
@@ -1212,34 +1279,34 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
     separate arrays they must be combined into a single contiguous array.
 
     If the column data is already in a single contiguous array (such as an
-    existing record array) it may be better to create a BinTableHDU instance
+    existing record array) it may be better to create a `BinTableHDU` instance
     directly.  See the Astropy documentation for more details.
 
     Parameters
     ----------
-    input : sequence of Column or ColDefs objects
-        The data to create a table from.
+    input : sequence of `Column` or a `ColDefs`
+        The data to create a table from
 
-    header : Header instance
-        Header to be used to populate the non-required keywords.
+    header : `Header` instance
+        Header to be used to populate the non-required keywords
 
     nrows : int
-        Number of rows in the new table.
+        Number of rows in the new table
 
     fill : bool
         If `True`, will fill all cells with zeros or blanks.  If
         `False`, copy the data from input, undefined cells will still
         be filled with zeros/blanks.
 
-    tbtype : str or class
-        Table type to be created (BinTableHDU or TableHDU) or the class
-        name as a string.  Currently only BinTableHDU and TableHDU (ASCII
+    tbtype : str or type
+        Table type to be created (`BinTableHDU` or `TableHDU`) or the class
+        name as a string.  Currently only `BinTableHDU` and `TableHDU` (ASCII
         tables) are supported.
     """
 
     # tbtype defaults to classes now, but in all prior version of PyFITS it was
     # a string, so we still support that use case as well
-    if not isinstance(tbtype, basestring):
+    if not isinstance(tbtype, string_types):
         cls = tbtype
         tbtype = cls.__name__
     else:
@@ -1249,25 +1316,12 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
         # hope is to deprecate this function anyways so there's not much point
         # in trying to make it more "generic".
         if tbtype.lower() == 'tablehdu':
-            tbtype = 'TableHDU'
             cls = TableHDU
         elif tbtype.lower() == 'bintablehdu':
-            tbtype = 'BinTableHDU'
             cls = BinTableHDU
         else:
             raise ValueError("tbtype must be one of 'TableHDU' or "
                              "'BinTableHDU'")
 
-    if isinstance(input, FITS_rec):  # input is a FITS_rec
-        # Create a new ColDefs object from the input FITS_rec's ColDefs
-        # object and assign it to the ColDefs attribute of the new hdu.
-        columns = ColDefs(input._coldefs, tbtype)
-    else:  # input is a list of Columns or possibly a recarray
-        # Create a new ColDefs object from the input list of Columns and
-        # assign it to the ColDefs attribute of the new hdu.
-        columns = ColDefs(input, tbtype)
-
-    data = FITS_rec.from_columns(columns, nrows=nrows, fill=fill)
-
     # construct a table HDU of the requested type
-    return cls(header=header, data=data)
+    return cls.from_columns(input, header=header, nrows=nrows, fill=fill)
