@@ -1,21 +1,30 @@
 # Licensed under a 3-clause BSD style license - see PYFITS.rst
+from __future__ import division, with_statement
 
 import gzip
 import io
 import mmap
 import os
+import shutil
 import warnings
 import zipfile
 
 import numpy as np
 
+try:
+    import StringIO
+    HAVE_STRINGIO = True
+except ImportError:
+    HAVE_STRINGIO = False
+
 from ....io import fits
 from ....tests.helper import pytest, raises, catch_warnings
-
+from ....utils.exceptions import AstropyDeprecationWarning
 from . import FitsTestCase
 from .util import ignore_warnings
 from ..convenience import _getext
-from ..file import _File
+from ..diff import FITSDiff
+from ..file import _File, GZIP_MAGIC
 
 
 class TestCore(FitsTestCase):
@@ -47,7 +56,7 @@ class TestCore(FitsTestCase):
 
         c = fits.Column(name='foo', format='i2', bscale=1, bzero=32768,
                         array=n)
-        t = fits.new_table([c])
+        t = fits.BinTableHDU.from_columns([c])
 
         l.append(p)
         l.append(t)
@@ -70,24 +79,30 @@ class TestCore(FitsTestCase):
         table = hdulist[1]
         assert table.data.dtype.names == ('c1', 'c2', 'c3', 'c4')
         assert table.columns.names == ['c1', 'c2', 'c3', 'c4']
-        table.columns.del_col('c1')
+        table.columns.del_col(str('c1'))
         assert table.data.dtype.names == ('c2', 'c3', 'c4')
         assert table.columns.names == ['c2', 'c3', 'c4']
 
-        table.columns.del_col('c3')
+        table.columns.del_col(str('c3'))
         assert table.data.dtype.names == ('c2', 'c4')
         assert table.columns.names == ['c2', 'c4']
 
-        table.columns.add_col(fits.Column('foo', '3J'))
+        table.columns.add_col(fits.Column(str('foo'), str('3J')))
         assert table.data.dtype.names == ('c2', 'c4', 'foo')
         assert table.columns.names == ['c2', 'c4', 'foo']
 
         hdulist.writeto(self.temp('test.fits'), clobber=True)
-        with fits.open(self.temp('test.fits')) as hdulist:
-            table = hdulist[1]
-            assert table.data.dtype.names == ('c2', 'c4', 'foo')
-            assert table.columns.names == ['c2', 'c4', 'foo']
+        with ignore_warnings():
+            # TODO: The warning raised by this test is actually indication of a
+            # bug and should *not* be ignored. But as it is a known issue we
+            # hide it for now.  See
+            # https://github.com/spacetelescope/PyFITS/issues/44
+            with fits.open(self.temp('test.fits')) as hdulist:
+                table = hdulist[1]
+                assert table.data.dtype.names == ('c2', 'c4', 'foo')
+                assert table.columns.names == ['c2', 'c4', 'foo']
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_update_header_card(self):
         """A very basic test for the Header.update method--I'd like to add a
         few more cases to this at some point.
@@ -114,6 +129,7 @@ class TestCore(FitsTestCase):
         header.update('BITPIX', 16, 'foobarbaz', savecomment=True)
         assert header.ascard['BITPIX'].comment == comment
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_set_card_value(self):
         """Similar to test_update_header_card(), but tests the the
         `header['FOO'] = 'bar'` method of updating card values.
@@ -140,6 +156,7 @@ class TestCore(FitsTestCase):
         assert hdulist_i[1].data.dtype == np.uint16
         assert np.all(hdulist_f[1].data == hdulist_i[1].data)
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_fix_missing_card_append(self):
         hdu = fits.ImageHDU()
         errs = hdu.req_cards('TESTKW', None, None, 'foo', 'silentfix', [])
@@ -160,7 +177,7 @@ class TestCore(FitsTestCase):
         errs = hdu.req_cards('TESTKW', None, lambda v: v == 'bar', 'bar',
                              'exception', [])
         assert len(errs) == 1
-        assert errs[0] == "'TESTKW' card has invalid value 'foo'."
+        assert errs[0][1] == "'TESTKW' card has invalid value 'foo'."
 
         # See if fixing will work
         hdu.req_cards('TESTKW', None, lambda v: v == 'bar', 'bar', 'silentfix',
@@ -202,7 +219,7 @@ class TestCore(FitsTestCase):
                 hdu.verify('ignore')
             except Exception as e:
                 self.fail('An exception occurred when the verification error '
-                          'should have been ignored: %s' % e)
+                          'should have been ignored: %s' % exc)
         # Make sure the error wasn't fixed either, silently or otherwise
         assert 'NAXIS' not in hdu.header
 
@@ -211,6 +228,76 @@ class TestCore(FitsTestCase):
         hdu = fits.ImageHDU()
         hdu.verify('foobarbaz')
 
+    def test_combined_verify_options(self):
+        """
+        Test verify options like fix+ignore.
+        """
+
+        def make_invalid_hdu():
+            hdu = fits.ImageHDU()
+            # Add one keyword to the header that contains a fixable defect, and one
+            # with an unfixable defect
+            c1 = fits.Card.fromstring("test    = '    test'")
+            c2 = fits.Card.fromstring("P.I.    = '  Hubble'")
+            hdu.header.append(c1)
+            hdu.header.append(c2)
+            return hdu
+
+        # silentfix+ignore should be completely silent
+        hdu = make_invalid_hdu()
+        with catch_warnings():
+            warnings.simplefilter('error')
+            try:
+                hdu.verify('silentfix+ignore')
+            except Exception as exc:
+                self.fail('An exception occurred when the verification error '
+                          'should have been ignored: %s' % exc)
+
+        # silentfix+warn should be quiet about the fixed HDU and only warn
+        # about the unfixable one
+        hdu = make_invalid_hdu()
+        with catch_warnings() as w:
+            hdu.verify('silentfix+warn')
+            assert len(w) == 4
+            assert 'Illegal keyword name' in str(w[2].message)
+
+        # silentfix+exception should only mention the unfixable error in the
+        # exception
+        hdu = make_invalid_hdu()
+        try:
+            hdu.verify('silentfix+exception')
+        except fits.VerifyError as exc:
+            assert 'Illegal keyword name' in str(exc)
+            assert 'not upper case' not in str(exc)
+        else:
+            self.fail('An exception should have been raised.')
+
+        # fix+ignore is not too useful, but it should warn about the fixed
+        # problems while saying nothing about the unfixable problems
+        hdu = make_invalid_hdu()
+        with catch_warnings() as w:
+            hdu.verify('fix+ignore')
+            assert len(w) == 4
+            assert 'not upper case' in str(w[2].message)
+
+        # fix+warn
+        hdu = make_invalid_hdu()
+        with catch_warnings() as w:
+            hdu.verify('fix+warn')
+            assert len(w) == 6
+            assert 'not upper case' in str(w[2].message)
+            assert 'Illegal keyword name' in str(w[4].message)
+
+        # fix+exception
+        hdu = make_invalid_hdu()
+        try:
+            hdu.verify('fix+exception')
+        except fits.VerifyError as exc:
+            assert 'Illegal keyword name' in str(exc)
+            assert 'not upper case' in str(exc)
+        else:
+            self.fail('An exception should have been raised.')
+
     def test_getext(self):
         """
         Test the various different ways of specifying an extension header in
@@ -260,8 +347,8 @@ class TestCore(FitsTestCase):
 
     def test_extension_name_case_sensitive(self):
         """
-        Tests that setting fits.EXTENSION_NAME_CASE_SENSITIVE at runtime
-        works.
+        Tests that setting fits.conf.extension_name_case_sensitive at
+        runtime works.
         """
 
         if 'PYFITS_EXTENSION_NAME_CASE_SENSITIVE' in os.environ:
@@ -272,14 +359,11 @@ class TestCore(FitsTestCase):
         assert hdu.name == 'SCI'
         assert hdu.header['EXTNAME'] == 'SCI'
 
-        try:
-            fits.EXTENSION_NAME_CASE_SENSITIVE.set(True)
+        with fits.conf.set_temp('extension_name_case_sensitive', True):
             hdu = fits.ImageHDU()
             hdu.name = 'sCi'
             assert hdu.name == 'sCi'
             assert hdu.header['EXTNAME'] == 'sCi'
-        finally:
-            fits.EXTENSION_NAME_CASE_SENSITIVE.set(False)
 
         hdu.name = 'sCi'
         assert hdu.name == 'SCI'
@@ -320,7 +404,7 @@ class TestCore(FitsTestCase):
 
     def test_nonstandard_hdu(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/157
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/157
 
         Tests that "Nonstandard" HDUs with SIMPLE = F are read and written
         without prepending a superfluous and unwanted standard primary HDU.
@@ -405,6 +489,39 @@ class TestCore(FitsTestCase):
         pytest.raises(TypeError, setattr, h1, 'ver', 'FOO')
         pytest.raises(TypeError, setattr, h1, 'level', 'BAR')
 
+    def test_consecutive_writeto(self):
+        """
+        Regression test for an issue where calling writeto twice on the same
+        HDUList could write a corrupted file.
+
+        https://github.com/spacetelescope/PyFITS/issues/40 is actually a
+        particular instance of this problem, though isn't unique to sys.stdout.
+        """
+
+        with fits.open(self.data('test0.fits')) as hdul1:
+            # Add a bunch of header keywords so that the data will be forced to
+            # new offsets within the file:
+            for idx in range(40):
+                hdul1[1].header['TEST%d' % idx] = 'test'
+
+            hdul1.writeto(self.temp('test1.fits'))
+            hdul1.writeto(self.temp('test2.fits'))
+
+            # Open a second handle to the original file and compare it to hdul1
+            # (We only compare part of the one header that was modified)
+            # Compare also with the second writeto output
+            with fits.open(self.data('test0.fits')) as hdul2:
+                with fits.open(self.temp('test2.fits')) as hdul3:
+                    for hdul in (hdul1, hdul3):
+                        for idx, hdus in enumerate(zip(hdul1, hdul)):
+                            hdu1, hdu2 = hdus
+                            if idx != 1:
+                                assert hdu1.header == hdu2.header
+                            else:
+                                assert (hdu1.header ==
+                                        hdu2.header[:len(hdu1.header)])
+                            assert np.all(hdu1.data == hdu2.data)
+
 
 class TestConvenienceFunctions(FitsTestCase):
     def test_writeto(self):
@@ -421,9 +538,10 @@ class TestConvenienceFunctions(FitsTestCase):
         assert len(hdul) == 1
         assert (data == hdul[0].data).all()
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_writeto_2(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/107
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/107
 
         Test of `writeto()` with a trivial header containing a single keyword.
         """
@@ -453,7 +571,7 @@ class TestFileFunctions(FitsTestCase):
 
         try:
             fits.open(self.temp('foobar.fits'))
-        except IOError, e:
+        except IOError as e:
             assert 'File does not exist' in str(e)
         except:
             raise
@@ -526,6 +644,22 @@ class TestFileFunctions(FitsTestCase):
         pytest.raises(IOError, fits.open, zf, 'update')
         pytest.raises(IOError, fits.open, zf, 'append')
 
+    def test_read_open_astropy_gzip_file(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/2774
+
+        This tests reading from a ``GzipFile`` object from Astropy's
+        compatibility copy of the ``gzip`` module.
+        """
+
+        from ....utils.compat import gzip
+
+        gf = gzip.GzipFile(self._make_gzip_file())
+        try:
+            assert len(fits.open(gf)) == 5
+        finally:
+            gf.close()
+
     @raises(IOError)
     def test_open_multiple_member_zipfile(self):
         """
@@ -564,7 +698,7 @@ class TestFileFunctions(FitsTestCase):
             gf.close()
 
     def test_open_gzip_file_for_writing(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/195."""
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/195."""
 
         gf = self._make_gzip_file()
         with fits.open(gf, mode='update') as h:
@@ -574,6 +708,23 @@ class TestFileFunctions(FitsTestCase):
             # normal writes should work too...
             assert h[0].header['EXPFLAG'] == 'ABNORMAL'
 
+    def test_write_read_gzip_file(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/2794
+
+        Ensure files written through gzip are readable.
+        """
+
+        data = np.arange(100)
+        hdu = fits.PrimaryHDU(data=data)
+        hdu.writeto(self.temp('test.fits.gz'))
+
+        with open(self.temp('test.fits.gz'), 'rb') as f:
+            assert f.read(3) == GZIP_MAGIC
+
+        with fits.open(self.temp('test.fits.gz')) as hdul:
+            assert np.all(hdul[0].data == data)
+
     def test_read_file_like_object(self):
         """Test reading a FITS file from a file-like object."""
 
@@ -586,7 +737,7 @@ class TestFileFunctions(FitsTestCase):
 
     def test_updated_file_permissions(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/79
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/79
 
         Tests that when a FITS file is modified in update mode, the file
         permissions are preserved.
@@ -606,6 +757,52 @@ class TestFileFunctions(FitsTestCase):
 
         assert old_mode == os.stat(filename).st_mode
 
+    def test_fileobj_mode_guessing(self):
+        """Tests whether a file opened without a specified pyfits mode
+        ('readonly', etc.) is opened in a mode appropriate for the given file
+        object.
+        """
+
+        self.copy_file('test0.fits')
+
+        # Opening in text mode should outright fail
+        for mode in ('r', 'w', 'a'):
+            with open(self.temp('test0.fits'), mode) as f:
+                pytest.raises(ValueError, fits.HDUList.fromfile, f)
+
+        # Need to re-copy the file since opening it in 'w' mode blew it away
+        self.copy_file('test0.fits')
+
+        with open(self.temp('test0.fits'), 'rb') as f:
+            with fits.HDUList.fromfile(f) as h:
+                assert h.fileinfo(0)['filemode'] == 'readonly'
+
+        for mode in ('wb', 'ab'):
+            with open(self.temp('test0.fits'), mode) as f:
+                with fits.HDUList.fromfile(f) as h:
+                    # Basically opening empty files for output streaming
+                    assert len(h) == 0
+
+        # Need to re-copy the file since opening it in 'w' mode blew it away
+        self.copy_file('test0.fits')
+
+        with open(self.temp('test0.fits'), 'wb+') as f:
+            with fits.HDUList.fromfile(f) as h:
+                # wb+ still causes an existing file to be overwritten so there
+                # are no HDUs
+                assert len(h) == 0
+
+        # Need to re-copy the file since opening it in 'w' mode blew it away
+        self.copy_file('test0.fits')
+
+        with open(self.temp('test0.fits'), 'rb+') as f:
+            with fits.HDUList.fromfile(f) as h:
+                assert h.fileinfo(0)['filemode'] == 'update'
+
+        with open(self.temp('test0.fits'), 'ab+') as f:
+            with fits.HDUList.fromfile(f) as h:
+                assert h.fileinfo(0)['filemode'] == 'append'
+
     def test_mmap_unwriteable(self):
         """Regression test for https://github.com/astropy/astropy/issues/968
 
@@ -625,7 +822,7 @@ class TestFileFunctions(FitsTestCase):
 
         try:
             self.copy_file('test0.fits')
-            with warnings.catch_warnings(record=True) as w:
+            with catch_warnings() as w:
                 with fits.open(self.temp('test0.fits'), mode='update',
                                memmap=True) as h:
                     h[1].data[0, 0] = 999
@@ -640,6 +837,126 @@ class TestFileFunctions(FitsTestCase):
             mmap.mmap = old_mmap
             _File._mmap_available = None
 
+    def test_mmap_unwriteable(self):
+        """Regression test for
+        https://github.com/astropy/astropy/issues/968
+
+        Temporarily patches mmap.mmap to exhibit platform-specific bad
+        behavior.
+        """
+
+        class MockMmap(mmap.mmap):
+            def flush(self):
+                raise mmap.error('flush is broken on this platform')
+
+        old_mmap = mmap.mmap
+        mmap.mmap = MockMmap
+
+        # Force the mmap test to be rerun
+        _File._mmap_available = None
+
+        try:
+            # TODO: Use self.copy_file once it's merged into Astropy
+            shutil.copy(self.data('test0.fits'), self.temp('test0.fits'))
+            with catch_warnings() as w:
+                with fits.open(self.temp('test0.fits'), mode='update',
+                               memmap=True) as h:
+                    h[1].data[0, 0] = 999
+
+                assert len(w) == 1
+                assert 'mmap.flush is unavailable' in str(w[0].message)
+
+            # Double check that writing without mmap still worked
+            with fits.open(self.temp('test0.fits')) as h:
+                assert h[1].data[0, 0] == 999
+        finally:
+            mmap.mmap = old_mmap
+            _File._mmap_available = None
+
+    def test_uncloseable_file(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/2356
+
+        Demonstrates that FITS files can still be read from "file-like" objects
+        that don't have an obvious "open" or "closed" state.
+        """
+
+        class MyFileLike(object):
+            def __init__(self, foobar):
+                self._foobar = foobar
+
+            def read(self, n):
+                return self._foobar.read(n)
+
+            def seek(self, offset, whence=os.SEEK_SET):
+                self._foobar.seek(offset, whence)
+
+            def tell(self):
+                return self._foobar.tell()
+
+
+        with open(self.data('test0.fits'), 'rb') as f:
+            fileobj = MyFileLike(f)
+
+            with fits.open(fileobj) as hdul1:
+                with fits.open(self.data('test0.fits')) as hdul2:
+                    assert hdul1.info(output=False) == hdul2.info(output=False)
+                    for hdu1, hdu2 in zip(hdul1, hdul2):
+                        assert hdu1.header == hdu2.header
+                        if hdu1.data is not None and hdu2.data is not None:
+                            assert np.all(hdu1.data == hdu2.data)
+
+    @pytest.mark.skipif('not HAVE_STRINGIO')
+    def test_write_stringio(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/2463
+
+        Only test against `StringIO.StringIO` on Python versions that have it.
+        Note: `io.StringIO` is not supported for this purpose as it does not
+        accept a bytes stream.
+        """
+
+        self._test_write_string_bytes_io(StringIO.StringIO())
+
+    @pytest.mark.skipif('not HAVE_STRINGIO')
+    def test_write_stringio_discontiguous(self):
+        """
+        Regression test related to
+        https://github.com/astropy/astropy/issues/2794#issuecomment-55441539
+
+        Demonstrates that writing an HDU containing a discontiguous Numpy array
+        should work properly.
+        """
+
+        data = np.arange(100)[::3]
+        hdu = fits.PrimaryHDU(data=data)
+        fileobj = StringIO.StringIO()
+        hdu.writeto(fileobj)
+
+        fileobj.seek(0)
+
+        with fits.open(fileobj) as h:
+            assert np.all(h[0].data == data)
+
+    def test_write_bytesio(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/2463
+
+        Test againt `io.BytesIO`.  `io.StringIO` is not supported.
+        """
+
+        self._test_write_string_bytes_io(io.BytesIO())
+
+    def _test_write_string_bytes_io(self, fileobj):
+        """
+        Implemented for both test_write_stringio and test_write_bytesio.
+        """
+
+        with fits.open(self.data('test0.fits')) as hdul:
+            hdul.writeto(fileobj)
+            hdul2 = fits.HDUList.fromstring(fileobj.getvalue())
+            assert FITSDiff(hdul, hdul2).identical
+
     def _make_gzip_file(self, filename='test0.fits.gz'):
         gzfile = self.temp(filename)
         with open(self.data('test0.fits'), 'rb') as f:
@@ -737,3 +1054,8 @@ class TestStreamingFunctions(FitsTestCase):
         hd['NAXIS2'] = 5
         hd['EXTEND'] = True
         return fits.StreamingHDU(fileobj, hd)
+
+    def test_blank_ignore(self):
+
+        with fits.open(self.data('blank.fits'), ignore_blank=True) as f:
+            assert f[0].data.flat[0] == 2
