@@ -2,7 +2,7 @@
 from distutils.core import Extension
 from os.path import join
 
-from astropy import setup_helpers
+from astropy_helpers import setup_helpers
 
 
 def get_extensions(build_type='release'):
