@@ -22,17 +22,16 @@ from . import exceptions
 from . import tree
 from ...utils.xml import iterparser
 from ...utils import data
-from ...config import ConfigurationItem
+from ...config import ConfigAlias
 
 
 __all__ = ['parse', 'parse_single_table', 'from_table', 'writeto', 'validate',
            'reset_vo_warnings']
 
 
-PEDANTIC = ConfigurationItem(
-    'pedantic',
-    False,
-    'When True, treat fixable violations of the VOTable spec as exceptions.')
+PEDANTIC = ConfigAlias(
+    '0.4', 'PEDANTIC', 'pedantic',
+    'astropy.io.votable.table', 'astropy.io.votable')
 
 
 def parse(source, columns=None, invalid='exception', pedantic=None,
@@ -41,7 +40,7 @@ def parse(source, columns=None, invalid='exception', pedantic=None,
           _debug_python_based_parser=False):
     """
     Parses a VOTABLE_ xml file (or file-like object), and returns a
-    `~astropy.io.votable.tree.VOTable` object.
+    `~astropy.io.votable.tree.VOTableFile` object.
 
     Parameters
     ----------
@@ -66,7 +65,7 @@ def parse(source, columns=None, invalid='exception', pedantic=None,
         the standard Python mechanisms.  See the `warnings`
         module in the Python standard library for more information.
         When not provided, uses the configuration setting
-        `astropy.io.votable.pedantic`, which defaults to False.
+        ``astropy.io.votable.pedantic``, which defaults to False.
 
     chunk_size : int, optional
         The number of rows to read before converting to an array.
@@ -77,11 +76,11 @@ def parse(source, columns=None, invalid='exception', pedantic=None,
         The number of table in the file to read in.  If `None`, all
         tables will be read.  If a number, 0 refers to the first table
         in the file, and only that numbered table will be parsed and
-        read in.  Should not be used with `table_id`.
+        read in.  Should not be used with ``table_id``.
 
     table_id : str, optional
         The ID of the table in the file to read in.  Should not be
-        used with `table_number`.
+        used with ``table_number``.
 
     filename : str, optional
         A filename, URL or other identifier to use in error messages.
@@ -95,25 +94,27 @@ def parse(source, columns=None, invalid='exception', pedantic=None,
         string, must be the name of a unit formatter. The built-in
         formats include ``generic``, ``fits``, ``cds``, and
         ``vounit``.  A custom formatter may be provided by passing a
-        `astropy.units.format.Base` instance.  If `None` (default),
+        `~astropy.units.UnitBase` instance.  If `None` (default),
         the unit format to use will be the one specified by the
-        VOTable specification (which is `cds` up to version 1.2 of
-        VOTable, and (probably) `vounit` in future versions of the
+        VOTable specification (which is ``cds`` up to version 1.2 of
+        VOTable, and (probably) ``vounit`` in future versions of the
         spec).
 
     Returns
     -------
-    votable : `astropy.io.votable.tree.VOTableFile` object
+    votable : `~astropy.io.votable.tree.VOTableFile` object
 
     See also
     --------
     astropy.io.votable.exceptions : The exceptions this function may raise.
     """
+    from . import conf
+
     invalid = invalid.lower()
     assert invalid in ('exception', 'mask')
 
     if pedantic is None:
-        pedantic = PEDANTIC()
+        pedantic = conf.pedantic
 
     config = {
         'columns'      :      columns,
@@ -144,7 +145,7 @@ def parse_single_table(source, **kwargs):
 
     Returns
     -------
-    votable : `astropy.io.votable.tree.Table` object
+    votable : `~astropy.io.votable.tree.Table` object
     """
     if kwargs.get('table_number') is None:
         kwargs['table_number'] = 0
@@ -154,16 +155,23 @@ def parse_single_table(source, **kwargs):
     return votable.get_first_table()
 
 
-def writeto(table, file):
+def writeto(table, file, tabledata_format=None):
     """
-    Writes a `astropy.io.vo.VOTableFile` to a VOTABLE_ xml file.
+    Writes a `~astropy.io.votable.tree.VOTableFile` to a VOTABLE_ xml file.
 
     Parameters
     ----------
-    table : `astropy.io.vo.VOTableFile` or `astropy.table.Table` instance.
+    table : `~astropy.io.votable.tree.VOTableFile` or `~astropy.table.Table` instance.
 
     file : str or writable file-like object
         Path or file object to write to
+
+    tabledata_format : str, optional
+        Override the format of the table(s) data to write.  Must be
+        one of ``tabledata`` (text representation), ``binary`` or
+        ``binary2``.  By default, use the format that was specified in
+        each ``table`` object as it was created or read in.  See
+        :ref:`votable-serialization`.
     """
     from ...table import Table
     if isinstance(table, Table):
@@ -172,7 +180,8 @@ def writeto(table, file):
         raise TypeError(
             "first argument must be astropy.io.vo.VOTableFile or "
             "astropy.table.Table instance")
-    table.to_xml(file, _debug_python_based_parser=True)
+    table.to_xml(file, tabledata_format=tabledata_format,
+                 _debug_python_based_parser=True)
 
 
 def validate(source, output=None, xmllint=False, filename=None):
@@ -185,14 +194,14 @@ def validate(source, output=None, xmllint=False, filename=None):
         Path to a VOTABLE_ xml file.
 
     output : writable file-like object, optional
-        Where to output the report.  Defaults to `sys.stdout`.
+        Where to output the report.  Defaults to ``sys.stdout``.
         If `None`, the output will be returned as a string.
 
     xmllint : bool, optional
-        When `True`, also send the file to `xmllint` for schema and
-        DTD validation.  Requires that `xmllint` is installed.  The
-        default is `False`.  `source` must be a file on the local
-        filesystem in order for `xmllint` to work.
+        When `True`, also send the file to ``xmllint`` for schema and
+        DTD validation.  Requires that ``xmllint`` is installed.  The
+        default is `False`.  ``source`` must be a file on the local
+        filesystem in order for ``xmllint`` to work.
 
     filename : str, optional
         A filename to use in the error messages.  If not provided, one
@@ -201,7 +210,7 @@ def validate(source, output=None, xmllint=False, filename=None):
     Returns
     -------
     is_valid : bool or str
-        Returns `True` if no warnings were found.  If `output` is
+        Returns `True` if no warnings were found.  If ``output`` is
         `None`, the return value will be a string.
     """
 
@@ -303,21 +312,21 @@ def validate(source, output=None, xmllint=False, filename=None):
 
 def from_table(table, table_id=None):
     """
-    Given an `astropy.table.Table` object, return a
+    Given an `~astropy.table.Table` object, return a
     `~astropy.io.votable.tree.VOTableFile` file structure containing
     just that single table.
 
     Parameters
     ----------
-    table : `astropy.table.Table` instance
+    table : `~astropy.table.Table` instance
 
     table_id : str, optional
         If not `None`, set the given id on the returned
-        `~astropy.tree.Table` instance.
+        `~astropy.io.votable.tree.Table` instance.
 
     Returns
     -------
-    votable : `astropy.io.votable.tree.VOTableFile` instance
+    votable : `~astropy.io.votable.tree.VOTableFile` instance
     """
     return tree.VOTableFile.from_table(table, table_id=table_id)
 
