@@ -7,7 +7,8 @@ It is unlikely users will need to work with these classes directly, unless they
 define their own models.
 """
 
-from __future__ import division
+from __future__ import (absolute_import, unicode_literals, division,
+                        print_function)
 
 import inspect
 import functools
@@ -16,7 +17,8 @@ import numbers
 import numpy as np
 
 from ..utils import isiterable
-
+from ..utils.compat import ignored
+from ..extern import six
 
 __all__ = ['Parameter', 'InputParameterError']
 
@@ -31,24 +33,25 @@ def _tofloat(value):
     if isiterable(value):
         try:
             value = np.array(value, dtype=np.float)
-            shape = value.shape
         except (TypeError, ValueError):
             # catch arrays with strings or user errors like different
             # types of parameters in a parameter set
             raise InputParameterError(
                 "Parameter of {0} could not be converted to "
                 "float".format(type(value)))
+    elif isinstance(value, np.ndarray):
+        # A scalar/dimensionless array
+        value = float(value.item())
+    elif isinstance(value, (numbers.Number, np.number)):
+        value = float(value)
     elif isinstance(value, bool):
         raise InputParameterError(
             "Expected parameter to be of numerical type, not boolean")
-    elif isinstance(value, (numbers.Number, np.number)):
-        value = float(value)
-        shape = ()
     else:
         raise InputParameterError(
             "Don't know how to convert parameter of {0} to "
             "float".format(type(value)))
-    return value, shape
+    return value
 
 
 class Parameter(object):
@@ -64,7 +67,7 @@ class Parameter(object):
     Parameter instances never store the actual value of the parameter
     directly.  Rather, each instance of a model stores its own parameters
     as either hidden attributes or (in the case of
-    `~astropy.modeling.core.ParametricModel`) in an array.  A *bound*
+    `~astropy.modeling.FittableModel`) in an array.  A *bound*
     Parameter simply wraps the value in a Parameter proxy which provides some
     additional information about the parameter such as its constraints.
 
@@ -77,6 +80,8 @@ class Parameter(object):
     ----------
     name : str
         parameter name
+    description : str
+        parameter description
     default : float or array
         default value to use for this parameter
     getter : callable
@@ -104,14 +109,17 @@ class Parameter(object):
     # See the _nextid classmethod
     _nextid = 1
 
-    def __init__(self, name, description='', default=None, getter=None,
+    def __init__(self, name='', description='', default=None, getter=None,
                  setter=None, fixed=False, tied=False, min=None, max=None,
                  model=None):
         super(Parameter, self).__init__()
+
+        if model is not None and not name:
+            raise TypeError('Bound parameters must have a name specified.')
+
         self._name = name
         self.__doc__ = description.strip()
         self._default = default
-        self._attr = '_' + name
 
         self._default_fixed = fixed
         self._default_tied = tied
@@ -137,11 +145,10 @@ class Parameter(object):
             self._setter = None
 
         if model is not None:
-            try:
+            with ignored(AttributeError):
+                # This can only work if the paramter's value has been set by
+                # the model
                 _, self._shape = self._validate_value(model, self.value)
-            except AttributeError:
-                # This can happen if the paramter's value has not been set yet
-                pass
         else:
             # Only Parameters declared as class-level descriptors require
             # and ordering ID
@@ -153,35 +160,30 @@ class Parameter(object):
             return self
 
         return self.__class__(self._name, default=self._default,
-                              getter=self._getter,
-                              setter=self._setter, model=obj)
+                              getter=self._getter, setter=self._setter,
+                              fixed=self._default_fixed,
+                              tied=self._default_tied, min=self._default_min,
+                              max=self._default_max, model=obj)
 
     def __set__(self, obj, value):
         value, shape = self._validate_value(obj, value)
-        # Compare the shape against the previous value's shape, if it exists
-        if hasattr(obj, self._attr):
-            current_shape = getattr(obj, self.name).shape
-            if shape != current_shape:
-                raise InputParameterError(
-                    "Input value for parameter {0!r} does not have the "
-                    "required shape {1}".format(self.name, current_shape))
 
         if self._setter is not None:
             setter = self._create_value_wrapper(self._setter, obj)
             value = setter(value)
 
-        setattr(obj, self._attr, value)
+        self._set_model_value(obj, value)
 
     def __len__(self):
         if self._model is None:
             raise TypeError('Parameter definitions do not have a length.')
-        return self._model.param_dim
+        return len(self._model)
 
     def __getitem__(self, key):
         value = self.value
-        if self._model.param_dim == 1:
+        if len(self._model) == 1:
             # Wrap the value in a list so that getitem can work for sensible
-            # indcies like [0] and [-1]
+            # indices like [0] and [-1]
             value = [value]
         return value[key]
 
@@ -189,35 +191,33 @@ class Parameter(object):
         # Get the existing value and check whether it even makes sense to
         # apply this index
         oldvalue = self.value
-        param_dim = self._model.param_dim
+        n_models = len(self._model)
 
-        if param_dim == 1:
-            # Convert the single-dimension value to a list to allow some slices
-            # that would be compatible with a length-1 array like [:] and [0:]
-            oldvalue = [oldvalue]
+        #if n_models == 1:
+        #    # Convert the single-dimension value to a list to allow some slices
+        #    # that would be compatible with a length-1 array like [:] and [0:]
+        #    oldvalue = [oldvalue]
 
         if isinstance(key, slice):
             if len(oldvalue[key]) == 0:
                 raise InputParameterError(
                     "Slice assignment outside the parameter dimensions for "
-                    "{0!r}".format(self.name))
+                    "'{0}'".format(self.name))
             for idx, val in zip(range(*key.indices(len(self))), value):
                 self.__setitem__(idx, val)
         else:
             try:
                 oldvalue[key] = value
-                if param_dim == 1:
-                    self.value = value
             except IndexError:
                 raise InputParameterError(
                     "Input dimension {0} invalid for {1!r} parameter with "
-                    "dimension {2}".format(key, self.name, param_dim))
+                    "dimension {2}".format(key, self.name, n_models))
 
     def __repr__(self):
         if self._model is None:
-            return 'Parameter({0!r})'.format(self._name)
+            return "Parameter('{0}')".format(self._name)
         else:
-            return 'Parameter({0!r}, value={1!r})'.format(
+            return "Parameter('{0}', value={1})".format(
                 self._name, self.value)
 
     @property
@@ -230,40 +230,54 @@ class Parameter(object):
     def default(self):
         """Parameter default value"""
 
-        if self._model is None:
+        if (self._model is None or self._default is None or
+                len(self._model) == 1):
             return self._default
 
-        if self._model.param_dim == 1:
-            return self._default
-        else:
-            return np.repeat(self._default, self._model.param_dim)
+        # Otherwise the model we are providing for has more than one parameter
+        # sets, so ensure that the default is repeated the correct number of
+        # times along the model_set_axis if necessary
+        n_models = len(self._model)
+        model_set_axis = self._model._model_set_axis
+        default = self._default
+        new_shape = (np.shape(default) +
+                     (1,) * (model_set_axis + 1 - np.ndim(default)))
+        default = np.reshape(default, new_shape)
+        # Now roll the new axis into its correct position if necessary
+        default = np.rollaxis(default, -1, model_set_axis)
+        # Finally repeat the last newly-added axis to match n_models
+        default = np.repeat(default, n_models, axis=-1)
+
+        # NOTE: Regardless of what order the last two steps are performed in,
+        # the resulting array will *look* the same, but only if the repeat is
+        # performed last will it result in a *contiguous* array
+
+        return default
 
     @property
     def value(self):
         """The unadorned value proxied by this parameter"""
 
-        if self._model is not None:
-            if not hasattr(self._model, self._attr):
-                if self._default is not None:
-                    value = self.default
-                else:
-                    raise AttributeError(
-                        'Parameter value for {0!r} not set'.format(self._name))
-            else:
-                value = getattr(self._model, self._attr)
-            if self._getter is None:
-                return value
-            else:
-                return self._getter(value)
-        raise AttributeError('Parameter definition does not have a value')
+        if self._model is None:
+            raise AttributeError('Parameter definition does not have a value')
+
+        value = self._get_model_value(self._model)
+
+        if self._getter is None:
+            return value
+        else:
+            return self._getter(value)
 
     @value.setter
-    def value(self, val):
-        if self._model is not None:
-            if self._setter is not None:
-                val = self._setter(val)
-            setattr(self._model, self._attr, val)
-        raise AttributeError('Cannot set a value on a parameter definition')
+    def value(self, value):
+        if self._model is None:
+            raise AttributeError('Cannot set a value on a parameter '
+                                 'definition')
+
+        if self._setter is not None:
+            val = self._setter(value)
+
+        self._set_model_value(self._model, value)
 
     @property
     def shape(self):
@@ -284,8 +298,8 @@ class Parameter(object):
         """
 
         if self._model is not None:
-            fixed = self._model._constraints.setdefault('fixed', {})
-            return fixed.setdefault(self._name, self._default_fixed)
+            fixed = self._model._constraints['fixed']
+            return fixed.get(self._name, self._default_fixed)
         else:
             return self._default_fixed
 
@@ -293,11 +307,9 @@ class Parameter(object):
     def fixed(self, value):
         """Fix a parameter"""
         if self._model is not None:
-            assert isinstance(value, bool), "Fixed can be True or False"
-            fixed = self._model._constraints.setdefault('fixed', {})
-            fixed[self._name] = value
-            self._model._fit_parameters, self._model._fit_param_indices, \
-                = self._model._model_to_fit_params()
+            if not isinstance(value, bool):
+                raise TypeError("Fixed can be True or False")
+            self._model._constraints['fixed'][self._name] = value
         else:
             raise AttributeError("can't set attribute 'fixed' on Parameter "
                                  "definition")
@@ -311,8 +323,8 @@ class Parameter(object):
         """
 
         if self._model is not None:
-            tied = self._model._constraints.setdefault('tied', {})
-            return tied.setdefault(self._name, self._default_tied)
+            tied = self._model._constraints['tied']
+            return tied.get(self._name, self._default_tied)
         else:
             return self._default_tied
 
@@ -321,12 +333,9 @@ class Parameter(object):
         """Tie a parameter"""
 
         if self._model is not None:
-            assert callable(value) or value in (False, None), \
-                    "Tied must be a callable"
-            tied = self._model._constraints.setdefault('tied', {})
-            tied[self._name] = value
-            self._model._fit_parameters, self._model._fit_param_indices, \
-                = self._model._model_to_fit_params()
+            if not six.callable(value) and value not in (False, None):
+                    raise TypeError("Tied must be a callable")
+            self._model._constraints['tied'][self._name] = value
         else:
             raise AttributeError("can't set attribute 'tied' on Parameter "
                                  "definition")
@@ -336,9 +345,9 @@ class Parameter(object):
         """The minimum and maximum values of a parameter as a tuple"""
 
         if self._model is not None:
-            bounds = self._model._constraints.setdefault('bounds', {})
-            return bounds.setdefault(self._name,
-                                     (self._default_min, self._default_max))
+            bounds = self._model._constraints['bounds']
+            default_bounds = (self._default_min, self._default_max)
+            return bounds.get(self._name, default_bounds)
         else:
             return (self._default_min, self._default_max)
 
@@ -349,18 +358,17 @@ class Parameter(object):
         if self._model is not None:
             _min, _max = value
             if _min is not None:
-                assert isinstance(_min, numbers.Number), \
-                        "Min value must be a number"
+                if not isinstance(_min, numbers.Number):
+                        raise TypeError("Min value must be a number")
                 _min = float(_min)
 
             if _max is not None:
-                assert isinstance(_max, numbers.Number), \
-                        "Max value must be a number"
+                if not isinstance(_max, numbers.Number):
+                        raise TypeError("Max value must be a number")
                 _max = float(_max)
 
             bounds = self._model._constraints.setdefault('bounds', {})
-            bounds[self._name] = (_min, _max)
-            self._model._model_to_fit_params()
+            self._model._constraints['bounds'][self._name] = (_min, _max)
         else:
             raise AttributeError("can't set attribute 'bounds' on Parameter "
                                  "definition")
@@ -410,22 +418,67 @@ class Parameter(object):
         cls._nextid += 1
         return nextid
 
+    def _get_model_value(self, model):
+        """
+        This method implements how to retrieve the value of this parameter from
+        the model instance.  See also `Parameter._set_model_value`.
+
+        These methods take an explicit model argument rather than using
+        self._model so that they can be used from unbound `Parameter`
+        instances.
+        """
+
+        if not hasattr(model, '_parameters'):
+            # The _parameters array hasn't been initialized yet; just translate
+            # this to an AttributeError
+            raise AttributeError(self._name)
+
+        # Use the _param_metrics to extract the parameter value from the
+        # _parameters array
+        param_slice, param_shape = model._param_metrics[self._name]
+        value = model._parameters[param_slice]
+        if param_shape:
+            value = value.reshape(param_shape)
+        else:
+            value = value[0]
+        return value
+
+    def _set_model_value(self, model, value):
+        """
+        This method implements how to store the value of a parameter on the
+        model instance.
+
+        Currently there is only one storage mechanism (via the ._parameters
+        array) but other mechanisms may be desireable, in which case really the
+        model class itself should dictate this and *not* `Parameter` itself.
+        """
+
+        # TODO: Maybe handle exception on invalid input shape
+        param_slice, param_shape = model._param_metrics[self._name]
+        param_size = np.prod(param_shape)
+
+        if np.size(value) != param_size:
+            raise InputParameterError(
+                "Input value for parameter {0!r} does not have {1} elements "
+                "as the current value does".format(self._name, param_size))
+
+        model._parameters[param_slice] = np.array(value).ravel()
+
     def _validate_value(self, model, value):
         if model is None:
             return
 
-        param_dim = model.param_dim
-        if param_dim == 1:
+        n_models = len(model)
+        value = _tofloat(value)
+        if n_models == 1:
             # Just validate the value with _tofloat
-            return _tofloat(value)
+            return value, np.shape(value)
         else:
-            try:
-                # Validate each value
-                value, shape = _tofloat(value)
-            except (TypeError, IndexError):
-                raise InputParameterError(
-                    "Expected a multivalued input of dimension {0} "
-                    "for parameter {1!r}".format(param_dim, self.name))
+            shape = np.shape(value)
+            model_axis = model._model_set_axis
+            if model_axis < 0:
+                model_axis = len(shape) + model_axis
+            shape = shape[:model_axis] + shape[model_axis + 1:]
 
             return value, shape
 
@@ -465,7 +518,7 @@ class Parameter(object):
         return self.value + val
 
     def __radd__(self, val):
-        return self.value + val
+        return val + self.value
 
     def __sub__(self, val):
         return self.value - val
@@ -477,11 +530,14 @@ class Parameter(object):
         return self.value * val
 
     def __rmul__(self, val):
-        return self.value * val
+        return val * self.value
 
     def __pow__(self, val):
         return self.value ** val
 
+    def __rpow__(self, val):
+        return val ** self.value
+
     def __div__(self, val):
         return self.value / val
 
@@ -513,7 +569,7 @@ class Parameter(object):
         return (np.asarray(self) >= np.asarray(val)).all()
 
     def __neg__(self):
-        return -self.value * (-1)
+        return -self.value
 
     def __abs__(self):
         return np.abs(self.value)
