@@ -4,15 +4,14 @@
 This module contains predefined polynomial models.
 """
 
-from __future__ import division
+from __future__ import (absolute_import, unicode_literals, division,
+                        print_function)
 
 import collections
 
-from textwrap import dedent
-
 import numpy as np
 
-from .core import ParametricModel, Model, SerialCompositeModel, format_input
+from .core import FittableModel, Model, SerialCompositeModel, format_input
 from .functional_models import Shift
 from .parameters import Parameter
 from .utils import poly_map_domain, comb
@@ -28,7 +27,7 @@ __all__ = [
 ]
 
 
-class PolynomialBase(ParametricModel):
+class PolynomialBase(FittableModel):
     """
     Base class for all polynomial-like models with an arbitrary number of
     parameters in the form of coeffecients.
@@ -42,7 +41,7 @@ class PolynomialBase(ParametricModel):
     _param_names = []
 
     linear = True
-    col_deriv = False
+    col_fit_deriv = False
 
     @lazyproperty
     def param_names(self):
@@ -63,8 +62,8 @@ class PolynomialBase(ParametricModel):
     def __getattr__(self, attr):
         if self._param_names and attr in self._param_names:
             return Parameter(attr, default=0.0, model=self)
-        else:
-            return super(PolynomialBase, self).__getattr__(attr)
+
+        raise AttributeError(attr)
 
     def __setattr__(self, attr, value):
         # TODO: Support a means of specifying default values for coefficients
@@ -82,16 +81,14 @@ class PolynomialBase(ParametricModel):
         else:
             super(PolynomialBase, self).__setattr__(attr, value)
 
-    def _deriv_with_constraints(self, params=None, x=None, y=None):
-        if y is None:
-            d = np.array(self.deriv(x=x))
-        else:
-            d = np.array(self.deriv(x=x, y=y))
+    def _validate_params(self, **params):
+        valid_params = set(self._param_names)
+        provided_params = set(params)
+        intersection = valid_params.intersection(provided_params)
+        if len(intersection) != len(provided_params):
+            diff = list(provided_params.difference(valid_params))
+            raise TypeError('Unrecognized input parameters: %s' % diff)
 
-        if self.col_deriv:
-            return d[self._fit_param_indices]
-        else:
-            return d[:, self._fit_param_indices]
 
 class PolynomialModel(PolynomialBase):
     """
@@ -102,8 +99,8 @@ class PolynomialModel(PolynomialBase):
     default values, names and ordering.
     """
 
-    def __init__(self, degree, n_inputs=1, n_outputs=1, param_dim=1,
-                 **params):
+    def __init__(self, degree, n_inputs=1, n_outputs=1, n_models=None,
+                 model_set_axis=None, **params):
         self._degree = degree
         self._order = self.get_num_coeff(n_inputs)
         self._param_names = self._generate_coeff_names(n_inputs)
@@ -111,32 +108,20 @@ class PolynomialModel(PolynomialBase):
         self._n_outputs = n_outputs
 
         if params:
-            p = params.get('c0', params.get('c0_0'))
-            if isinstance(p, collections.Sequence):
-                n_params = len(p)
-            else:
-                n_params = 1
-            if param_dim != n_params:
-                if param_dim == 1:
-                    log.info("Inferred {0} dimensions when creating a {1} model. "
-                             "Resetting param_dim to {2}".format(
-                                 n_params, self.__class__.__name__, n_params))
-                    param_dim = n_params
-                else:
-                    raise ValueError("Number of coefficient sets ({0}) does "
-                                     "not match number of parameter sets "
-                                     "({1}).".format(n_params, param_dim))
-
             self._validate_params(**params)
 
-        super(PolynomialModel, self).__init__(param_dim=param_dim)
+        super(PolynomialModel, self).__init__(
+            n_models=n_models, model_set_axis=model_set_axis, **params)
 
-        for name, value in params.items():
-            setattr(self, name, value)
+    def __repr__(self):
+        return self._format_repr([self.degree])
+
+    def __str__(self):
+        return self._format_str([('Degree', self.degree)])
 
     @property
     def degree(self):
-        """TODO: Docstring for me"""
+        """Degree of polynomial."""
 
         return self._degree
 
@@ -177,15 +162,6 @@ class PolynomialModel(PolynomialBase):
                     c.append((j, i))
         return c[::-1]
 
-    def _validate_params(self, **params):
-        numcoeff = self._order
-        assert(len(params) == numcoeff)
-        valid_params = set(self._param_names)
-        provided_params = set(params)
-        diff = valid_params.difference(provided_params)
-        if diff:
-            raise TypeError('Unrecognized input parameters: %s' % diff)
-
     def _generate_coeff_names(self, ndim):
         names = []
         if ndim == 1:
@@ -218,10 +194,10 @@ class OrthoPolynomialBase(PolynomialBase):
         degree in y
     x_domain : list or None
         domain of the x independent variable
-    y_domain : list or None
-        domain of the y independent variable
     x_window : list or None
         range of the x independent variable
+    y_domain : list or None
+        domain of the y independent variable
     y_window : list or None
         range of the y independent variable
     param_dim : int
@@ -234,7 +210,8 @@ class OrthoPolynomialBase(PolynomialBase):
     n_outputs = 1
 
     def __init__(self, x_degree, y_degree, x_domain=None, x_window=None,
-                 y_domain=None, y_window=None, param_dim=1, **params):
+                 y_domain=None, y_window=None, n_models=None,
+                 model_set_axis=None, **params):
         # TODO: Perhaps some of these other parameters should be properties?
         # TODO: An awful lot of the functionality in this method is still
         # shared by PolynomialModel; perhaps some of it can be generalized in
@@ -249,28 +226,18 @@ class OrthoPolynomialBase(PolynomialBase):
         self._param_names = self._generate_coeff_names()
 
         if params:
-            p = params.get('c0_0')
-            if isinstance(p, collections.Sequence):
-                n_params = len(p)
-            else:
-                n_params = 1
-            if param_dim != n_params:
-                if param_dim == 1:
-                    log.info("Inferred {0} dimensions when creating a {1} "
-                             "model. Resetting param_dim to {2}".format(
-                                 n_params, self.__class__.__name__, n_params))
-                    param_dim = n_params
-                else:
-                    raise ValueError("Number of coefficient sets {0} does "
-                                     "not match number of parameter sets "
-                                     "({1})".format(n_params, param_dim))
-
             self._validate_params(**params)
 
-        super(OrthoPolynomialBase, self).__init__(param_dim=param_dim)
+        super(OrthoPolynomialBase, self).__init__(
+            n_models=n_models, model_set_axis=model_set_axis, **params)
 
-        for name, value in params.items():
-            setattr(self, name, value)
+    def __repr__(self):
+        return self._format_repr([self.x_degree, self.y_degree])
+
+    def __str__(self):
+        return self._format_str(
+            [('X-Degree', self.x_degree),
+             ('Y-Degree', self.y_degree)])
 
     def get_num_coeff(self):
         """
@@ -284,10 +251,6 @@ class OrthoPolynomialBase(PolynomialBase):
 
         return (self.x_degree + 1) * (self.y_degree + 1)
 
-    def _validate_params(self, **params):
-        numcoeff = self.get_num_coeff()
-        assert(len(params) == numcoeff)
-
     def _invlex(self):
         # TODO: This is a very slow way to do this; fix it and related methods
         # like _alpha
@@ -365,7 +328,7 @@ class OrthoPolynomialBase(PolynomialBase):
         raise NotImplementedError("Subclasses should implement this")
 
     @format_input
-    def __call__(self, x, y):
+    def __call__(self, x, y, model_set_axis=None):
         """
         Transforms data using this model.
 
@@ -375,8 +338,8 @@ class OrthoPolynomialBase(PolynomialBase):
         y : scalar, lis or array
         """
 
-        assert x.shape == y.shape, \
-            "Expected input arrays to have the same shape"
+        if x.shape != y.shape:
+            raise ValueError("Expected input arrays to have the same shape")
         if self.x_domain is not None:
             x = poly_map_domain(x, self.x_domain, self.x_window)
         if self.y_domain is not None:
@@ -404,18 +367,16 @@ class Chebyshev1D(PolynomialModel):
         keyword : value pairs, representing parameter_name: value
     """
 
-    def __init__(self, degree, domain=None, window=[-1, 1], param_dim=1,
-                 **params):
+    def __init__(self, degree, domain=None, window=[-1, 1], n_models=None,
+                 model_set_axis=None, **params):
         self.domain = domain
         self.window = window
-        super(Chebyshev1D, self).__init__(degree, n_inputs=1, n_outputs=1,
-                                          param_dim=param_dim,
-                                          **params)
+        super(Chebyshev1D, self).__init__(
+            degree, n_inputs=1, n_outputs=1, n_models=n_models,
+            model_set_axis=model_set_axis, **params)
 
     def clenshaw(self, x, coeff):
-        """
-        Evaluates the polynomial using Clenshaw's algorithm.
-        """
+        """Evaluates the polynomial using Clenshaw's algorithm."""
 
         if isinstance(x, tuple) or isinstance(x, list):
             x = np.asarray(x)
@@ -435,7 +396,7 @@ class Chebyshev1D(PolynomialModel):
                 c1 = tmp + c1 * x2
         return c0 + c1 * x
 
-    def deriv(self, x, *params):
+    def fit_deriv(self, x, *params):
         """
         Computes the Vandermonde matrix.
 
@@ -462,7 +423,7 @@ class Chebyshev1D(PolynomialModel):
         return np.rollaxis(v, 0, v.ndim)
 
     @format_input
-    def __call__(self, x):
+    def __call__(self, x, model_set_axis=None):
         """
         Transforms data using this model.
 
@@ -495,13 +456,13 @@ class Legendre1D(PolynomialModel):
         keyword: value pairs, representing parameter_name: value
     """
 
-    def __init__(self, degree, domain=None, window=[-1, 1], param_dim=1,
-                 **params):
+    def __init__(self, degree, domain=None, window=[-1, 1], n_models=None,
+                 model_set_axis=None, **params):
         self.domain = domain
         self.window = window
-        super(Legendre1D, self).__init__(degree, n_inputs=1, n_outputs=1,
-                                         param_dim=param_dim,
-                                         **params)
+        super(Legendre1D, self).__init__(
+            degree, n_inputs=1, n_outputs=1, n_models=n_models,
+            model_set_axis=model_set_axis, **params)
 
     def clenshaw(self, x, coeff):
         if isinstance(x, tuple) or isinstance(x, list):
@@ -523,7 +484,7 @@ class Legendre1D(PolynomialModel):
                 c1 = tmp + (c1 * x * (2 * nd - 1)) / nd
         return c0 + c1 * x
 
-    def deriv(self, x, *params):
+    def fit_deriv(self, x, *params):
         """
         Computes the Vandermonde matrix.
 
@@ -549,7 +510,7 @@ class Legendre1D(PolynomialModel):
         return np.rollaxis(v, 0, v.ndim)
 
     @format_input
-    def __call__(self, x):
+    def __call__(self, x, model_set_axis=None):
         """
         Transforms data using this model.
 
@@ -582,15 +543,15 @@ class Polynomial1D(PolynomialModel):
         keyword: value pairs, representing parameter_name: value
     """
 
-    def __init__(self, degree, domain=[-1, 1], window=[-1, 1], param_dim=1,
-                 **params):
+    def __init__(self, degree, domain=[-1, 1], window=[-1, 1], n_models=None,
+                 model_set_axis=None, **params):
         self.domain = domain
         self.window = window
-        super(Polynomial1D, self).__init__(degree, n_inputs=1,
-                                           n_outputs=1,
-                                           param_dim=param_dim, **params)
+        super(Polynomial1D, self).__init__(
+            degree, n_inputs=1, n_outputs=1, n_models=n_models,
+            model_set_axis=model_set_axis, **params)
 
-    def deriv(self, x, *params):
+    def fit_deriv(self, x, *params):
         """
         Computes the Vandermonde matrix.
 
@@ -621,7 +582,7 @@ class Polynomial1D(PolynomialModel):
         return c0
 
     @format_input
-    def __call__(self, x):
+    def __call__(self, x, model_set_axis=None):
         """
         Transforms data using this model.
 
@@ -641,8 +602,9 @@ class Polynomial2D(PolynomialModel):
     Represents a general polynomial of degree n:
 
     .. math::
-        P(x,y) = c_{0_0} + c_{1_0}*x + ...+ c_{n_0}*x^n + c_{0_1}*y + ...+ c_{0_n}*y^n
-        + c_{1_1}*x*y + c_{1_2}*x*y^2 + ... + c_{1_(n-1)}*x*y^{n-1}+ ... + c_{(n-1)_1}*x^{n-1}*y
+
+        P(x,y) = c_{00} + c_{10}x + ...+ c_{n0}x^n + c_{01}y + ...+ c_{0n}y^n
+        + c_{11}xy + c_{12}xy^2 + ... + c_{1(n-1)}xy^{n-1}+ ... + c_{(n-1)1}x^{n-1}y
 
     Parameters
     ----------
@@ -664,11 +626,11 @@ class Polynomial2D(PolynomialModel):
     """
 
     def __init__(self, degree, x_domain=[-1, 1], y_domain=[-1, 1],
-                 x_window=[-1, 1], y_window=[-1, 1],
-                 param_dim=1, **params):
-        super(Polynomial2D, self).__init__(degree, n_inputs=2,
-                                           n_outputs=1,
-                                           param_dim=param_dim, **params)
+                 x_window=[-1, 1], y_window=[-1, 1], n_models=None,
+                 model_set_axis=None, **params):
+        super(Polynomial2D, self).__init__(
+            degree, n_inputs=2, n_outputs=1, n_models=n_models,
+            model_set_axis=model_set_axis, **params)
         self.x_domain = x_domain
         self.y_domain = y_domain
         self.x_window = x_window
@@ -697,7 +659,7 @@ class Polynomial2D(PolynomialModel):
             r0 = coeff[n + 1]
         return r0 + r1 + r2
 
-    def deriv(self, x, y, *params):
+    def fit_deriv(self, x, y, *params):
         """
         Computes the Vandermonde matrix.
 
@@ -749,7 +711,7 @@ class Polynomial2D(PolynomialModel):
         return np.array(coeff[::-1])
 
     @format_input
-    def __call__(self, x, y):
+    def __call__(self, x, y, model_set_axis=None):
         """
         Transforms data using this model.
 
@@ -762,8 +724,8 @@ class Polynomial2D(PolynomialModel):
         """
 
         invcoeff = self.invlex_coeff()
-        assert x.shape == y.shape, \
-            "Expected input arrays to have the same shape"
+        if x.shape != y.shape:
+            raise ValueError("Expected input arrays to have the same shape")
 
         return self.mhorner(x, y, invcoeff)
 
@@ -799,13 +761,12 @@ class Chebyshev2D(OrthoPolynomialBase):
     """
 
     def __init__(self, x_degree, y_degree, x_domain=None, x_window=[-1, 1],
-                 y_domain=None, y_window=[-1,1], param_dim=1, **params):
-        super(Chebyshev2D, self).__init__(x_degree, y_degree,
-                                          x_domain=x_domain,
-                                          y_domain=y_domain,
-                                          x_window=x_window,
-                                          y_window=y_window,
-                                          param_dim=param_dim, **params)
+                 y_domain=None, y_window=[-1,1], n_models=None,
+                 model_set_axis=None, **params):
+        super(Chebyshev2D, self).__init__(
+            x_degree, y_degree, x_domain=x_domain, y_domain=y_domain,
+            x_window=x_window, y_window=y_window, n_models=n_models,
+            model_set_axis=model_set_axis, **params)
 
     def _fcache(self, x, y):
         """
@@ -826,13 +787,15 @@ class Chebyshev2D(OrthoPolynomialBase):
             kfunc[n] = 2 * y * kfunc[n - 1] - kfunc[n - 2]
         return kfunc
 
-    def deriv(self, x, y, *params):
+    def fit_deriv(self, x, y, *params):
         """
         Derivatives with respect to the coefficients.
 
         This is an array with Chebyshev polynomials:
 
-        Tx0Ty0  Tx1Ty0...TxnTy0...TxnTym
+        .. math::
+
+            T_{x_0}T_{y_0}, T_{x_1}T_{y_0}...T_{x_n}T_{y_0}...T_{x_n}T_{y_m}
 
         Parameters
         ----------
@@ -912,13 +875,12 @@ class Legendre2D(OrthoPolynomialBase):
     """
 
     def __init__(self, x_degree, y_degree, x_domain=None, x_window=[-1, 1],
-                 y_domain=None, y_window=[-1, 1], param_dim=1, **params):
-        super(Legendre2D, self).__init__(x_degree, y_degree,
-                                         x_domain=x_domain,
-                                         y_domain=y_domain,
-                                         x_window=x_window,
-                                         y_window=y_window,
-                                         param_dim=param_dim, **params)
+                 y_domain=None, y_window=[-1, 1], n_models=None,
+                 model_set_axis=None, **params):
+        super(Legendre2D, self).__init__(
+            x_degree, y_degree, x_domain=x_domain, y_domain=y_domain,
+            x_window=x_window, y_window=y_window, n_models=n_models,
+            model_set_axis=model_set_axis, **params)
 
     def _fcache(self, x, y):
         """
@@ -941,7 +903,7 @@ class Legendre2D(OrthoPolynomialBase):
                                   (n - 1) * kfunc[n + x_terms - 2]) / (n)
         return kfunc
 
-    def deriv(self, x, y, *params):
+    def fit_deriv(self, x, y, *params):
         """
         Derivatives with repect to the coefficients.
         This is an array with Legendre polynomials:
@@ -990,7 +952,7 @@ class Legendre2D(OrthoPolynomialBase):
         return np.rollaxis(d, 0, d.ndim)
 
 
-class _SIP1D(Model):
+class _SIP1D(PolynomialBase):
     """
     This implements the Simple Imaging Polynomial Model (SIP) in 1D.
 
@@ -1000,71 +962,25 @@ class _SIP1D(Model):
 
     n_inputs = 2
 
-    def __init__(self, order, coeff_prefix, param_dim=1, **params):
+    def __init__(self, order, coeff_prefix, n_models=None,
+                 model_set_axis=None, **params):
         self.order = order
         self.coeff_prefix = coeff_prefix
         self._param_names = self._generate_coeff_names(coeff_prefix)
-        coeffname = '{0}_0_2'.format(coeff_prefix)
 
         if params:
-            p = params.get(coeffname, None)
-            if isinstance(p, collections.Sequence):
-                n_params = len(p)
-            else:
-                n_params = 1
+            self._validate_params(**params)
 
-            # TODO: This pattern is repeated in so many models and seems like a
-            # check that could be moved into a method
-            if param_dim != n_params:
-                if param_dim == 1:
-                    log.info("Inferred {0} dimensions when creating a {1} "
-                             "model. Resetting param_dim to {2}".format(
-                                 n_params, self.__class__.__name__, n_params))
-                    param_dim = n_params
-                else:
-                    raise ValueError(
-                        "Number of coefficient sets ({0}) does not match the "
-                        "number of parameter sets ({1}).".format(
-                            n_params, param_dim))
-
-            self._validate_params(ndim=2, **params)
-
-        super(_SIP1D, self).__init__(param_dim=param_dim, **params)
+        super(_SIP1D, self).__init__(n_models=n_models,
+                                     model_set_axis=model_set_axis, **params)
 
     def __repr__(self):
-        fmt = """
-        Model: {0}
-        order: {1}
-        coeff_prefix: {2}
-        param_dim: {3}
-        params: {4}
-        """.format(
-              self.__class__.__name__,
-              self.order,
-              self.coeff_prefix,
-              self.param_dim,
-              indent('\n'.join('{0}: {1}'.format(n, getattr(self, '_' + n))
-                     for n in self.param_names), width=19))
-
-        return dedent(fmt[1:])
+        return self._format_repr(args=[self.order, self.coeff_prefix])
 
     def __str__(self):
-        fmt = """
-        Model: {0}
-        order: {1}
-        coeff_prefix: {2}
-        Parameter sets: {3}
-        Parameters:
-                   {3}
-        """.format(
-              self.__class__.__name__,
-              self.order,
-              self.coeff_prefix,
-              self.param_dim,
-              indent('\n'.join('{0}: {1}'.format(n, getattr(self, '_' + n))
-                     for n in self.param_names), width=19))
-
-        return dedent(fmt[1:])
+        return self._format_str(
+            [('Order', self.order),
+             ('Coeff. Prefix', self.coeff_prefix)])
 
     def get_num_coeff(self, ndim):
         """
@@ -1074,8 +990,9 @@ class _SIP1D(Model):
         if self.order < 2 or self.order > 9:
             raise ValueError("Degree of polynomial must be 2< deg < 9")
 
-        nmixed = comb(self.order - 1, ndim)
-        numc = self.order * ndim + nmixed + 1
+        nmixed = comb(self.order, ndim)
+        # remove 3 terms because SIP deg >= 2
+        numc = self.order * ndim + nmixed - 2
         return numc
 
     def _generate_coeff_names(self, coeff_prefix):
@@ -1090,29 +1007,25 @@ class _SIP1D(Model):
                     names.append('{0}_{1}_{2}'.format(coeff_prefix, i, j))
         return names
 
-    def _validate_params(self, ndim, **params):
-        numcoeff = self.get_num_coeff(ndim)
-        assert(len(params) == numcoeff)
-
     def _coef_matrix(self, coeff_prefix):
         mat = np.zeros((self.order + 1, self.order + 1))
         for i in range(2, self.order + 1):
-            attr = '_{0}_{1}_{2}'.format(coeff_prefix, i, 0)
+            attr = '{0}_{1}_{2}'.format(coeff_prefix, i, 0)
             mat[i, 0] = getattr(self, attr).value
         for i in range(2, self.order + 1):
-            attr = '_{0}_{1}_{2}'.format(coeff_prefix, 0, i)
+            attr = '{0}_{1}_{2}'.format(coeff_prefix, 0, i)
             mat[0, i] = getattr(self, attr).value
         for i in range(1, self.order):
             for j in range(1, self.order):
                 if i + j < self.order + 1:
-                    attr = '_{0}_{1}_{2}'.format(coeff_prefix, i, j)
+                    attr = '{0}_{1}_{2}'.format(coeff_prefix, i, j)
                     mat[i, j] = getattr(self, attr).value
         return mat
 
     def _eval_sip(self, x, y, coef):
         x = np.asarray(x, dtype=np.float64)
         y = np.asarray(y, dtype=np.float64)
-        if self.coeff_prefix == 'a':
+        if self.coeff_prefix == 'A':
             result = np.zeros(x.shape)
         else:
             result = np.zeros(y.shape)
@@ -1128,7 +1041,7 @@ class _SIP1D(Model):
         return self._eval_sip(x, y, mcoef)
 
 
-class _SIPModel(SerialCompositeModel):
+class SIP(Model):
     """
     Simple Imaging Polynomial (SIP) model.
 
@@ -1141,21 +1054,19 @@ class _SIPModel(SerialCompositeModel):
         CRPIX values
     a_order : int
         SIP polynomial order for first axis
-    a_coeff : dict
-        SIP coefficients for first axis
-    coeff_prefix : str: 'a', 'b', 'A' or 'B'
-        SIP coefficient preffix
     b_order : int
         SIP order for second axis
+    a_coeff : dict
+        SIP coefficients for first axis
     b_coeff : dict
         SIP coefficients for the second axis
-    a_inv_order : int
+    ap_order : int
         order for the inverse transformation (AP coefficients)
-    a_inv_coeff : dict
-        coefficients for the inverse transform
-    b_inv_order : int
+    bp_order : int
         order for the inverse transformation (BP coefficients)
-    b_inv_coeff : dict
+    ap_coeff : dict
+        coefficients for the inverse transform
+    bp_coeff : dict
         coefficients for the inverse transform
     param_dim : int
         number of parameter sets
@@ -1168,140 +1079,114 @@ class _SIPModel(SerialCompositeModel):
     n_inputs = 2
     n_outputs = 2
 
-    def __init__(self, crpix, a_order, a_coeff, b_order, b_coeff,
-                 ap_order=None, ap_coeff=None, bp_order=None, bp_coeff=None,
-                 param_dim=1):
-        self.crpix = crpix
-        self.a_order = a_order
-        self.b_order = b_order
-        self.a_coeff = a_coeff
-        self.b_coeff = b_coeff
-        self.ap_order = ap_order
-        self.bp_order = bp_order
-        self.ap_coeff = ap_coeff
-        self.bp_coeff = bp_coeff
+    def __init__(self, crpix, a_order, b_order, a_coeff={}, b_coeff={},
+                 ap_order=None, bp_order=None, ap_coeff={}, bp_coeff={},
+                 n_models=None, model_set_axis=None):
+        self._crpix = crpix
+        self._a_order = a_order
+        self._b_order = b_order
+        self._a_coeff = a_coeff
+        self._b_coeff = b_coeff
+        self._ap_order = ap_order
+        self._bp_order = bp_order
+        self._ap_coeff = ap_coeff
+        self._bp_coeff = bp_coeff
         self.shift_a = Shift(-crpix[0])
         self.shift_b = Shift(-crpix[1])
-        self.sip1d_a = _SIP1D(a_order, coeff_prefix='A',
-                              param_dim=param_dim, **a_coeff)
-        self.sip1d_b = _SIP1D(b_order, coeff_prefix='B',
-                              param_dim=param_dim, **b_coeff)
-        if (ap_order is not None and ap_coeff is not None and
-                bp_order is not None and bp_coeff is not None):
-            self.sip1d_ap = _SIP1D(ap_order, coeff_prefix='AP', **ap_coeff)
-            self.sip1d_bp = _SIP1D(bp_order, coeff_prefix='BP', **bp_coeff)
-        else:
-            self.sip1d_ap = None
-            self.sip1d_bp = None
-
-        super(_SIPModel, self).__init__([self.shift_a, self.shift_b,
-                                         self.sip1d_a, self.sip1d_b])
-        
-
-class SIP(_SIPModel):
-    def __init__(self, crpix, a_order, a_coeff, b_order, b_coeff,
-                 ap_order=None, ap_coeff=None, bp_order=None, bp_coeff=None,
-                 param_dim=1):
-
-        super(SIP, self).__init__(crpix, a_order, a_coeff,
-                                  b_order, b_coeff, ap_order, ap_coeff,
-                                  bp_order, bp_coeff, param_dim=1)
-
-    def inverse(self):
-        if (self.ap_order is not None and self.ap_coeff is not None and
-                self.bp_order is not None and self.bp_coeff is not None):
-            return InverseSIP(self.crpix, self.a_order, self.a_coeff,
-                              self.b_order, self.b_coeff,
-                              self.ap_order, self.ap_coeff,
-                              self.bp_order, self.bp_coeff)
-        else:
-            raise NotImplementedError("An analytical inverse transform has "
-                                      "not been implemented for this model.")
+        self.sip1d_a = _SIP1D(a_order, coeff_prefix='A', n_models=n_models,
+                              model_set_axis=model_set_axis, **a_coeff)
+        self.sip1d_b = _SIP1D(b_order, coeff_prefix='B', n_models=n_models,
+                              model_set_axis=model_set_axis, **b_coeff)
+        super(SIP, self).__init__(n_models=n_models,
+                                  model_set_axis=model_set_axis)
 
     def __repr__(self):
-        models = [self.shift_a, self.shift_b, self.sip1d_a, self.sip1d_b]
-        fmt = """
-            Model:  {0}
-            """.format(self.__class__.__name__)
-        fmt1 = " %s  " * len(models) % tuple([repr(model) for model in models])
-        fmt = fmt + fmt1
-        return fmt
+        return '<{0}({1!r})>'.format(self.__class__.__name__,
+            [self.shift_a, self.shift_b, self.sip1d_a, self.sip1d_b])
 
     def __str__(self):
-        models = [self.shift_a, self.shift_b, self.sip1d_a, self.sip1d_b]
-        fmt = """
-            Model:  {0}
-            """.format(self.__class__.__name__)
-        fmt1 = " %s  " * len(models) % tuple([str(model) for model in models])
-        fmt = fmt + fmt1
-        return fmt
+        parts = ['Model: {0}'.format(self.__class__.__name__)]
+        for model in [self.shift_a, self.shift_b, self.sip1d_a, self.sip1d_b]:
+            parts.append(indent(str(model), width=4))
+            parts.append('')
+
+        return '\n'.join(parts)
+
+    def inverse(self):
+        if (self._ap_order is not None and self._bp_order is not None):
+            return InverseSIP(self._ap_order, self._bp_order,
+                              self._ap_coeff, self._bp_coeff)
+        else:
+            raise NotImplementedError("SIP inverse coefficients are not available.")
 
     def __call__(self, x, y):
-        """
-        Transforms data using this model.
+        u = self.shift_a(x)
+        v = self.shift_b(y)
+        f = self.sip1d_a(u, v)
+        g = self.sip1d_b(u, v)
+        return f, g
 
-        Parameters
-        ----------
-        x : scalar, list, array
-            input
-        y : scalar, list or array
-            input
-        """
-        x = self.shift_a(x)
-        y = self.shift_b(y)
-        x1 = self.sip1d_a(x, y)
-        y1 = self.sip1d_b(x, y)
+
+class InverseSIP(Model):
+    """
+    Inverse Simple Imaging Polynomial
+
+    Parameters
+    ----------
+    ap_order : int
+        order for the inverse transformation (AP coefficients)
+    bp_order : int
+        order for the inverse transformation (BP coefficients)
+    ap_coeff : dict
+        coefficients for the inverse transform
+    bp_coeff : dict
+        coefficients for the inverse transform
+    param_dim : int
+        number of parameter sets
+
+    """
+
+    n_inputs = 2
+    n_outputs = 2
+
+    def __init__(self, ap_order, bp_order, ap_coeff={}, bp_coeff={},
+                 n_models=None, model_set_axis=None):
+        self._ap_order = ap_order
+        self._bp_order = bp_order
+        self._ap_coeff = ap_coeff
+        self._bp_coeff = bp_coeff
+
+        # define the 0th term in order to use Polynomial2D
+        ap_coeff.setdefault('AP_0_0', 0)
+        bp_coeff.setdefault('BP_0_0', 0)
+
+        ap_coeff_params = dict((k.replace('AP_', 'c'), v)
+                               for k, v in ap_coeff.items())
+        bp_coeff_params = dict((k.replace('BP_', 'c'), v)
+                               for k, v in bp_coeff.items())
+
+        self.sip1d_ap = Polynomial2D(degree=ap_order,
+                                     model_set_axis=model_set_axis,
+                                     **ap_coeff_params)
+        self.sip1d_bp = Polynomial2D(degree=bp_order,
+                                     model_set_axis=model_set_axis,
+                                     **bp_coeff_params)
+        super(InverseSIP, self).__init__(n_models=n_models,
+                                         model_set_axis=model_set_axis)
+
+    def __call__(self, x, y):
+        x1 = self.sip1d_ap(x, y)
+        y1 = self.sip1d_bp(x, y)
         return x1, y1
 
-
-class InverseSIP(_SIPModel):
-    def __init__(self, crpix, a_order, a_coeff, b_order, b_coeff,
-                 ap_order, ap_coeff, bp_order, bp_coeff,
-                 param_dim=1):
-
-        super(InverseSIP, self).__init__(crpix, a_order, a_coeff,
-                                         b_order, b_coeff,
-                                         ap_order, ap_coeff,
-                                         bp_order, bp_coeff, param_dim=1)
-
-    def inverse(self):
-        if (self.a_order is not None and self.a_coeff is not None and
-                self.b_order is not None and self.bp_coeff is not None):
-            return SIP(self.crpix, self.a_order, self.a_coeff,
-                       self.b_order, self.b_coeff,
-                       self.ap_order, self.ap_coeff,
-                       self.bp_order, self.bp_coeff)
-        else:
-            raise NotImplementedError("An analytical inverse transform has "
-                                      "not been implemented for this model.")
-
-    def __call__(self, x, y):
-        if self.sip1d_ap is not None and self.sip1d_bp is not None:
-            x1 = self.sip1d_ap(x, y)
-            y1 = self.sip1d_bp(x, y)
-            x = self.shift_a.inverse()(x1)
-            y = self.shift_b.inverse()(y1)
-            return x, y
-        else:
-            raise NotImplementedError("An analytical inverse transform has "
-                                      "not been implemented for this model.")
-
     def __repr__(self):
-        models = [self.sip1d_ap, self.sip1d_bp, self.shift_a.inverse(),
-                  self.shift_b.inverse()]
-        fmt = """
-            Model:  {0}
-            """.format(self.__class__.__name__)
-        fmt1 = " %s  " * len(models) % tuple([repr(model) for model in models])
-        fmt = fmt + fmt1
-        return fmt
+        return '<{0}({1!r})>'.format(self.__class__.__name__,
+            [self.sip1d_ap, self.sip1d_bp])
 
     def __str__(self):
-        models = [self.sip1d_ap, self.sip1d_bp, self.shift_a.inverse(),
-                  self.shift_b.inverse()]
-        fmt = """
-            Model:  {0}
-            """.format(self.__class__.__name__)
-        fmt1 = " %s  " * len(models) % tuple([str(model) for model in models])
-        fmt = fmt + fmt1
-        return fmt
+        parts = ['Model: {0}'.format(self.__class__.__name__)]
+        for model in [self.sip1d_ap, self.sip1d_bp]:
+            parts.append(indent(str(model), width=4))
+            parts.append('')
+
+        return '\n'.join(parts)
