@@ -1,12 +1,19 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import division
+
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
 import types
+
+from ..core import Fittable1DModel
+from ..parameters import Parameter
 from .. import models
 from .. import fitting
 import numpy as np
 from numpy.testing import utils
 from numpy.random import RandomState
 from ...tests.helper import pytest
+from .utils import ignore_non_integer_warning
 
 try:
     from scipy import optimize
@@ -31,7 +38,7 @@ class TestNonLinearConstraints(object):
     @pytest.mark.skipif('not HAS_SCIPY')
     def test_fixed_par(self):
         g1 = models.Gaussian1D(10, mean=14.9, stddev=.3, fixed={'amplitude': True})
-        fitter = fitting.NonLinearLSQFitter()
+        fitter = fitting.LevMarLSQFitter()
         model = fitter(g1, self.x, self.ny1)
         assert model.amplitude.value == 10
 
@@ -42,8 +49,8 @@ class TestNonLinearConstraints(object):
             mean = 50 * model.stddev
             return mean
         g1 = models.Gaussian1D(10, mean=14.9, stddev=.3, tied={'mean': tied})
-        fitter = fitting.NonLinearLSQFitter()
-        model = fitter(g1,self.x, self.ny1)
+        fitter = fitting.LevMarLSQFitter()
+        model = fitter(g1, self.x, self.ny1)
         utils.assert_allclose(model.mean.value, 50 * model.stddev, rtol=10 ** (-5))
 
     @pytest.mark.skipif('not HAS_SCIPY')
@@ -81,7 +88,7 @@ class TestNonLinearConstraints(object):
         n = np.random.randn(100)
         ny = y + n
         fitpar, s = optimize.leastsq(errf, p0, args=(self.x, ny))
-        fitter = fitting.NonLinearLSQFitter()
+        fitter = fitting.LevMarLSQFitter()
         model = fitter(g1, self.x, ny)
         utils.assert_allclose(model.parameters, fitpar, rtol=5 * 10 ** (-3))
 
@@ -113,7 +120,7 @@ class TestBounds(object):
         guess_intercept = 0.0
         bounds = {'slope': (-1.5, 5.0), 'intercept': (-1.0, 1.0)}
         line_model = models.Linear1D(guess_slope, guess_intercept, bounds=bounds)
-        fitter = fitting.NonLinearLSQFitter()
+        fitter = fitting.LevMarLSQFitter()
         model = fitter(line_model, self.x, self.y)
         slope = model.slope.value
         intercept = model.intercept.value
@@ -127,8 +134,9 @@ class TestBounds(object):
         guess_intercept = 0.0
         bounds = {'slope': (-1.5, 5.0), 'intercept': (-1.0, 1.0)}
         line_model = models.Linear1D(guess_slope, guess_intercept, bounds=bounds)
-        fitter = fitting.SLSQPFitter()
-        model = fitter(line_model, self.x, self.y)
+        fitter = fitting.SLSQPLSQFitter()
+        with ignore_non_integer_warning():
+            model = fitter(line_model, self.x, self.y)
         slope = model.slope.value
         intercept = model.intercept.value
         assert slope + 10 ** -5 >= bounds['slope'][0]
@@ -145,7 +153,7 @@ class TestBounds(object):
         gauss = models.Gaussian2D(amplitude=10., x_mean=5., y_mean=5.,
                                   x_stddev=4., y_stddev=4., theta=0.5,
                                   bounds=bounds)
-        gauss_fit = fitting.NonLinearLSQFitter()
+        gauss_fit = fitting.LevMarLSQFitter()
         model = gauss_fit(gauss, X, Y, self.data)
         x_mean = model.x_mean.value
         y_mean = model.y_mean.value
@@ -169,8 +177,9 @@ class TestBounds(object):
         gauss = models.Gaussian2D(amplitude=10., x_mean=5., y_mean=5.,
                                   x_stddev=4., y_stddev=4., theta=0.5,
                                   bounds=bounds)
-        gauss_fit = fitting.SLSQPFitter()
-        model = gauss_fit(gauss, X, Y, self.data)
+        gauss_fit = fitting.SLSQPLSQFitter()
+        with ignore_non_integer_warning():
+            model = gauss_fit(gauss, X, Y, self.data)
         x_mean = model.x_mean.value
         y_mean = model.y_mean.value
         x_stddev = model.x_stddev.value
@@ -212,15 +221,11 @@ def test_set_fixed_1():
     gauss = models.Gaussian1D(amplitude=20, mean=2, stddev=1)
     gauss.mean.fixed = True
     assert gauss.fixed == {'amplitude': False, 'mean': True, 'stddev': False}
-    fitparams, _ = gauss._model_to_fit_params()
-    assert np.all(fitparams == [20, 1])
 
 
 def test_set_fixed_2():
     gauss = models.Gaussian1D(amplitude=20, mean=2, stddev=1, fixed={'mean': True})
     assert gauss.mean.fixed is True
-    fitparams, _ = gauss._model_to_fit_params()
-    assert np.all(fitparams == [20, 1])
 
 
 def test_set_tied_1():
@@ -238,7 +243,7 @@ def test_set_tied_2():
         return 50 * model.stddev
 
     gauss = models.Gaussian1D(amplitude=20, mean=2, stddev=1,
-                                   tied={'amplitude': tie_amplitude})
+                              tied={'amplitude': tie_amplitude})
     assert gauss.amplitude.tied != False
 
 
@@ -246,8 +251,6 @@ def test_unset_fixed():
     gauss = models.Gaussian1D(amplitude=20, mean=2, stddev=1, fixed={'mean': True})
     gauss.mean.fixed = False
     assert gauss.fixed == {'amplitude': False, 'mean': False, 'stddev': False}
-    fitparams, _  = gauss._model_to_fit_params()
-    assert np.all(fitparams == [20, 2, 1])
 
 
 def test_unset_tied():
@@ -258,8 +261,6 @@ def test_unset_tied():
                               tied={'amplitude': tie_amplitude})
     gauss.amplitude.tied = False
     assert gauss.tied == {'amplitude': False, 'mean': False, 'stddev': False}
-    fitparams, _  = gauss._model_to_fit_params()
-    assert np.all(fitparams == [20, 2, 1])
 
 
 def test_set_bounds_1():
@@ -269,6 +270,7 @@ def test_set_bounds_1():
                             'mean': (None, None),
                             'stddev': (0.0, None)}
 
+
 def test_set_bounds_2():
     gauss = models.Gaussian1D(amplitude=20, mean=2, stddev=1)
     gauss.stddev.min = 0.
@@ -285,3 +287,120 @@ def test_unset_bounds():
     assert gauss.bounds == {'amplitude': (None, None),
                             'mean': (None, None),
                             'stddev': (None, None)}
+
+
+def test_default_constraints():
+    """Regression test for https://github.com/astropy/astropy/issues/2396
+
+    Ensure that default constraints defined on parameters are carried through
+    to instances of the models those parameters are defined for.
+    """
+
+    class MyModel(Fittable1DModel):
+        a = Parameter(default=1)
+        b = Parameter(default=0, min=0, fixed=True)
+
+        @staticmethod
+        def eval(x, a, b):
+            return x * a + b
+
+    assert MyModel.a.default == 1
+    assert MyModel.b.default == 0
+    assert MyModel.b.min == 0
+    assert MyModel.b.bounds == (0, None)
+    assert MyModel.b.fixed is True
+
+    m = MyModel()
+    assert m.a.value == 1
+    assert m.b.value == 0
+    assert m.b.min == 0
+    assert m.b.bounds == (0, None)
+    assert m.b.fixed is True
+    assert m.bounds == {'a': (None, None), 'b': (0, None)}
+    assert m.fixed == {'a': False, 'b': True}
+
+    # Make a model instance that overrides the default constraints and values
+    m = MyModel(3, 4, bounds={'a': (1, None), 'b': (2, None)},
+                fixed={'a': True, 'b': False})
+    assert m.a.value == 3
+    assert m.b.value == 4
+    assert m.a.min == 1
+    assert m.b.min == 2
+    assert m.a.bounds == (1, None)
+    assert m.b.bounds == (2, None)
+    assert m.a.fixed is True
+    assert m.b.fixed is False
+    assert m.bounds == {'a': (1, None), 'b': (2, None)}
+    assert m.fixed == {'a': True, 'b': False}
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_fit_with_fixed_and_bound_constraints():
+    """
+    Regression test for https://github.com/astropy/astropy/issues/2235
+
+    Currently doesn't test that the fit is any *good*--just that parameters
+    stay within their given constraints.
+    """
+
+    m = models.Gaussian1D(amplitude=3, mean=4, stddev=1,
+                          bounds={'mean': (4, 5)},
+                          fixed={'amplitude': True})
+    x = np.linspace(0, 10, 10)
+    y = np.exp(-x ** 2 / 2)
+
+    f = fitting.LevMarLSQFitter()
+    fitted_1 = f(m, x, y)
+    assert fitted_1.mean >= 4
+    assert fitted_1.mean <= 5
+    assert fitted_1.amplitude == 3.0
+
+    m.amplitude.fixed = False
+    fitted_2 = f(m, x, y)
+    # It doesn't matter anymore what the amplitude ends up as so long as the
+    # bounds constraint was still obeyed
+    assert fitted_1.mean >= 4
+    assert fitted_1.mean <= 5
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_fit_with_bound_constraints_estimate_jacobian():
+    """
+    Regression test for https://github.com/astropy/astropy/issues/2400
+
+    Checks that bounds constraints are obeyed on a custom model that does not
+    define fit_deriv (and thus its Jacobian must be estimated for non-linear
+    fitting).
+    """
+
+    class MyModel(Fittable1DModel):
+        a = Parameter(default=1)
+        b = Parameter(default=2)
+
+        @staticmethod
+        def eval(x, a, b):
+            return a * x + b
+
+    m_real = MyModel(a=1.5, b=-3)
+    x = np.arange(100)
+    y = m_real(x)
+
+    m = MyModel()
+    f = fitting.LevMarLSQFitter()
+    fitted_1 = f(m, x, y)
+
+    # This fit should be trivial so even without constraints on the bounds it
+    # should be right
+    assert np.allclose(fitted_1.a, 1.5)
+    assert np.allclose(fitted_1.b, -3)
+
+    m2 = MyModel()
+    m2.a.bounds = (-2, 2)
+    f2 = fitting.LevMarLSQFitter()
+    fitted_2 = f2(m2, x, y)
+    assert np.allclose(fitted_1.a, 1.5)
+    assert np.allclose(fitted_1.b, -3)
+
+    # Check that the estimated Jacobian was computed (it doesn't matter what
+    # the values are so long as they're not all zero.
+    assert np.any(f2.fit_info['fjac'] != 0)
