@@ -3,19 +3,17 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import collections
 from copy import deepcopy
 
 import numpy as np
 
-from ..units import Unit
+from ..units import Unit, Quantity, UnitsError, dimensionless_unscaled
 from .. import log
 
 from .flag_collection import FlagCollection
 from .nduncertainty import IncompatibleUncertaintiesException, NDUncertainty
-from ..utils.compat.odict import OrderedDict
 from ..io import registry as io_registry
-from ..config import ConfigurationItem
+from ..config import ConfigAlias
 from ..utils.metadata import MetaData
 
 __all__ = ['NDData']
@@ -24,13 +22,9 @@ __all__ = ['NDData']
 __doctest_skip__ = ['NDData']
 
 
-WARN_UNSUPPORTED_CORRELATED = ConfigurationItem(
-    'warn_unsupported_correlated', True,
-    'Whether to issue a warning if NDData arithmetic is performed with '
-    'uncertainties and the uncertainties do not support the propagation '
-    'of correlated uncertainties.'
-)
-
+WARN_UNSUPPORTED_CORRELATED = ConfigAlias(
+    '0.4', 'WARN_UNSUPPORTED_CORRELATED', 'warn_unsupported_correlated',
+    'astropy.nddata.nddata', 'astropy.nddata')
 
 class NDData(object):
     """A Superclass for array-based data in Astropy.
@@ -41,25 +35,27 @@ class NDData(object):
 
     Parameters
     -----------
-    data : `~numpy.ndarray` or `~astropy.nddata.NDData`
+    data : `~numpy.ndarray` or `NDData`
         The actual data contained in this `NDData` object. Not that this
         will always be copies by *reference* , so you should make copy
-        the `data` before passing it in if that's the  desired behavior.
+        the ``data`` before passing it in if that's the  desired behavior.
 
     uncertainty : `~astropy.nddata.NDUncertainty`, optional
         Uncertainties on the data.
 
-    mask : `~numpy.ndarray`, optional
-        Mask for the data, given as a boolean Numpy array with a shape
+    mask : `~numpy.ndarray`-like, optional
+        Mask for the data, given as a boolean Numpy array or any object that
+        can be converted to a boolean Numpy array with a shape
         matching that of the data. The values must be ``False`` where
         the data is *valid* and ``True`` when it is not (like Numpy
-        masked arrays). If `data` is a numpy masked array, providing
-        `mask` here will causes the mask from the masked array to be
+        masked arrays). If ``data`` is a numpy masked array, providing
+        ``mask`` here will causes the mask from the masked array to be
         ignored.
 
-    flags : `~numpy.ndarray` or `~astropy.nddata.FlagCollection`, optional
+    flags : `~numpy.ndarray`-like or `~astropy.nddata.FlagCollection`, optional
         Flags giving information about each pixel. These can be specified
-        either as a Numpy array of any type with a shape matching that of the
+        either as a Numpy array of any type (or an object which can be converted
+        to a Numpy array) with a shape matching that of the
         data, or as a `~astropy.nddata.FlagCollection` instance which has a
         shape matching that of the data.
 
@@ -77,15 +73,15 @@ class NDData(object):
         of this particular object.  e.g., creation date, unique identifier,
         simulation parameters, exposure time, telescope name, etc.
 
-    unit : `astropy.units.UnitBase` instance or str, optional
+    unit : `~astropy.units.UnitBase` instance or str, optional
         The units of the data.
 
 
     Raises
     ------
-    ValueError
+    ValueError :
         If the `uncertainty` or `mask` inputs cannot be broadcast (e.g., match
-        shape) onto `data`.
+        shape) onto ``data``.
 
     Notes
     -----
@@ -120,6 +116,12 @@ class NDData(object):
 
         if isinstance(data, self.__class__):
             self.data = np.array(data.data, subok=True, copy=False)
+            self.uncertainty = data.uncertainty
+            self.mask = data.mask
+            self.flags = data.flags
+            self.wcs = data.wcs
+            self.meta = data.meta
+            self.unit = data.unit
 
             if uncertainty is not None:
                 self.uncertainty = uncertainty
@@ -156,15 +158,29 @@ class NDData(object):
                              "mask will be ignored.")
                 else:
                     self.mask = data.mask
+            elif isinstance(data, Quantity):
+                self.data = np.array(data.value, subok=True, copy=False)
+                self.mask = mask
             else:
                 self.data = np.array(data, subok=True, copy=False)
                 self.mask = mask
 
-            self.uncertainty = uncertainty
             self.flags = flags
             self.wcs = wcs
             self.meta = meta
-            self.unit = unit
+            if isinstance(data, Quantity):
+                if unit is not None:
+                    raise ValueError("Cannot use the unit argument when data "
+                                     "is a Quantity")
+                else:
+                    self.unit = data.unit
+            else:
+                self.unit = unit
+            # This must come after self's unit has been set so that the unit
+            # of the uncertainty, if any, can be converted to the unit of the
+            # unit of self.
+            self.uncertainty = uncertainty
+
 
     def __str__(self):
         return str(self.data)
@@ -176,23 +192,23 @@ class NDData(object):
 
     @property
     def mask(self):
-        return self._mask
+        if self._mask is np.ma.nomask:
+            return None
+        else:
+            return self._mask
 
     @mask.setter
     def mask(self, value):
-        if value is not None:
-            if isinstance(value, np.ndarray):
-                if value.dtype != np.bool_:
-                    raise TypeError("mask must be a boolean Numpy array")
-                else:
-                    if value.shape != self.shape:
-                        raise ValueError("dimensions of mask do not match data")
-                    else:
-                        self._mask = value
+        # Check that value is not either type of null mask.
+        if (value is not None) and (value is not np.ma.nomask):
+            mask = np.array(value, dtype=np.bool_, copy=False)
+            if mask.shape != self.shape:
+                raise ValueError("dimensions of mask do not match data")
             else:
-                raise TypeError("mask must be a Numpy array")
+                self._mask = mask
         else:
-            self._mask = value
+            # internal representation should be one numpy understands
+            self._mask = np.ma.nomask
 
     @property
     def flags(self):
@@ -201,18 +217,17 @@ class NDData(object):
     @flags.setter
     def flags(self, value):
         if value is not None:
-            if isinstance(value, np.ndarray):
-                if value.shape != self.shape:
-                    raise ValueError("dimensions of flags do not match data")
-                else:
-                    self._flags = value
-            elif isinstance(value, FlagCollection):
+            if isinstance(value, FlagCollection):
                 if value.shape != self.shape:
                     raise ValueError("dimensions of FlagCollection does not match data")
                 else:
                     self._flags = value
             else:
-                raise TypeError("flags should be a Numpy array or a FlagCollection instance")
+                flags = np.array(value, copy=False)
+                if flags.shape != self.shape:
+                    raise ValueError("dimensions of flags do not match data")
+                else:
+                    self._flags = flags
         else:
             self._flags = value
 
@@ -224,6 +239,20 @@ class NDData(object):
     def uncertainty(self, value):
         if value is not None:
             if isinstance(value, NDUncertainty):
+                class_name = self.__class__.__name__
+                if self.unit and value._unit:
+                    try:
+                        scaling = (1 * value._unit).to(self.unit)
+                    except UnitsError:
+                        raise UnitsError('Cannot convert unit of uncertainty '
+                                         'to unit of '
+                                         '{0} object.'.format(class_name))
+                    value.array *= scaling
+                elif not self.unit and value._unit:
+                    # Raise an error if uncertainty has unit and data does not
+                    raise ValueError("Cannot assign an uncertainty with unit "
+                                     "to {0} without "
+                                     "a unit".format(class_name))
                 self._uncertainty = value
                 self._uncertainty.parent_nddata = self
             else:
@@ -237,6 +266,19 @@ class NDData(object):
 
     @unit.setter
     def unit(self, value):
+        from . import conf
+
+        try:
+            if self._unit is not None and conf.warn_setting_unit_directly:
+                log.info('Setting the unit directly changes the unit without '
+                         'updating the data or uncertainty. Use the '
+                         '.convert_unit_to() method to change the unit and '
+                         'scale values appropriately.')
+        except AttributeError:
+            # raised if self._unit has not been set yet, in which case the
+            # warning is irrelevant
+            pass
+
         if value is None:
             self._unit = None
         else:
@@ -278,7 +320,16 @@ class NDData(object):
         if self.mask is not None:
             return np.ma.masked_array(self.data, self.mask)
         else:
-            return self.data
+            return np.array(self.data)
+
+    def __array_prepare__(self, array, context=None):
+        """
+        This ensures that a masked array is returned if self is masked.
+        """
+        if self.mask is not None:
+            return np.ma.masked_array(array, self.mask)
+        else:
+            return array
 
     def __getitem__(self, item):
 
@@ -291,12 +342,18 @@ class NDData(object):
 
         if self.mask is not None:
             new_mask = self.mask[item]
+            # mask setter expects an array, always
+            if new_mask.shape == ():
+                new_mask = np.array(new_mask)
         else:
             new_mask = None
 
         if self.flags is not None:
             if isinstance(self.flags, np.ndarray):
                 new_flags = self.flags[item]
+                # flags setter expects an array, always
+                if new_flags.shape == ():
+                    new_flags = np.array(new_flags)
             elif isinstance(self.flags, FlagCollection):
                 raise NotImplementedError('Slicing complex Flags is currently not implemented')
         else:
@@ -313,7 +370,7 @@ class NDData(object):
 
     def _arithmetic(self, operand, propagate_uncertainties, name, operation):
         """
-        {name} another dataset (`operand`) to this dataset.
+        {name} another dataset (``operand``) to this dataset.
 
         Parameters
         ----------
@@ -330,57 +387,133 @@ class NDData(object):
 
         Notes
         -----
-        This method requires the datasets to have identical WCS properties,
-        equivalent units, and identical shapes. Flags and meta-data get set to
-        None in the resulting dataset. The unit in the result is the same as
-        the unit in `self`. Uncertainties are propagated, although correlated
-        errors are not supported by any of the built-in uncertainty classes.
-        If uncertainties are assumed to be correlated, a warning is issued by
-        default (though this can be disabled via the
-        `WARN_UNSUPPORTED_CORRELATED` configuration item). Values masked in
-        either dataset before the operation are masked in the resulting
-        dataset.
+        This method requires the datasets to have identical WCS
+        properties, equivalent units, and identical shapes. Flags and
+        meta-data get set to None in the resulting dataset. The unit
+        in the result is the same as the unit in ``self``. Uncertainties
+        are propagated, although correlated errors are not supported
+        by any of the built-in uncertainty classes.  If uncertainties
+        are assumed to be correlated, a warning is issued by default
+        (though this can be disabled via the
+        ``astropy.nddata.conf.warn_unsupported_correlated``
+        configuration item). Values masked in either dataset before
+        the operation are masked in the resulting dataset.
         """
+        from . import conf
+
         if self.wcs != operand.wcs:
             raise ValueError("WCS properties do not match")
 
-        if not (self.unit is None and operand.unit is None):
-            if (self.unit is None or operand.unit is None
-                    or not self.unit.is_equivalent(operand.unit)):
-                raise ValueError("operand units do not match")
+        # get a sensible placeholder if .unit is None
+        self_unit = self.unit or dimensionless_unscaled
+        operand_unit = operand.unit or dimensionless_unscaled
+
+        # This check could be rolled into the calculation of the result
+        # but checking now avoids a potentially expensive calculation that
+        # would fail anyway.
+        try:
+            # Quantity is designed to work with numpy ufuncs, but plain
+            # Unit is not, so convert units to quantities
+            result_unit = operation(1 * self_unit, 1 * operand_unit).unit
+        except UnitsError:
+            # current API raises ValueError in this case, not UnitError
+            raise ValueError("operand units do not match")
 
         if self.shape != operand.shape:
             raise ValueError("operand shapes do not match")
 
-        if self.unit is not None:
+        # Instead of manually scaling the operand data just let Quantity
+        # handle it.
+        # Order of the arguments is important here if the operation is
+        # addition or subtraction and the units of the operands are different
+        # but compatible. NDData follows the convention that Quantity follows
+        # in that case, with the units of the first operand (i.e. self)
+        # determining the units of the result.
+        data = operation(self.data * self_unit, operand.data * operand_unit)
+
+        result_unit = data.unit
+        # If neither self nor operand had units then should return a result
+        # that has no unit. A check that the result_unit is dimensionless
+        # should not be necessary, but also does no harm.
+        if self.unit is None and operand.unit is None:
+            if result_unit is dimensionless_unscaled:
+                result_unit = None
+            else:
+                raise ValueError("arithmetic result was not unitless even "
+                                 "though operands were unitless")
+        data = data.value
+        new_wcs = deepcopy(self.wcs)
+
+        # Call __class__ in case we are dealing with an inherited type
+        result = self.__class__(data, uncertainty=None,
+                                mask=None, flags=None, wcs=new_wcs,
+                                meta=None, unit=result_unit)
+
+        # Prepare to scale uncertainty if it is needed
+        if operand.uncertainty:
+            operand_uncert_value = operand.uncertainty.array
+
+        # By this point the arithmetic has succeeded, so the input units were
+        # consistent with each other given the operation.
+        #
+        # If the operation is addition or subtraction then need to ensure that
+        # the uncertainty of the operand is the same units as the result
+        # (which will be the same as self.unit).
+
+        # The data ought to also be scaled in this case -- for addition of
+        # a StdDevUncertainty this isn't really necessary but other
+        # uncertainties when added/subtracted may depend on both the operand
+        # uncertainty and the operand data.
+
+        # Since the .unit.to methods create a copy, avoid the conversion
+        # unless it is necessary.
+        if (operation in [np.add, np.subtract] and
+                self.unit != operand.unit):
             operand_data = operand.unit.to(self.unit, operand.data)
+            if operand.uncertainty:
+                operand_uncert_value = operand.unit.to(self.unit,
+                                                       operand_uncert_value)
         else:
             operand_data = operand.data
-        data = operation(self.data, operand_data)
-        result = self.__class__(data)  # in case we are dealing with an inherited type
+
+        if operand.uncertainty:
+            # Create a copy here in case this is returned as the uncertainty
+            # of the result.
+            operand_uncertainty = \
+                operand.uncertainty.__class__(operand_uncert_value, copy=True)
+        else:
+            operand_uncertainty = None
 
         if propagate_uncertainties is None:
             result.uncertainty = None
         elif self.uncertainty is None and operand.uncertainty is None:
             result.uncertainty = None
         elif self.uncertainty is None:
-            result.uncertainty = operand.uncertainty
+            result.uncertainty = operand_uncertainty
         elif operand.uncertainty is None:
-            result.uncertainty = self.uncertainty
+            result.uncertainty = self.uncertainty.__class__(self.uncertainty,
+                                                            copy=True)
         else:  # both self and operand have uncertainties
-            if (WARN_UNSUPPORTED_CORRELATED() and
+            if (conf.warn_unsupported_correlated and
                 (not self.uncertainty.support_correlated or
                  not operand.uncertainty.support_correlated)):
                 log.info("The uncertainty classes used do not support the "
                          "propagation of correlated errors, so uncertainties"
                          " will be propagated assuming they are uncorrelated")
+            operand_scaled = operand.__class__(operand_data,
+                                               uncertainty=operand_uncertainty,
+                                               unit=operand.unit,
+                                               wcs=operand.wcs,
+                                               mask=operand.mask,
+                                               flags=operand.flags,
+                                               meta=operand.meta)
             try:
                 method = getattr(self.uncertainty, propagate_uncertainties)
-                result.uncertainty = method(operand, result.data)
+                result.uncertainty = method(operand_scaled, result.data)
             except IncompatibleUncertaintiesException:
                 raise IncompatibleUncertaintiesException(
-                    "Cannot propagate uncertainties of type {0:s} with uncertainties of "
-                    "type {1:s} for {2:s}".format(
+                    "Cannot propagate uncertainties of type {0:s} with "
+                    "uncertainties of type {1:s} for {2:s}".format(
                         self.uncertainty.__class__.__name__,
                         operand.uncertainty.__class__.__name__,
                         name))
@@ -388,16 +521,11 @@ class NDData(object):
         if self.mask is None and operand.mask is None:
             result.mask = None
         elif self.mask is None:
-            result.mask = operand.mask
+            result.mask = operand.mask.copy()
         elif operand.mask is None:
-            result.mask = self.mask
+            result.mask = self.mask.copy()
         else:  # combine masks as for Numpy masked arrays
-            result.mask = self.mask & operand.mask
-
-        result.flags = None
-        result.wcs = self.wcs
-        result.meta = None
-        result.unit = self.unit
+            result.mask = self.mask | operand.mask  # copy implied by operator
 
         return result
 
@@ -460,18 +588,30 @@ class NDData(object):
         ------
         UnitsError
             If units are inconsistent.
+
+        Notes
+        -----
+        Flags are set to None in the result.
         """
         if self.unit is None:
             raise ValueError("No unit specified on source data")
         data = self.unit.to(unit, self.data, equivalencies=equivalencies)
-        result = self.__class__(data)  # in case we are dealing with an inherited type
-
-        result.uncertainty = self.uncertainty
-        result.mask = self.mask
-        result.flags = None
-        result.wcs = self.wcs
-        result.meta = self.meta
-        result.unit = unit
+        if self.uncertainty is not None:
+            uncertainty_values = self.unit.to(unit, self.uncertainty.array,
+                                              equivalencies=equivalencies)
+            # should work for any uncertainty class
+            uncertainty = self.uncertainty.__class__(uncertainty_values)
+        else:
+            uncertainty = None
+        if self.mask is not None:
+            new_mask = self.mask.copy()
+        else:
+            new_mask = None
+        # Call __class__ in case we are dealing with an inherited type
+        result = self.__class__(data, uncertainty=uncertainty,
+                                mask=new_mask, flags=self.flags,
+                                wcs=self.wcs,
+                                meta=self.meta, unit=unit)
 
         return result
 
