@@ -1,4 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+##############################################################################
+# Note: this file exists only for backward-compatibility purposes - the      #
+#       contents have been moved to the separate astropy-helpers package,    #
+#       located at https://github.com/astropy/astropy-helpers. Any new       #
+#       development or bug fixes should be done there.                       #
+##############################################################################
+
 """
 This module contains a number of utilities for use during
 setup/build/packaging that are useful to astropy as a whole.
@@ -22,12 +30,14 @@ import warnings
 from distutils import log, ccompiler, sysconfig
 from distutils.cmd import DistutilsOptionError
 from distutils.dist import Distribution
-from distutils.errors import DistutilsError, DistutilsFileError
+from distutils.errors import DistutilsError
 from distutils.core import Extension
 from distutils.core import Command
 from distutils.command.sdist import sdist as DistutilsSdist
 from setuptools.command.build_ext import build_ext as SetuptoolsBuildExt
 from setuptools.command.build_py import build_py as SetuptoolsBuildPy
+from setuptools.command.install import install as SetuptoolsInstall
+from setuptools.command.install_lib import install_lib as SetuptoolsInstallLib
 
 from setuptools.command.register import register as SetuptoolsRegister
 from setuptools import find_packages
@@ -38,6 +48,8 @@ from .utils.compat.misc import invalidate_caches
 from .utils.misc import walk_skip_hidden
 from .utils.exceptions import AstropyDeprecationWarning
 
+from .extern import six
+
 
 try:
     import Cython
@@ -362,30 +374,27 @@ def register_commands(package, version, release):
     _registered_commands = {
         'test': generate_test_command(package),
 
-         # Use distutils' sdist because it respects package_data.
-         # setuptools/distributes sdist requires duplication of information in
-         # MANIFEST.in
-         'sdist': DistutilsSdist,
+        # Use distutils' sdist because it respects package_data.
+        # setuptools/distributes sdist requires duplication of information in
+        # MANIFEST.in
+        'sdist': DistutilsSdist,
 
-         # The exact form of the build_ext command depends on whether or not
-         # we're building a release version
-         'build_ext': generate_build_ext_command(package, release),
+        # The exact form of the build_ext command depends on whether or not
+        # we're building a release version
+        'build_ext': generate_build_ext_command(package, release),
 
-         # We have a custom build_py to generate the default configuration file
-         'build_py': AstropyBuildPy,
+        # We have a custom build_py to generate the default configuration file
+        'build_py': AstropyBuildPy,
 
-         'register': AstropyRegister
+        # Since install can (in some circumstances) be run without
+        # first building, we also need to override install and
+        # install_lib.  See #2223
+        'install': AstropyInstall,
+        'install_lib': AstropyInstallLib,
+
+        'register': AstropyRegister
     }
 
-    try:
-        import bdist_mpkg
-    except ImportError:
-        pass
-    else:
-        # Use a custom command to build a dmg (on MacOS X)
-        _registered_commands['bdist_dmg'] = bdist_dmg
-
-
     if HAVE_SPHINX:
         _registered_commands['build_sphinx'] = AstropyBuildSphinx
     else:
@@ -520,19 +529,15 @@ def generate_build_ext_command(packagename, release):
 
             invalidate_caches()
 
+        # REMOVE: in astropy 0.5
         if not self.distribution.is_pure() and os.path.isdir(self.build_lib):
             # Finally, generate the default astropy.cfg; this can only be done
             # after extension modules are built as some extension modules
             # include config items.  We only do this if it's not pure python,
             # though, because if it is, we already did it in build_py
-            default_cfg = generate_default_config(
-                    os.path.abspath(self.build_lib),
-                    self.distribution.packages[0])
-            if default_cfg:
-                default_cfg = os.path.relpath(default_cfg)
-                self.copy_file(default_cfg,
-                               os.path.join(self.build_lib, default_cfg),
-                               preserve_mode=False)
+            generate_default_config(
+                os.path.abspath(self.build_lib),
+                self.distribution.packages[0], self)
 
     attrs['run'] = run
     attrs['finalize_options'] = finalize_options
@@ -542,6 +547,29 @@ def generate_build_ext_command(packagename, release):
     return type('build_ext', (basecls, object), attrs)
 
 
+def _get_platlib_dir(cmd):
+    plat_specifier = '.{0}-{1}'.format(cmd.plat_name, sys.version[0:3])
+    return os.path.join(cmd.build_base, 'lib' + plat_specifier)
+
+
+class AstropyInstall(SetuptoolsInstall):
+
+    def finalize_options(self):
+        build_cmd = self.get_finalized_command('build')
+        platlib_dir = _get_platlib_dir(build_cmd)
+        self.build_lib = platlib_dir
+        SetuptoolsInstall.finalize_options(self)
+
+
+class AstropyInstallLib(SetuptoolsInstallLib):
+
+    def finalize_options(self):
+        build_cmd = self.get_finalized_command('build')
+        platlib_dir = _get_platlib_dir(build_cmd)
+        self.build_dir = platlib_dir
+        SetuptoolsInstallLib.finalize_options(self)
+
+
 class AstropyBuildPy(SetuptoolsBuildPy):
 
     def finalize_options(self):
@@ -550,22 +578,12 @@ class AstropyBuildPy(SetuptoolsBuildPy):
         # for projects with only pure-Python source (this is desirable
         # specifically for support of multiple Python version).
         build_cmd = self.get_finalized_command('build')
-        plat_specifier = '.{0}-{1}'.format(build_cmd.plat_name,
-                                           sys.version[0:3])
-        # Do this unconditionally
-        build_purelib = os.path.join(build_cmd.build_base,
-                                     'lib' + plat_specifier)
-        build_cmd.build_purelib = build_purelib
-        build_cmd.build_lib = build_purelib
+        platlib_dir = _get_platlib_dir(build_cmd)
+
+        build_cmd.build_purelib = platlib_dir
+        build_cmd.build_lib = platlib_dir
+        self.build_lib = platlib_dir
 
-        # Ugly hack: We also need to 'fix' the build_lib option on the
-        # install command--it would be better just to override that command
-        # entirely, but we can get around that extra effort by doing it here
-        install_cmd = self.get_finalized_command('install')
-        install_cmd.build_lib = build_purelib
-        install_lib_cmd = self.get_finalized_command('install_lib')
-        install_lib_cmd.build_dir = build_purelib
-        self.build_lib = build_purelib
         SetuptoolsBuildPy.finalize_options(self)
 
     def run_2to3(self, files, doctests=False):
@@ -585,27 +603,45 @@ class AstropyBuildPy(SetuptoolsBuildPy):
         # first run the normal build_py
         SetuptoolsBuildPy.run(self)
 
+        pkgnm = self.distribution.packages[0]
+
+        # REMOVE: in astropy 0.5
         if self.distribution.is_pure():
             # Generate the default astropy.cfg - we only do this here if it's
             # pure python.  Otherwise, it'll happen at the end of build_exp
-            default_cfg = generate_default_config(
-                    os.path.abspath(self.build_lib),
-                    self.distribution.packages[0])
-            if default_cfg:
-                default_cfg = os.path.relpath(default_cfg)
-                self.copy_file(default_cfg,
-                               os.path.join(self.build_lib, default_cfg),
-                               preserve_mode=False)
+            generate_default_config(
+                os.path.abspath(self.build_lib),
+                pkgnm, self)
+
+        # Also copy over *only* the pytest part of setup.cfg.  This is necessary
+        # because the astropy.test() function needs this information to know how
+        # it should be configured.
+        p = six.moves.configparser.SafeConfigParser()
+        p.add_section('pytest')
+
+        pkgnmsection = r'"' + pkgnm + r'[\/]'
+        for k, (fnsrc, v) in six.iteritems(self.distribution.command_options['pytest']):
+            # also need to adjust any part of the pytest section that contains
+            # reference to the package directory (e.g., "astropy" for the core
+            # package), because this file is actually written *into* that
+            # directory.
+            if pkgnmsection in v:
+                v = v.replace(pkgnmsection, '"')
+            p.set('pytest', k, v)
+
+        with open(os.path.join(self.build_lib, pkgnm, 'pytest.ini'), 'w') as f:
+            p.write(f)
 
 
-def generate_default_config(build_lib, package):
+# REMOVE: in astropy 0.5
+def generate_default_config(build_lib, package, command):
     config_path = os.path.relpath(package)
     filename = os.path.join(config_path, package + '.cfg')
 
     if os.path.exists(filename):
-        log.info('regenerating default {0}.cfg file'.format(package))
-    else:
-        log.info('generating default {0}.cfg file'.format(package))
+        return
+
+    log.info('generating default {0}.cfg file in {1}'.format(package, filename))
 
     if PY3:
         builtins = 'builtins'
@@ -636,7 +672,10 @@ def generate_default_config(build_lib, package):
     stdout, stderr = proc.communicate()
 
     if proc.returncode == 0 and os.path.exists(filename):
-        return filename
+        default_cfg = os.path.relpath(filename)
+        command.copy_file(default_cfg,
+                          os.path.join(command.build_lib, default_cfg),
+                          preserve_mode=False)
     else:
         msg = ('Generation of default configuration item failed! Stdout '
                'and stderr are shown below.\n'
@@ -1054,6 +1093,9 @@ def get_package_info(srcdir):
     package_dir = {}
     skip_2to3 = []
 
+    # Add the package's .cfg file
+    package_data[''] = [srcdir + ".cfg"]
+
     # Use the find_packages tool to locate all packages and modules
     packages = filter_packages(find_packages())
 
@@ -1263,11 +1305,10 @@ def get_numpy_include_path():
     # install, since Numpy may still think it's in "setup mode", when
     # in fact we're ready to use it to build astropy now.
 
-    if sys.version_info[0] >= 3:
+    if PY3:
         import builtins
         if hasattr(builtins, '__NUMPY_SETUP__'):
             del builtins.__NUMPY_SETUP__
-        import imp
         import numpy
         imp.reload(numpy)
     else:
@@ -1457,132 +1498,6 @@ def filter_packages(packagenames):
     return [x for x in packagenames if not x.endswith(exclude)]
 
 
-class bdist_dmg(Command):
-    """
-    The bdist_dmg command is used to produce the disk image containing the
-    installer, and with a custom background and icon placement.
-    """
-
-    user_options = [
-                    ('background=', 'b', "background image to use (should be 500x500px)"),
-                    ('dist-dir=', 'd', "directory to put final built distributions in")
-                   ]
-    description = "Create a Mac OS X disk image with the package installer"
-
-    def initialize_options(self):
-        self.dist_dir = None
-        self.background = None
-        self.finalized = False
-
-    def finalize_options(self):
-        self.set_undefined_options('bdist', ('dist_dir', 'dist_dir'))
-        self.finalized = True
-
-    def run(self):
-
-        pkg_dir = os.path.join(self.dist_dir, 'pkg')
-
-        # Remove directory if it already exists
-        if os.path.exists(pkg_dir):
-            shutil.rmtree(pkg_dir)
-
-        # First create the package installer with bdist_mpkg
-        mpkg = self.reinitialize_command('bdist_mpkg', reinit_subcommands=1)
-        mpkg.dist_dir = pkg_dir
-        mpkg.ensure_finalized()
-        mpkg.run()
-
-        # Find the name of the pkg file. Since we removed the dist directory
-        # at the start of the script, our pkg should be the only file there.
-        files = os.listdir(pkg_dir)
-        if len(files) != 1:
-            raise DistutilsFileError(
-                "Expected a single file in the {pkg_dir} "
-                "directory".format(pkg_dir=pkg_dir))
-        pkg_file = os.path.basename(files[0])
-        pkg_name = os.path.splitext(pkg_file)[0]
-
-        # Build the docs
-        docs = self.reinitialize_command('build_sphinx', reinit_subcommands=1)
-        docs.ensure_finalized()
-        docs.run()
-
-        # Copy over the docs to the dist directory
-        shutil.copytree(os.path.join(docs.build_dir, 'html'),
-                        os.path.join(pkg_dir, 'Documentation'))
-
-        # Copy over the background to the disk image
-        if self.background is not None:
-            background_dir = os.path.join(pkg_dir, '.background')
-            os.mkdir(background_dir)
-            shutil.copy2(self.background,
-                         os.path.join(background_dir, 'background.png'))
-
-        # Start creating the volume
-        dmg_path = os.path.join(self.dist_dir, pkg_name + '.dmg')
-        dmg_path_tmp = os.path.join(self.dist_dir, pkg_name + '_tmp.dmg')
-        volume_name = pkg_name
-
-        # Remove existing dmg files
-        if os.path.exists(dmg_path):
-            os.remove(dmg_path)
-        if os.path.exists(dmg_path_tmp):
-            os.remove(dmg_path_tmp)
-
-        # Check if a volume is already mounted
-        volume_path = os.path.join('/', 'Volumes', volume_name)
-        if os.path.exists(volume_path):
-            raise DistutilsFileError(
-                "A volume named {volume_name} is already mounted - please "
-                "eject this and try again".format(volume_name=volume_name))
-
-        shell_script = """
-
-        # Create DMG file
-        hdiutil create -volname {volume_name} -srcdir {pkg_dir} -fs HFS+ -fsargs "-c c=64,a=16,e=16" -format UDRW -size 24m {dmg_path_tmp}
-
-        # Mount disk image, and keep reference to device
-        device=$(hdiutil attach -readwrite -noverify -noautoopen {dmg_path_tmp} | egrep '^/dev/' | sed 1q | awk '{{print $1}}')
-
-        echo '
-        tell application "Finder"
-            tell disk "{volume_name}"
-                open
-                set current view of container window to icon view
-                set toolbar visible of container window to false
-                set statusbar visible of container window to false
-                set the bounds of container window to {{100, 100, 600, 600}}
-                set theViewOptions to the icon view options of container window
-                set arrangement of theViewOptions to not arranged
-                set icon size of theViewOptions to 128
-                set the background picture of theViewOptions to file ".background:background.png"
-                set position of item "{pkg_file}" of container window to {{125, 320}}
-                set position of item "Documentation" of container window to {{375, 320}}
-                close
-                open
-                update without registering applications
-                delay 5
-            end tell
-        end tell
-        ' | osascript
-
-        # Eject disk image
-        hdiutil detach ${{device}}
-
-        # Convert to final read-only disk image
-        hdiutil convert {dmg_path_tmp} -format UDZO -imagekey zlib-level=9 -o {dmg_path}
-
-        """.format(volume_name=volume_name, pkg_dir=pkg_dir,
-                   pkg_file=pkg_file, dmg_path_tmp=dmg_path_tmp,
-                   dmg_path=dmg_path)
-
-        # Make the disk image with the above shell script
-        os.system(shell_script)
-
-        # Remove temporary disk image
-        os.remove(dmg_path_tmp)
-
-
 class FakeBuildSphinx(Command):
     """
     A dummy build_sphinx command that is called if Sphinx is not
