@@ -1,21 +1,19 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+# TEST_UNICODE_LITERALS
+
 import operator
 
 from distutils import version
 
 import numpy as np
 
-from ...tests.helper import pytest, catch_warnings
+from ...tests.helper import pytest, catch_warnings, assert_follows_unicode_guidelines
 from ...utils.exceptions import AstropyDeprecationWarning
 from ... import table
 from ... import units as u
 
-@pytest.fixture(params=[table.Column, table.MaskedColumn])
-def Column(request):
-    # Fixture to run all the Column tests for both an unmasked (ndarray)
-    # and masked (MaskedArray) column.
-    return request.param
+NUMPY_LT_1P8 = [int(x) for x in np.__version__.split('.')[:2]] < [1, 8]
 
 
 class TestColumn():
@@ -55,6 +53,27 @@ class TestColumn():
         lt = c - 1 < arr
         assert np.all(lt)
 
+    def test_numpy_boolean_ufuncs(self, Column):
+        """Show that basic numpy operations with Column behave sensibly"""
+
+        arr = np.array([1, 2, 3])
+        c = Column(arr, name='a')
+
+        for ufunc, test_true in ((np.isfinite, True),
+                                 (np.isinf, False),
+                                 (np.isnan, False),
+                                 (np.sign, True),
+                                 (np.signbit, False)):
+            result = ufunc(c)
+            assert len(result) == len(c)
+            assert np.all(result) if test_true else not np.any(result)
+            if Column is table.Column:
+                assert type(result) == np.ndarray
+            else:
+                assert type(result) == np.ma.core.MaskedArray
+                if ufunc is not np.sign:
+                    assert result.dtype.str == '|b1'
+
     def test_view(self, Column):
         c = np.array([1, 2, 3]).view(Column)
         if Column == table.MaskedColumn:
@@ -68,13 +87,12 @@ class TestColumn():
 
     def test_format(self, Column):
         """Show that the formatted output from str() works"""
-        MAX_LINES_val = table.pprint.MAX_LINES()
-        table.pprint.MAX_LINES.set(7)
-        c1 = Column(np.arange(2000), name='a', dtype=float,
-                    format='%6.2f')
-        assert str(c1) == ('   a   \n-------\n   0.00\n'
-                           '   1.00\n    ...\n1998.00\n1999.00')
-        table.pprint.MAX_LINES.set(MAX_LINES_val)
+        from ... import conf
+        with conf.set_temp('max_lines', 7):
+            c1 = Column(np.arange(2000), name='a', dtype=float,
+                        format='%6.2f')
+            assert str(c1) == ('   a   \n-------\n   0.00\n'
+                               '   1.00\n    ...\n1998.00\n1999.00')
 
     def test_convert_numpy_array(self, Column):
         d = Column([1, 2, 3], name='a', dtype='i8')
@@ -91,21 +109,6 @@ class TestColumn():
         d.convert_unit_to("km")
         assert np.all(d.data == [0.001, 0.002, 0.003])
 
-    def test_deprecated_attributes(self, Column, recwarn):
-        d = Column([1, 2, 3], name='a', dtype="f8", unit="m")
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            d.units
-            assert warning_lines[0].category == AstropyDeprecationWarning
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            c = Column([1,2,3], name='a', dtypes="f8", unit="m")
-            assert warning_lines[0].category == AstropyDeprecationWarning
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            c = Column([1,2,3], name='a', dtype="f8", units="m")
-            assert warning_lines[0].category == AstropyDeprecationWarning
-
     def test_array_wrap(self):
         """Test that the __array_wrap__ method converts a reduction ufunc
         output that has a different shape into an ndarray view.  Without this a
@@ -245,3 +248,76 @@ class TestMetaColumn(MetaBaseTest):
 class TestMetaMaskedColumn(MetaBaseTest):
     test_class = table.MaskedColumn
     args = ()
+
+
+def test_getitem_metadata_regression():
+    """
+    Regression test for #1471: MaskedArray does not call __array_finalize__ so
+    the meta-data was not getting copied over. By overloading _update_from we
+    are able to work around this bug.
+    """
+
+    # Make sure that meta-data gets propagated with __getitem__
+
+    c = table.Column(data=[1,2], name='a', description='b', unit='m', format="%i", meta={'c': 8})
+    assert c[1:2].name == 'a'
+    assert c[1:2].description == 'b'
+    assert c[1:2].unit == 'm'
+    assert c[1:2].format == '%i'
+    assert c[1:2].meta['c'] == 8
+
+    c = table.MaskedColumn(data=[1,2], name='a', description='b', unit='m', format="%i", meta={'c': 8})
+    assert c[1:2].name == 'a'
+    assert c[1:2].description == 'b'
+    assert c[1:2].unit == 'm'
+    assert c[1:2].format == '%i'
+    assert c[1:2].meta['c'] == 8
+
+    # As above, but with take() - check the method and the function
+
+    c = table.Column(data=[1,2,3], name='a', description='b', unit='m', format="%i", meta={'c': 8})
+    for subset in [c.take([0, 1]), np.take(c, [0, 1])]:
+        assert subset.name == 'a'
+        assert subset.description == 'b'
+        assert subset.unit == 'm'
+        assert subset.format == '%i'
+        assert subset.meta['c'] == 8
+
+    # Metadata isn't copied for scalar values
+    if NUMPY_LT_1P8:
+        with pytest.raises(ValueError):
+            c.take(0)
+        with pytest.raises(ValueError):
+            np.take(c, 0)
+    else:
+        for subset in [c.take(0), np.take(c, 0)]:
+            assert subset == 1
+            assert subset.shape == ()
+            assert not isinstance(subset, table.Column)
+
+    c = table.MaskedColumn(data=[1,2,3], name='a', description='b', unit='m', format="%i", meta={'c': 8})
+    for subset in [c.take([0, 1]), np.take(c, [0, 1])]:
+        assert subset.name == 'a'
+        assert subset.description == 'b'
+        assert subset.unit == 'm'
+        assert subset.format == '%i'
+        assert subset.meta['c'] == 8
+
+    # Metadata isn't copied for scalar values
+    if NUMPY_LT_1P8:
+        with pytest.raises(ValueError):
+            c.take(0)
+        with pytest.raises(ValueError):
+            np.take(c, 0)
+    else:
+        for subset in [c.take(0), np.take(c, 0)]:
+            assert subset == 1
+            assert subset.shape == ()
+            assert not isinstance(subset, table.MaskedColumn)
+
+
+def test_unicode_guidelines():
+    arr = np.array([1, 2, 3])
+    c = table.Column(arr, name='a')
+
+    assert_follows_unicode_guidelines(c)
