@@ -1,4 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+# TEST_UNICODE_LITERALS
+
 from __future__ import print_function  # For print debugging with python 2 or 3
 
 from distutils import version
@@ -7,24 +10,43 @@ import numpy as np
 
 from ...tests.helper import pytest
 from ... import table
-from ...table import Column
+from ...table import Column, TableColumns
 from ...utils import OrderedDict
 
 
-class MaskedTable(table.Table):
-    def __init__(self, *args, **kwargs):
-        kwargs['masked'] = True
-        table.Table.__init__(self, *args, **kwargs)
+class TestTableColumnsInit():
+    def test_init(self):
+        """Test initialisation with lists, tuples, dicts of arrays
+        rather than Columns [regression test for #2647]"""
+        x1 = np.arange(10.)
+        x2 = np.arange(5.)
+        x3 = np.arange(7.)
+        col_list = [('x1',x1), ('x2',x2), ('x3',x3)]
+        tc_list = TableColumns(col_list)
+        for col in col_list:
+            assert col[0] in tc_list
+            assert tc_list[col[0]] is col[1]
+
+        col_tuple = (('x1',x1), ('x2',x2), ('x3',x3))
+        tc_tuple = TableColumns(col_tuple)
+        for col in col_tuple:
+            assert col[0] in tc_tuple
+            assert tc_tuple[col[0]] is col[1]
+
+        col_dict = dict([('x1',x1), ('x2',x2), ('x3',x3)])
+        tc_dict = TableColumns(col_dict)
+        for col in tc_dict.keys():
+            assert col in tc_dict
+            assert tc_dict[col] is col_dict[col]
+
+        columns = [Column(col[1], name=col[0]) for col in col_list]
+        tc = TableColumns(columns)
+        for col in columns:
+            assert col.name in tc
+            assert tc[col.name] is col
 
 
-# Fixture to run all the Column tests for both an unmasked (ndarray)
-# and masked (MaskedArray) column.
-@pytest.fixture(params=[False, True])
-def table_type(request):
-    return MaskedTable if request.param else table.Table
-
-
-@pytest.mark.usefixtures('table_type')
+#pytest.mark.usefixtures('table_type')
 class BaseInitFrom():
     def _setup(self, table_type):
         pass
@@ -38,7 +60,7 @@ class BaseInitFrom():
         assert np.all(t['c'] == np.array([3, 5]))
         assert all(t[name].name == name for name in t.colnames)
 
-    def test_set_dtypes(self, table_type):
+    def test_set_dtype(self, table_type):
         self._setup(table_type)
         t = table_type(self.data, names=('a', 'b', 'c'), dtype=('i4', 'f4', 'f8'))
         assert t.colnames == ['a', 'b', 'c']
@@ -50,7 +72,7 @@ class BaseInitFrom():
         assert t['c'].dtype.type == np.float64
         assert all(t[name].name == name for name in t.colnames)
 
-    def test_names_dtypes_mismatch(self, table_type):
+    def test_names_dtype_mismatch(self, table_type):
         self._setup(table_type)
         with pytest.raises(ValueError):
             table_type(self.data, names=('a',), dtype=('i4', 'f4', 'i4'))
@@ -104,7 +126,7 @@ class TestInitFromNdarrayHomo(BaseInitFromListLike):
         # NOTE: assert np.all(t._data == self.data) fails because when
         # homogenous array is viewcast to structured then the == is False
 
-    def test_partial_names_dtypes(self, table_type):
+    def test_partial_names_dtype(self, table_type):
         self._setup(table_type)
         t = table_type(self.data, names=['a', None, 'c'], dtype=[None, None, 'f8'])
         assert t.colnames == ['a', 'col1', 'c']
@@ -126,7 +148,7 @@ class TestInitFromNdarrayHomo(BaseInitFromListLike):
 @pytest.mark.usefixtures('table_type')
 class TestInitFromListOfLists(BaseInitFromListLike):
 
-    def setup_method(self, method):
+    def setup_method(self, table_type):
         self._setup(table_type)
         self.data = [(np.int32(1), np.int32(3)),
                      Column(name='col1', data=[2, 4], dtype=np.int32),
@@ -138,7 +160,7 @@ class TestInitFromListOfLists(BaseInitFromListLike):
         assert t.colnames == ['col0', 'col1', 'col2']
         assert all(t[name].name == name for name in t.colnames)
 
-    def test_partial_names_dtypes(self, table_type):
+    def test_partial_names_dtype(self, table_type):
         self._setup(table_type)
         t = table_type(self.data, names=['b', None, 'c'],
                   dtype=['f4', None, 'f8'])
@@ -193,7 +215,7 @@ class TestInitFromColsList(BaseInitFromListLike):
         assert t.colnames == ['x', 'col1', 'col2']
         assert all(t[name].name == name for name in t.colnames)
 
-    def test_partial_names_dtypes(self, table_type):
+    def test_partial_names_dtype(self, table_type):
         self._setup(table_type)
         t = table_type(self.data, names=['b', None, 'c'], dtype=['f4', None, 'f8'])
         assert t.colnames == ['b', 'col1', 'c']
@@ -214,7 +236,7 @@ class TestInitFromNdarrayStruct(BaseInitFromDictLike):
     def _setup(self, table_type):
         self.data = np.array([(1, 2, 3),
                               (3, 4, 5)],
-                             dtype=[('x', 'i8'), ('y', 'i4'), ('z', 'i8')])
+                             dtype=[(str('x'), 'i8'), (str('y'), 'i4'), (str('z'), 'i8')])
 
     def test_ndarray_ref(self, table_type):
         """Init with ndarray and copy=False and show that table uses reference
@@ -228,7 +250,7 @@ class TestInitFromNdarrayStruct(BaseInitFromDictLike):
         assert np.all(t._data == self.data)
         assert all(t[name].name == name for name in t.colnames)
 
-    def test_partial_names_dtypes(self, table_type):
+    def test_partial_names_dtype(self, table_type):
         self._setup(table_type)
         t = table_type(self.data, names=['e', None, 'd'], dtype=['f4', None, 'f8'])
         assert t.colnames == ['e', 'y', 'd']
@@ -276,7 +298,7 @@ class TestInitFromRow(BaseInitFromDictLike):
     def _setup(self, table_type):
         arr = np.array([(1, 2, 3),
                         (3, 4, 5)],
-                       dtype=[('x', 'i8'), ('y', 'i8'), ('z', 'f8')])
+                       dtype=[(str('x'), 'i8'), (str('y'), 'i8'), (str('z'), 'f8')])
         self.data = table_type(arr, meta={'comments': ['comment1', 'comment2']})
 
     def test_init_from_row(self, table_type):
@@ -302,7 +324,7 @@ class TestInitFromTable(BaseInitFromDictLike):
     def _setup(self, table_type):
         arr = np.array([(1, 2, 3),
                         (3, 4, 5)],
-                       dtype=[('x', 'i8'), ('y', 'i8'), ('z', 'f8')])
+                       dtype=[(str('x'), 'i8'), (str('y'), 'i8'), (str('z'), 'f8')])
         self.data = table_type(arr, meta={'comments': ['comment1', 'comment2']})
 
     def test_data_meta_copy(self, table_type):
@@ -327,7 +349,7 @@ class TestInitFromTable(BaseInitFromDictLike):
         assert np.all(t._data == self.data._data)
         assert all(t[name].name == name for name in t.colnames)
 
-    def test_partial_names_dtypes(self, table_type):
+    def test_partial_names_dtype(self, table_type):
         self._setup(table_type)
         t = table_type(self.data, names=['e', None, 'd'], dtype=['f4', None, 'i8'])
         assert t.colnames == ['e', 'y', 'd']
@@ -381,3 +403,33 @@ class TestInitFromNone():
         assert t['a'].dtype.type == np.float32
         assert t['b'].dtype.type == np.int32
         assert t.colnames == ['a', 'b']
+
+
+@pytest.mark.usefixtures('table_types')
+class TestInitFromRows():
+
+    def test_init_with_rows(self, table_type):
+        for rows in ([[1, 'a'], [2, 'b']],
+                     [(1, 'a'), (2, 'b')],
+                     ((1, 'a'), (2, 'b'))):
+            t = table_type(rows=rows, names=('a', 'b'))
+            assert np.all(t['a'] == [1, 2])
+            assert np.all(t['b'] == ['a', 'b'])
+            assert t.colnames == ['a', 'b']
+            assert t['a'].dtype.kind == 'i'
+            assert t['b'].dtype.kind in ('S', 'U')
+
+        rows = np.arange(6).reshape(2, 3)
+        t = table_type(rows=rows, names=('a', 'b', 'c'), dtype=['f8', 'f4', 'i8'])
+        assert np.all(t['a'] == [0, 3])
+        assert np.all(t['b'] == [1, 4])
+        assert np.all(t['c'] == [2, 5])
+        assert t.colnames == ['a', 'b', 'c']
+        assert t['a'].dtype.str.endswith('f8')
+        assert t['b'].dtype.str.endswith('f4')
+        assert t['c'].dtype.str.endswith('i8')
+
+    def test_init_with_rows_and_data(self, table_type):
+        with pytest.raises(ValueError) as err:
+            table_type(data=[[1]], rows=[[1]])
+        assert "Cannot supply both `data` and `rows` values" in str(err)
