@@ -1,4 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+# TEST_UNICODE_LITERALS
+
 """Test behavior related to masked tables"""
 
 from distutils import version
@@ -8,12 +11,6 @@ import numpy.ma as ma
 from ...tests.helper import pytest
 from ...table import Column, MaskedColumn, Table
 
-class SubclassTable(Table):
-    pass
-
-@pytest.fixture(params = [True, False])
-def tableclass(request):
-    return Table if request.param else SubclassTable
 
 class SetupData(object):
     def setup_method(self, method):
@@ -166,10 +163,10 @@ class TestMaskedColumnInit(SetupData):
         assert np.all(b.mask == mask_list)
 
     def test_incomplete_mask_spec(self):
-        """Incomplete mask specification (mask values cycle through available)"""
+        """Incomplete mask specification raises MaskError"""
         mask_list = [False, True]
-        b = MaskedColumn(name='b', length=4, mask=mask_list)
-        assert np.all(b.mask == mask_list + mask_list)
+        with pytest.raises(ma.MaskError):
+            MaskedColumn(name='b', length=4, mask=mask_list)
 
 
 class TestTableInit(SetupData):
@@ -184,6 +181,16 @@ class TestTableInit(SetupData):
         t = Table([self.ca, ma.array([1, 2, 3])])
         assert t.masked is True
 
+    def test_mask_false_if_no_input_masked(self):
+        """Masking not true if not (requested or input requires mask)"""
+        t0 = Table([[3,4]], masked=False)
+        t1 = Table(t0, masked=True)
+        t2 = Table(t1, masked=False)
+        assert not t0.masked
+        assert t1.masked
+        assert not t2.masked
+
+
     def test_mask_property(self):
         t = self.t
         # Access table mask (boolean structured array) by column name
@@ -240,6 +247,17 @@ class TestAddColumn(object):
         assert np.all(t['b'] == np.array([4, 5, 6]))
         assert np.all(t['b'].mask == np.array([1, 0, 1], bool))
 
+    def test_convert_to_masked_table_only_if_necessary(self):
+        # Do not convert to masked table, if new column has no masked value.
+        # See #1185 for details.
+        t = Table(masked=False)
+        assert not t.masked
+        t.add_column(Column(name='a', data=[1, 2, 3]))
+        assert not t.masked
+        t.add_column(MaskedColumn(name='b', data=[4, 5, 6], mask=[0, 0, 0]))
+        assert not t.masked
+        assert np.all(t['a'] == np.array([1, 2, 3]))
+        assert np.all(t['b'] == np.array([4, 5, 6]))
 
 class TestRenameColumn(object):
 
