@@ -1,4 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+# TEST_UNICODE_LITERALS
+
 import sys
 
 from distutils import version
@@ -8,22 +11,17 @@ from ...tests.helper import pytest, catch_warnings
 from ... import table
 from ...table import Row
 from ...utils.exceptions import AstropyDeprecationWarning
+from .conftest import MaskedTable
 
 numpy_lt_1p8 = version.LooseVersion(np.__version__) < version.LooseVersion('1.8')
 
 
-class MaskedTable(table.Table):
-    def __init__(self, *args, **kwargs):
-        kwargs['masked'] = True
-        table.Table.__init__(self, *args, **kwargs)
-
-
 def test_masked_row_with_object_col():
     """
     Numpy < 1.8 has a bug in masked array that prevents access a row if there is
     a column with object type.
     """
-    t = table.Table([[1]], dtypes=['O'], masked=True)
+    t = table.Table([[1]], dtype=['O'], masked=True)
     if numpy_lt_1p8:
         with pytest.raises(ValueError):
             t['col0'].mask = False
@@ -38,16 +36,6 @@ def test_masked_row_with_object_col():
         assert t[0]['col0'] is np.ma.masked
 
 
-# Fixture to run all tests for both an unmasked (ndarray) and masked (MaskedArray) column.
-@pytest.fixture(params=[False, True])
-def table_types(request):
-    class TableTypes:
-        def __init__(self, request):
-            self.Table = MaskedTable if request.param else table.Table
-            self.Column = table.MaskedColumn if request.param else table.Column
-    return TableTypes(request)
-
-
 @pytest.mark.usefixtures('table_types')
 class TestRow():
     def _setup(self, table_types):
@@ -152,19 +140,11 @@ class TestRow():
         assert d.colnames == list(np_data.dtype.names)
 
         with pytest.raises(ValueError):
-            np_data = np.array(d, dtype=[('c', 'i8'), ('d', 'i8')])
+            np_data = np.array(d, dtype=[(str('c'), 'i8'), (str('d'), 'i8')])
 
     def test_format_row(self, table_types):
         """Test formatting row"""
         self._setup(table_types)
         table = self.t
         row = table[0]
-        assert format(row, "").startswith("<Row 0 of table")
-
-    def test_deprecated_attributes(self, table_types):
-        self._setup(table_types)
-        r = Row(self.t, 2)
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            r.dtypes
-            assert warning_lines[0].category == AstropyDeprecationWarning
+        assert format(row, "").startswith("<{0} 0 of table".format(row.__class__.__name__))
