@@ -1,6 +1,10 @@
+# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+# TEST_UNICODE_LITERALS
+
 import copy
+import gc
 import platform
 import sys
 
@@ -9,34 +13,10 @@ from distutils import version
 import numpy as np
 
 from ...extern import six
-from ...tests.helper import pytest
+from ...tests.helper import pytest, assert_follows_unicode_guidelines
 from ... import table
 from ... import units as u
-
-
-# In Python 3, prior to Numpy 1.6.2, there was a bug (in Numpy) that caused
-# sorting of structured arrays to silently fail under certain circumstances (for
-# example if the Table contains string columns) on MacOS X and Windows
-NUMPY_VERSION = version.LooseVersion(np.__version__)
-SKIP_STRING_SORT = (platform.system() in ('Darwin', 'Windows') and six.PY3 and
-                    NUMPY_VERSION < version.LooseVersion('1.6.2'))
-
-
-class MaskedTable(table.Table):
-    def __init__(self, *args, **kwargs):
-        kwargs['masked'] = True
-        table.Table.__init__(self, *args, **kwargs)
-
-
-# Fixture to run all the Column tests for both an unmasked (ndarray)
-# and masked (MaskedArray) column.
-@pytest.fixture(params=[False, True])
-def table_types(request):
-    class TableTypes:
-        def __init__(self, request):
-            self.Table = MaskedTable if request.param else table.Table
-            self.Column = table.MaskedColumn if request.param else table.Column
-    return TableTypes(request)
+from .conftest import MaskedTable
 
 
 class SetupData(object):
@@ -49,7 +29,7 @@ class SetupData(object):
         if self._column_type is not None:
             if not hasattr(self, '_a'):
                 self._a = self._column_type(
-                    name='a', data=[1, 2, 3], format='%d',
+                    [1, 2, 3], name='a', format='%d',
                     meta={'aa': [0, 1, 2, 3, 4]})
             return self._a
 
@@ -58,28 +38,28 @@ class SetupData(object):
         if self._column_type is not None:
             if not hasattr(self, '_b'):
                 self._b = self._column_type(
-                    name='b', data=[4, 5, 6], format='%d', meta={'aa': 1})
+                    [4, 5, 6], name='b', format='%d', meta={'aa': 1})
             return self._b
 
     @property
     def c(self):
         if self._column_type is not None:
             if not hasattr(self, '_c'):
-                self._c = self._column_type(name='c', data=[7, 8, 9])
+                self._c = self._column_type([7, 8, 9], 'c')
             return self._c
 
     @property
     def d(self):
         if self._column_type is not None:
             if not hasattr(self, '_d'):
-                self._d = self._column_type(name='d', data=[7, 8, 7])
+                self._d = self._column_type([7, 8, 7], 'd')
             return self._d
 
     @property
     def obj(self):
         if self._column_type is not None:
             if not hasattr(self, '_obj'):
-                self._obj = self._column_type(name='obj', data=[1, 'string', 3], dtype='O')
+                self._obj = self._column_type([1, 'string', 3], 'obj', dtype='O')
             return self._obj
 
     @property
@@ -730,6 +710,15 @@ class TestRemove(SetupData):
         assert self.t._data.dtype.names == ('b',)
         assert np.all(self.t['b'] == np.array([4, 5, 6]))
 
+    def test_3(self, table_types):
+        """Check remove_columns works for a single column with a name of
+        more than one character.  Regression test against #2699"""
+        self._setup(table_types)
+        self.t['new_column'] = self.t['a']
+        assert 'new_column' in self.t.columns.keys()
+        self.t.remove_columns('new_column')
+        assert 'new_column' not in self.t.columns.keys()
+
     def test_remove_nonexistent_row(self, table_types):
         self._setup(table_types)
         with pytest.raises(IndexError):
@@ -780,8 +769,8 @@ class TestRemove(SetupData):
         self.t.add_column(self.b)
         self.t.remove_rows([0, 2])
         assert self.t['a'].meta == {'aa': [0, 1, 2, 3, 4]}
-        assert self.t.dtype == np.dtype([('a', 'int'),
-                                         ('b', 'int')])
+        assert self.t.dtype == np.dtype([(str('a'), 'int'),
+                                         (str('b'), 'int')])
 
     def test_delitem1(self, table_types):
         self._setup(table_types)
@@ -897,18 +886,32 @@ class TestSort():
         assert np.all(t['a'] == np.array([2, 1, 3, 1, 3, 2]))
         assert np.all(t['b'] == np.array([3, 4, 4, 5, 5, 6]))
 
-    @pytest.mark.skipif('SKIP_STRING_SORT')
-    def test_multiple_with_strings(self, table_types):
-        # Before Numpy 1.6.2, and on Python 3.x, Numpy had a bug that meant
-        # that sorting with multiple column names failed when a string column
-        # was present:
+    def test_multiple_with_bytes(self, table_types):
         t = table_types.Table()
-        t.add_column(table_types.Column(name='firstname', data=["Max", "Jo", "John"]))
-        t.add_column(table_types.Column(name='name', data=["Miller", "Miller", "Jackson"]))
+        t.add_column(table_types.Column(name='firstname', data=[b"Max", b"Jo", b"John"]))
+        t.add_column(table_types.Column(name='name', data=[b"Miller", b"Miller", b"Jackson"]))
         t.add_column(table_types.Column(name='tel', data=[12, 15, 19]))
         t.sort(['name','firstname'])
-        assert np.all([t['firstname'] == np.array(["John", "Jo", "Max"])])
-        assert np.all([t['name'] == np.array(["Jackson", "Miller", "Miller"])])
+        assert np.all([t['firstname'] == np.array([b"John", b"Jo", b"Max"])])
+        assert np.all([t['name'] == np.array([b"Jackson", b"Miller", b"Miller"])])
+        assert np.all([t['tel'] == np.array([19, 15, 12])])
+
+    def test_multiple_with_unicode(self, table_types):
+        # Before Numpy 1.6.2, sorting with multiple column names
+        # failed when a unicode column was present.
+        t = table_types.Table()
+        t.add_column(table_types.Column(
+            name='firstname',
+            data=[six.text_type(x) for x in ["Max", "Jo", "John"]]))
+        t.add_column(table_types.Column(
+            name='name',
+            data=[six.text_type(x) for x in ["Miller", "Miller", "Jackson"]]))
+        t.add_column(table_types.Column(name='tel', data=[12, 15, 19]))
+        t.sort(['name','firstname'])
+        assert np.all([t['firstname'] == np.array(
+            [six.text_type(x) for x in ["John", "Jo", "Max"]])])
+        assert np.all([t['name'] == np.array(
+            [six.text_type(x) for x in ["Jackson", "Miller", "Miller"]])])
         assert np.all([t['tel'] == np.array([19, 15, 12])])
 
     def test_argsort(self, table_types):
@@ -919,6 +922,51 @@ class TestSort():
         assert np.all(t.argsort('a') == t._data.argsort(order=['a']))
         assert np.all(t.argsort(['a', 'b']) == t._data.argsort(order=['a', 'b']))
 
+    def test_argsort_bytes(self, table_types):
+        t = table_types.Table()
+        t.add_column(table_types.Column(name='firstname', data=[b"Max", b"Jo", b"John"]))
+        t.add_column(table_types.Column(name='name', data=[b"Miller", b"Miller", b"Jackson"]))
+        t.add_column(table_types.Column(name='tel', data=[12, 15, 19]))
+        assert np.all(t.argsort(['name', 'firstname']) == np.array([2, 1, 0]))
+
+    def test_argsort_unicode(self, table_types):
+        # Before Numpy 1.6.2, sorting with multiple column names
+        # failed when a unicode column was present.
+        t = table_types.Table()
+        t.add_column(table_types.Column(
+            name='firstname',
+            data=[six.text_type(x) for x in ["Max", "Jo", "John"]]))
+        t.add_column(table_types.Column(
+            name='name',
+            data=[six.text_type(x) for x in ["Miller", "Miller", "Jackson"]]))
+        t.add_column(table_types.Column(name='tel', data=[12, 15, 19]))
+        assert np.all(t.argsort(['name', 'firstname']) == np.array([2, 1, 0]))
+
+    def test_rebuild_column_view_then_rename(self, table_types):
+        """
+        Issue #2039 where renaming fails after any method that calls
+        _rebuild_table_column_view (this includes sort and add_row).
+        """
+        t = table_types.Table([[1]], names=('a',))
+        assert t.colnames == ['a']
+        assert t._data.dtype.names == ('a',)
+
+        t.add_row((2,))
+        assert t.colnames == ['a']
+        assert t._data.dtype.names == ('a',)
+
+        t.rename_column('a', 'b')
+        assert t.colnames == ['b']
+        assert t._data.dtype.names == ('b',)
+
+        t.sort('b')
+        assert t.colnames == ['b']
+        assert t._data.dtype.names == ('b',)
+
+        t.rename_column('b', 'c')
+        assert t.colnames == ['c']
+        assert t._data.dtype.names == ('c',)
+
 
 @pytest.mark.usefixtures('table_types')
 class TestIterator():
@@ -926,7 +974,7 @@ class TestIterator():
     def test_iterator(self, table_types):
         d = np.array([(2, 1),
                       (3, 6),
-                      (4, 5)], dtype=[('a', 'i4'), ('b', 'i4')])
+                      (4, 5)], dtype=[(str('a'), 'i4'), (str('b'), 'i4')])
         t = table_types.Table(d)
         if t.masked:
             with pytest.raises(ValueError):
@@ -967,7 +1015,7 @@ class TestConvertNumpyArray():
         assert d.colnames == list(np_data.dtype.names)
 
         with pytest.raises(ValueError):
-            np_data = np.array(d, dtype=[('c', 'i8'), ('d', 'i8')])
+            np_data = np.array(d, dtype=[(str('c'), 'i8'), (str('d'), 'i8')])
 
 
 def _assert_copies(t, t2, deep=True):
@@ -1168,3 +1216,98 @@ from ...utils.tests.test_metadata import MetaBaseTest
 class TestMetaTable(MetaBaseTest):
     test_class = table.Table
     args = ()
+
+
+def test_unicode_column_names(table_types):
+    """
+    Test that unicode column names are accepted.  Only do this for
+    Python 2 since strings are unicode already in Python 3.
+    """
+    if six.PY2:
+        t = table_types.Table([[1]], names=(six.text_type('a'),))
+        assert t.colnames == ['a']
+        t[six.text_type('b')] = 0.0
+        assert t.colnames == ['a', 'b']
+
+
+def test_unicode_content():
+    if isinstance('', bytes):
+        return
+
+    string_a = 'астрономическая питона'
+    string_b = 'миллиарды световых лет'
+
+    a = table.Table(
+        [[string_a, 2],
+         [string_b, 3]],
+        names=('a', 'b'))
+
+    assert repr(string_a) in repr(a)
+    assert string_a in six.text_type(a)
+    # This only works because the coding of this file is utf-8, which
+    # matches the default encoding of Table.__str__
+    assert string_a.encode('utf-8') in bytes(a)
+
+
+def test_unicode_policy():
+    t = table.Table.read([' a b  c  d',
+                          ' 2 c 7.0 0',
+                          ' 2 b 5.0 1',
+                          ' 2 b 6.0 2',
+                          ' 2 a 4.0 3',
+                          ' 0 a 0.0 4',
+                          ' 1 b 3.0 5',
+                          ' 1 a 2.0 6',
+                          ' 1 a 1.0 7',
+                         ], format='ascii')
+    assert_follows_unicode_guidelines(t)
+
+
+def test_unicode_bytestring_conversion(table_types):
+    t = table_types.Table([['abc'], ['def'], [1]], dtype=('S', 'U', 'i'))
+    assert t['col0'].dtype.kind == 'S'
+    assert t['col1'].dtype.kind == 'U'
+    assert t['col2'].dtype.kind == 'i'
+
+    t1 = t.copy()
+    t1.convert_unicode_to_bytestring()
+    assert t1['col0'].dtype.kind == 'S'
+    assert t1['col1'].dtype.kind == 'S'
+    assert t1['col2'].dtype.kind == 'i'
+    assert t1['col0'][0] == 'abc'.encode('ascii')
+    assert t1['col1'][0] == 'def'.encode('ascii')
+    assert t1['col2'][0] == 1
+
+    t1 = t.copy()
+    t1.convert_bytestring_to_unicode()
+    assert t1['col0'].dtype.kind == 'U'
+    assert t1['col1'].dtype.kind == 'U'
+    assert t1['col2'].dtype.kind == 'i'
+    assert t1['col0'][0] == six.text_type('abc')
+    assert t1['col1'][0] == six.text_type('def')
+    assert t1['col2'][0] == 1
+
+
+def test_table_deletion():
+    """
+    Regression test for the reference cycle discussed in
+    https://github.com/astropy/astropy/issues/2877
+    """
+
+    deleted = set()
+
+    # A special table subclass which leaves a record when it is finalized
+    class TestTable(table.Table):
+        def __del__(self):
+            deleted.add(id(self))
+
+    t = TestTable({'a': [1, 2, 3]})
+    the_id = id(t)
+    assert t['a'].parent_table is t
+
+    del t
+
+    # Cleanup
+    gc.collect()
+
+    assert the_id in deleted
