@@ -668,3 +668,15 @@ def test_TimeFormat_scale():
 def test_scale_conversion():
     with pytest.raises(ScaleValueError):
         t = Time(Time.now().cxcsec, format='cxcsec', scale='ut1')
+
+
+def test_byteorder():
+    """Ensure that bigendian and little-endian both work (closes #2942)"""
+    mjd = np.array([53000.00,54000.00])
+    big_endian = mjd.astype('>f8')
+    little_endian = mjd.astype('<f8')
+    time_mjd = Time(mjd, format='mjd')
+    time_big = Time(big_endian, format='mjd')
+    time_little = Time(little_endian, format='mjd')
+    assert np.all(time_big == time_mjd)
+    assert np.all(time_little == time_mjd)
