@@ -5,13 +5,18 @@
 Regression tests for the units.format package
 """
 
+# TEST_UNICODE_LITERALS
+
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
+from ...extern import six
+
 from numpy.testing.utils import assert_allclose
 from ...tests.helper import raises, pytest
 
 from ... import units as u
+from ...constants import si
 from .. import core
 from .. import format as u_format
 from ..utils import is_effectively_unity
@@ -75,7 +80,11 @@ def test_cds_grammar():
         (["mmag"], u.mmag),
         (["Mpc"], u.Mpc),
         (["Gyr"], u.Gyr),
-        (["°"], u.degree)]
+        (["°"], u.degree),
+        (["°/s"], u.degree / u.s),
+        (["Å"], u.AA),
+        (["Å/s"], u.AA / u.s),
+        (["\\h"], si.h)]
 
     for strings, unit in data:
         for s in strings:
@@ -88,14 +97,79 @@ def test_cds_grammar_fail():
         print(s)
         u_format.CDS().parse(s)
 
-    data = ['0.1 nm', 'solMass(3/2)', 'km / s', 'km s-1',
-            'pix0.1nm', 'pix/(0.1nm)', 'km*s', 'km**2',
-            '5x8+3m', '0.1---', '---m', 'm---']
+    data = ['0.1 nm',
+            'solMass(3/2)',
+            'km / s',
+            'km s-1',
+            'pix0.1nm',
+            'pix/(0.1nm)',
+            'km*s',
+            'km**2',
+            '5x8+3m',
+            '0.1---',
+            '---m',
+            'm---']
 
     for s in data:
         yield _test_cds_grammar_fail, s
 
 
+def test_ogip_grammar():
+    def _test_ogip_grammar(s, unit):
+        print(s)
+        unit2 = u_format.OGIP().parse(s)
+        assert unit2 == unit
+
+    # These examples are taken from the EXAMPLES section of
+    # http://heasarc.gsfc.nasa.gov/docs/heasarc/ofwg/docs/general/ogip_93_001/
+    data = [
+        (["count /s", "count/s", "count s**(-1)", "count / s", "count /s "],
+         u.count / u.s),
+        (["/pixel /s", "/(pixel * s)"], (u.pixel * u.s) ** -1),
+        (["count /m**2 /s /eV", "count m**(-2) * s**(-1) * eV**(-1)",
+          "count /(m**2 * s * eV)"],
+         u.count * u.m ** -2 * u.s ** -1 * u.eV ** -1),
+        (["erg /pixel /s /GHz", "erg /s /GHz /pixel", "erg /pixel /(s * GHz)"],
+         u.erg / (u.s * u.GHz * u.pixel)),
+        (["keV**2 /yr /angstrom", "10**(10) keV**2 /yr /m",
+          # Though this is given as an example, it seems to violate the rules
+          # of not raising scales to powers, so I'm just excluding it
+          # "(10**2 MeV)**2 /yr /m"
+         ],
+         u.keV**2 / (u.yr * u.angstrom)),
+        (["10**(46) erg /s", "10**46 erg /s", "10**(39) J /s", "10**(39) W",
+          "10**(15) YW", "YJ /fs"],
+         10**46 * u.erg / u.s),
+        (["10**(-7) J /cm**2 /MeV", "10**(-9) J m**(-2) eV**(-1)",
+          "nJ m**(-2) eV**(-1)", "nJ /m**2 /eV"],
+         10 ** -7 * u.J * u.cm ** -2 * u.MeV ** -1),
+        (["sqrt(erg /pixel /s /GHz)", "(erg /pixel /s /GHz)**(0.5)",
+          "(erg /pixel /s /GHz)**(1/2)",
+          "erg**(0.5) pixel**(-0.5) s**(-0.5) GHz**(-0.5)"],
+         (u.erg * u.pixel ** -1 * u.s ** -1 * u.GHz ** -1) ** 0.5),
+        (["(count /s) (/pixel /s)", "(count /s) * (/pixel /s)",
+          "count /pixel /s**2"],
+         (u.count / u.s) * (1.0 / (u.pixel * u.s)))]
+
+    for strings, unit in data:
+        for s in strings:
+            yield _test_ogip_grammar, s, unit
+
+
+def test_ogip_grammar_fail():
+    @raises(ValueError)
+    def _test_ogip_grammar_fail(s):
+        u_format.OGIP().parse(s)
+
+    data = ['log(photon /m**2 /s /Hz)',
+            'sin( /pixel /s)',
+            'log(photon /cm**2 /s /Hz) /(sin( /pixel /s))',
+            'log(photon /cm**2 /s /Hz) (sin( /pixel /s))**(-1)']
+
+    for s in data:
+        yield _test_ogip_grammar_fail, s
+
+
 def test_roundtrip():
     def _test_roundtrip(unit):
         a = core.Unit(unit.to_string('generic'), format='generic')
@@ -104,21 +178,25 @@ def test_roundtrip():
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
     for key, val in u.__dict__.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip, val
 
 
 def test_roundtrip_vo_unit():
-    def _test_roundtrip_vo_unit(unit):
+    def _test_roundtrip_vo_unit(unit, skip_decompose):
         a = core.Unit(unit.to_string('vounit'), format='vounit')
-        b = core.Unit(unit.decompose().to_string('vounit'), format='vounit')
         assert_allclose(a.decompose().scale, unit.decompose().scale, rtol=1e-2)
+        if skip_decompose:
+            return
+        u = unit.decompose().to_string('vounit')
+        assert '  ' not in u
+        b = core.Unit(u, format='vounit')
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
     x = u_format.VOUnit()
     for key, val in x._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
-            yield _test_roundtrip_vo_unit, val
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
+            yield _test_roundtrip_vo_unit, val, val in (u.mag, u.dB)
 
 
 def test_roundtrip_fits():
@@ -128,23 +206,42 @@ def test_roundtrip_fits():
         assert_allclose(a.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
     for key, val in u_format.Fits()._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_fits, val
 
 
 def test_roundtrip_cds():
     def _test_roundtrip_cds(unit):
         a = core.Unit(unit.to_string('cds'), format='cds')
-        b = core.Unit(unit.decompose().to_string('cds'), format='cds')
         assert_allclose(a.decompose().scale, unit.decompose().scale, rtol=1e-2)
+        try:
+            b = core.Unit(unit.decompose().to_string('cds'), format='cds')
+        except ValueError:  # skip mag: decomposes into dex, unknown to OGIP
+            return
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
     x = u_format.CDS()
     for key, val in x._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_cds, val
 
 
+def test_roundtrip_ogip():
+    def _test_roundtrip_ogip(unit):
+        a = core.Unit(unit.to_string('ogip'), format='ogip')
+        assert_allclose(a.decompose().scale, unit.decompose().scale, rtol=1e-2)
+        try:
+            b = core.Unit(unit.decompose().to_string('ogip'), format='ogip')
+        except ValueError:  # skip mag: decomposes into dex, unknown to OGIP
+            return
+        assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
+
+    x = u_format.OGIP()
+    for key, val in x._units.items():
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
+            yield _test_roundtrip_ogip, val
+
+
 def test_fits_units_available():
     u_format.Fits()
 
@@ -161,10 +258,12 @@ def test_latex():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     assert fluxunit.to_string('latex') == r'$\mathrm{\frac{erg}{s\,cm^{2}}}$'
 
+
 def test_new_style_latex():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     assert "{0:latex}".format(fluxunit) == r'$\mathrm{\frac{erg}{s\,cm^{2}}}$'
 
+
 def test_format_styles():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     def _test_format_styles(format_spec, s):
@@ -181,23 +280,6 @@ def test_format_styles():
     for format_, s in format_s_pairs:
         yield _test_format_styles, format_, s
 
-def test_wcs_parse():
-    """
-    Tests that the output of to_string('fits') is also parsed by
-    wcslib.  Even if we deprecated access to wcslib's unit parser, we
-    may want to keep it around and hidden for this test.
-    """
-    def _test_wcs_parse(unit):
-        try:
-            fits_string = unit.decompose().to_string('fits')
-        except ValueError:
-            return
-        wcs.UnitConverter(fits_string, fits_string)
-
-    for key, val in u.__dict__.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
-            yield _test_wcs_parse, val
-
 
 def test_flatten_to_known():
     myunit = u.def_unit("FOOBAR_One", u.erg / u.Hz)
