@@ -1,5 +1,8 @@
+# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+# TEST_UNICODE_LITERALS
+
 """
 Regression tests for the units package
 """
@@ -11,11 +14,13 @@ from __future__ import (absolute_import, unicode_literals, division,
 import numpy as np
 from numpy.testing.utils import assert_allclose
 
+from ...extern import six
 from ...extern.six.moves import cPickle as pickle
 from ...tests.helper import pytest, raises, catch_warnings
 from ...utils.compat.fractions import Fraction
 
 from ... import units as u
+from ... import constants as c
 
 
 def test_getting_started():
@@ -32,13 +37,36 @@ def test_getting_started():
         assert_allclose(x, [2.23693629e-02, 2.23693629e+01, 1.11846815e+02])
 
 
+def test_initialisation():
+    assert u.Unit(u.m) is u.m
+
+    ten_meter = u.Unit(10.*u.m)
+    assert ten_meter == u.CompositeUnit(10., [u.m], [1])
+    assert u.Unit(ten_meter) is ten_meter
+
+    assert u.Unit(10.*ten_meter) == u.CompositeUnit(100., [u.m], [1])
+
+    foo = u.Unit('foo', (10. * ten_meter)**2, namespace=locals())
+    assert foo == u.CompositeUnit(10000., [u.m], [2])
+
+    assert u.Unit('m') == u.m
+    assert u.Unit('') == u.dimensionless_unscaled
+    assert u.one == u.dimensionless_unscaled
+    assert u.Unit('10 m') == ten_meter
+    assert u.Unit(10.) == u.CompositeUnit(10., [], [])
+
+
 def test_invalid_power():
-    x = u.m ** (1, 3)
+    x = u.m ** Fraction(1, 3)
     assert isinstance(x.powers[0], Fraction)
 
-    x = u.m ** (1, 2)
+    x = u.m ** Fraction(1, 2)
     assert isinstance(x.powers[0], float)
 
+    # Test the automatic conversion to a fraction
+    x = u.m ** (1. / 3.)
+    assert isinstance(x.powers[0], Fraction)
+
 
 def test_invalid_compare():
     assert not (u.m == u.s)
@@ -121,11 +149,17 @@ def test_unknown_unit():
     assert 'FOO' in str(warning_lines[0].message)
 
 
-def test_unknown_unit2():
+def test_multiple_solidus():
+    assert u.Unit("m/s/kg").to_string() == u.m / u.s / u.kg
+
     with catch_warnings(u.UnitsWarning) as warning_lines:
-        assert u.Unit("m/s/kg", parse_strict='warn').to_string() == 'm/s/kg'
+        assert u.Unit("m/s/kg").to_string() == u.m / (u.s * u.kg)
 
     assert 'm/s/kg' in str(warning_lines[0].message)
+    assert 'discouraged' in str(warning_lines[0].message)
+
+    with pytest.raises(ValueError):
+        u.Unit("m/s/kg", format="vounit")
 
 
 def test_unknown_unit3():
@@ -236,7 +270,7 @@ def test_decompose_bases():
 
     d = e.esu.unit.decompose(bases=cgs.bases)
     assert d._bases == [u.cm, u.g, u.s]
-    assert d._powers == [Fraction(3, 2), Fraction(1, 2), -1]
+    assert d._powers == [Fraction(3, 2), 0.5, -1]
     assert d._scale == 1.0
 
 
@@ -253,7 +287,7 @@ def test_equiv_compose():
 
 
 def test_empty_compose():
-    with pytest.raises(u.UnitsException):
+    with pytest.raises(u.UnitsError):
         composed = u.m.compose(units=[])
 
 
@@ -285,9 +319,11 @@ def test_compose_cgs_to_si():
         assert [x.is_equivalent(unit) for x in si]
         assert si[0] == unit.si
 
+    # Can't decompose Celsius
     for val in u.cgs.__dict__.values():
         if (isinstance(val, u.UnitBase) and
-                not isinstance(val, u.PrefixUnit)):
+                not isinstance(val, u.PrefixUnit) and
+                val != u.cgs.deg_C):
             yield _test_compose_cgs_to_si, val
 
 
@@ -305,9 +341,11 @@ def test_compose_si_to_cgs():
             assert [x.is_equivalent(unit) for x in cgs]
             assert cgs[0] == unit.cgs
 
+    # Can't decompose Celsius
     for val in u.si.__dict__.values():
         if (isinstance(val, u.UnitBase) and
-                not isinstance(val, u.PrefixUnit)):
+                not isinstance(val, u.PrefixUnit) and
+                val != u.si.deg_C):
             yield _test_compose_si_to_cgs, val
 
 
@@ -425,11 +463,19 @@ def test_pickling():
     assert other is u.m
 
     new_unit = u.IrreducibleUnit(['foo'], format={'baz': 'bar'})
-    u.add_enabled_units([new_unit])
-    p = pickle.dumps(new_unit)
-    new_unit_copy = pickle.loads(p)
-    assert new_unit_copy.names == ['foo']
-    assert new_unit_copy.get_format_name('baz') == 'bar'
+    with u.add_enabled_units([new_unit]):
+        p = pickle.dumps(new_unit)
+        new_unit_copy = pickle.loads(p)
+        assert new_unit_copy.names == ['foo']
+        assert new_unit_copy.get_format_name('baz') == 'bar'
+
+
+def test_pickle_unrecognized_unit():
+    """
+    Issue #2047
+    """
+    a = u.Unit('asdf', parse_strict='silent')
+    pickle.loads(pickle.dumps(a))
 
 
 @raises(ValueError)
@@ -459,7 +505,7 @@ def test_comparison():
     assert u.cm < u.m
     assert u.cm <= u.m
 
-    with pytest.raises(u.UnitsException):
+    with pytest.raises(u.UnitsError):
         u.m > u.kg
 
 
@@ -506,8 +552,80 @@ def test_composite_unit_get_format_name():
     assert (str(u.CompositeUnit(1, [unit1, unit2], [1, -1])) ==
             'nrad / (Hz(1/2) s)')
 
+
 def test_unicode_policy():
     from ...tests.helper import assert_follows_unicode_guidelines
 
     assert_follows_unicode_guidelines(
         u.degree, roundtrip=u.__dict__)
+
+
+def test_suggestions():
+    for search, matches in [
+            ('microns', 'micron'),
+            ('s/microns', 'micron'),
+            ('M', 'm'),
+            ('metre', 'meter'),
+            ('angstroms', 'Angstrom or angstrom'),
+            ('milimeter', 'millimeter'),
+            ('ångström', 'Angstrom or angstrom'),
+            ('kev', 'EV, eV, kV or keV')]:
+        try:
+            u.Unit(search)
+        except ValueError as e:
+            assert 'Did you mean {0}?'.format(matches) in six.text_type(e)
+        else:
+            assert False, 'Expected ValueError'
+
+
+def test_fits_hst_unit():
+    """See #1911."""
+    x = u.Unit("erg /s /cm**2 /angstrom")
+    assert x == u.erg * u.s ** -1 * u.cm ** -2 * u.angstrom ** -1
+
+
+def test_fractional_powers():
+    """See #2069"""
+    m = 1e9 * u.Msun
+    tH = 1. / (70. * u.km / u.s / u.Mpc)
+    vc = 200 * u.km/u.s
+
+    x = (c.G ** 2 * m ** 2 * tH.cgs) ** Fraction(1, 3) / vc
+    v1 = x.to('pc')
+
+    x = (c.G ** 2 * m ** 2 * tH) ** Fraction(1, 3) / vc
+    v2 = x.to('pc')
+
+    x = (c.G ** 2 * m ** 2 * tH.cgs) ** (1.0 / 3.0) / vc
+    v3 = x.to('pc')
+
+    x = (c.G ** 2 * m ** 2 * tH) ** (1.0 / 3.0) / vc
+    v4 = x.to('pc')
+
+    assert_allclose(v1, v2)
+    assert_allclose(v2, v3)
+    assert_allclose(v3, v4)
+
+    x = u.m ** (1.0 / 11.0)
+    assert isinstance(x.powers[0], float)
+
+    x = u.m ** (3.0 / 7.0)
+    assert isinstance(x.powers[0], Fraction)
+    assert x.powers[0].numerator == 3
+    assert x.powers[0].denominator == 7
+
+
+def test_inherit_docstrings():
+    assert u.UnrecognizedUnit.is_unity.__doc__ == u.UnitBase.is_unity.__doc__
+
+
+def test_sqrt_mag():
+    sqrt_mag = u.mag ** 0.5
+    assert hasattr(sqrt_mag.decompose().scale, 'imag')
+    assert (sqrt_mag.decompose())**2 == u.mag
+
+
+def test_composite_compose():
+    # Issue #2382
+    composite_unit = u.s.compose(units=[u.Unit("s")])[0]
+    u.s.compose(units=[composite_unit])
