@@ -12,6 +12,7 @@ import re
 import math
 import multiprocessing
 import os
+import struct
 import sys
 import threading
 import time
@@ -29,52 +30,86 @@ try:
 except NameError:
     OutStream = None
     IPythonIOStream = None
-    stdio = sys
 else:
     try:
         from IPython.zmq.iostream import OutStream
-        from IPython.utils import io
+    except ImportError:
+        try:
+            from IPython.kernel.zmq.iostream import OutStream
+        except ImportError:
+            OutStream = None
+
+    if OutStream is not None:
+        from IPython.utils import io as ipyio
         # On Windows in particular this is necessary, as the io.stdout stream
         # in IPython gets hooked up to some pyreadline magic to handle colors
-        stdio = io
-        IPythonIOStream = io.IOStream
-    except ImportError:
+        IPythonIOStream = ipyio.IOStream
+    else:
         OutStream = None
         IPythonIOStream = None
-        stdio = sys
 
-try:
-    import IPython  # pylint: disable=W0611
-    # This is just to set a flag that IPython is installed at all
-    _HAVE_IPYTHON = True
-except ImportError:
-    _HAVE_IPYTHON = False
+    # On Windows, in IPython 2 the standard I/O streams will wrap
+    # pyreadline.Console objects if pyreadline is available; this should
+    # be considered a TTY
+    try:
+        from pyreadyline.console import Console as PyreadlineConsole
+    except ImportError:
+        # Just define a dummy class
+        class PyreadlineConsole(object): pass
 
-from ..config import ConfigurationItem
+from ..config import ConfigAlias
 from ..extern import six
 from ..extern.six.moves import range
+from .. import conf
 
 from .misc import deprecated, isiterable
 
 
 __all__ = [
     'isatty', 'color_print', 'human_time', 'human_file_size',
-    'ProgressBar', 'Spinner', 'print_code_line', 'ProgressBarOrSpinner']
+    'ProgressBar', 'Spinner', 'print_code_line', 'ProgressBarOrSpinner',
+    'terminal_size']
 
 
 # Only use color by default on Windows if IPython is installed.
-USE_COLOR = ConfigurationItem(
-    'use_color', sys.platform != 'win32' or _HAVE_IPYTHON,
-    'When True, use ANSI color escape sequences when writing to the console.')
-from ..import UNICODE_OUTPUT
-
-
-IS_PY3 = sys.version_info[0] == 3
+USE_COLOR = ConfigAlias(
+    '0.4', 'USE_COLOR', 'use_color', 'astropy.utils.console', 'astropy')
 
 
 _DEFAULT_ENCODING = 'utf-8'
 
 
+def _get_stdout(stderr=False):
+    """
+    This utility fucntion contains the logic to determine what streams to use
+    by default for standard out/err.
+
+    Typically this will just return `sys.stdout`, but it contains additional
+    logic for use in IPython on Windows to determine the correct stream to use
+    (usually ``IPython.util.io.stdout`` but only if sys.stdout is a TTY).
+    """
+
+    if stderr:
+        stream = 'stderr'
+    else:
+        stream = 'stdout'
+
+    sys_stream = getattr(sys, stream)
+
+    if IPythonIOStream is None:
+        return sys_stream
+
+    ipyio_stream = getattr(ipyio, stream)
+
+    if isatty(sys_stream) and isatty(ipyio_stream):
+        # Use the IPython console output stream
+        return ipyio_stream
+    else:
+        # sys.stdout was set to some other non-TTY stream (a file perhaps)
+        # so just use it directly
+        return sys_stream
+
+
 def isatty(file):
     """
     Returns `True` if `file` is a tty.
@@ -87,15 +122,59 @@ def isatty(file):
         threading.current_thread().getName() != 'MainThread'):
         return False
 
-    if (OutStream is not None and
-        isinstance(file, OutStream) and
-        file.name == 'stdout'):
-        return True
-    elif hasattr(file, 'isatty'):
+    if hasattr(file, 'isatty'):
         return file.isatty()
+    elif (OutStream is not None and
+          isinstance(file, (OutStream, IPythonIOStream)) and
+          ((hasattr(file, 'name') and file.name == 'stdout') or
+           (hasattr(file, 'stream') and
+               isinstance(file.stream, PyreadlineConsole)))):
+        # File is an IPython OutStream or IOStream and
+        #    File name is 'stdout' or
+        #    File wraps a Console
+        return True
     return False
 
 
+def terminal_size(file=None):
+    """
+    Returns a tuple (height, width) containing the height and width of
+    the terminal.
+
+    This function will look for the width in height in multiple areas
+    before falling back on the width and height in astropy's
+    configuration.
+    """
+
+    if file is None:
+        file = _get_stdout()
+
+    try:
+        s = struct.pack(str("HHHH"), 0, 0, 0, 0)
+        x = fcntl.ioctl(file, termios.TIOCGWINSZ, s)
+        (lines, width, xpixels, ypixels) = struct.unpack(str("HHHH"), x)
+        if lines > 12:
+            lines -= 6
+        if width > 10:
+            width -= 1
+        return (lines, width)
+    except:
+        try:
+            # see if POSIX standard variables will work
+            return (int(os.environ.get('LINES')),
+                    int(os.environ.get('COLUMNS')))
+        except TypeError:
+            # fall back on configuration variables, or if not
+            # set, (25, 80)
+            lines = conf.max_lines
+            width = conf.max_width
+            if lines is None:
+                lines = 25
+            if width is None:
+                width = 80
+            return lines, width
+
+
 def _color_text(text, color):
     """
     Returns a string wrapped in ANSI color codes for coloring the
@@ -236,12 +315,12 @@ def color_print(*args, **kwargs):
         be printed after resetting any color or font state.
     """
 
-    file = kwargs.get('file', stdio.stdout)
+    file = kwargs.get('file', _get_stdout())
 
     end = kwargs.get('end', '\n')
 
     write = file.write
-    if isatty(file) and USE_COLOR():
+    if isatty(file) and conf.use_color:
         for i in range(0, len(args), 2):
             msg = args[i]
             if i + 1 == len(args):
@@ -255,7 +334,7 @@ def color_print(*args, **kwargs):
             # Some file objects support writing unicode sensibly on some Python
             # versions; if this fails try creating a writer using the locale's
             # preferred encoding. If that fails too give up.
-            if not IS_PY3 and isinstance(msg, bytes):
+            if not six.PY3 and isinstance(msg, bytes):
                 msg = _decode_preferred_encoding(msg)
 
             write = _write_with_fallback(msg, write, file)
@@ -264,7 +343,7 @@ def color_print(*args, **kwargs):
     else:
         for i in range(0, len(args), 2):
             msg = args[i]
-            if not IS_PY3 and isinstance(msg, bytes):
+            if not six.PY3 and isinstance(msg, bytes):
                 # Support decoding bytes to unicode on Python 2; use the
                 # preferred encoding for the locale (which is *sometimes*
                 # sensible)
@@ -318,12 +397,12 @@ def human_time(seconds):
     seconds = int(seconds)
 
     if seconds < 60:
-        return '   {0:02d}s'.format(seconds)
-    for i in xrange(len(units) - 1):
+        return '   {0:2d}s'.format(seconds)
+    for i in range(len(units) - 1):
         unit1, limit1 = units[i]
         unit2, limit2 = units[i + 1]
         if seconds >= limit1:
-            return '{0:02d}{1}{2:02d}{3}'.format(
+            return '{0:2d}{1}{2:2d}{3}'.format(
                 seconds // limit1, unit1,
                 (seconds % limit1) // limit2, unit2)
     return '  ~inf'
@@ -374,7 +453,7 @@ class ProgressBar(six.Iterator):
     """
     A class to display a progress bar in the terminal.
 
-    It is designed to be used either with the `with` statement::
+    It is designed to be used either with the ``with`` statement::
 
         with ProgressBar(len(items)) as bar:
             for item in enumerate(items):
@@ -396,11 +475,13 @@ class ProgressBar(six.Iterator):
         file : writable file-like object, optional
             The file to write the progress bar to.  Defaults to
             `sys.stdout`.  If `file` is not a tty (as determined by
-            calling its `isatty` member, if any), the scrollbar will
-            be completely silent.
+            calling its `isatty` member, if any, or special case hacks
+            to detect the IPython console), the progress bar will be
+            completely silent.
         """
+
         if file is None:
-            file = stdio.stdout
+            file = _get_stdout()
 
         if not isatty(file):
             self.update = self._silent_update
@@ -423,7 +504,7 @@ class ProgressBar(six.Iterator):
         self._start_time = time.time()
 
         self._should_handle_resize = (
-            _CAN_RESIZE_TERMINAL and isatty(self._file))
+            _CAN_RESIZE_TERMINAL and self._file.isatty())
         self._handle_resize()
         if self._should_handle_resize:
             signal.signal(signal.SIGWINCH, self._handle_resize)
@@ -435,18 +516,7 @@ class ProgressBar(six.Iterator):
         self.update(0)
 
     def _handle_resize(self, signum=None, frame=None):
-        if self._should_handle_resize:
-            # Don't import numpy at module level since it may not be
-            # available in all contexts that this module is imported
-            import numpy as np
-            data = fcntl.ioctl(self._file, termios.TIOCGWINSZ, '\0' * 8)
-            arr = np.fromstring(data, dtype=np.int16)
-            terminal_width = arr[1]
-        else:
-            try:
-                terminal_width = int(os.environ.get('COLUMNS'))
-            except (TypeError, ValueError):
-                terminal_width = 78
+        terminal_width = terminal_size(self._file)[1]
         self._bar_length = terminal_width - 37
 
     def __enter__(self):
@@ -492,7 +562,10 @@ class ProgressBar(six.Iterator):
         file = self._file
         write = file.write
 
-        bar_fill = int(float(self._bar_length) * frac)
+        if frac > 1:
+            bar_fill = int(self._bar_length)
+        else:
+            bar_fill = int(float(self._bar_length) * frac)
         write('\r|')
         color_print('=' * bar_fill, 'blue', file=file, end='')
         if bar_fill < self._bar_length:
@@ -553,10 +626,11 @@ class ProgressBar(six.Iterator):
             calling its `isatty` member, if any), the scrollbar will
             be completely silent.
         """
+
         results = []
 
         if file is None:
-            file = stdio.stdout
+            file = _get_stdout()
 
         with cls(len(items), file=file) as bar:
             step_size = max(200, bar._bar_length)
@@ -603,10 +677,12 @@ class ProgressBar(six.Iterator):
         Returns
         -------
         generator :
-            A generator over `items`
+            A generator over ``items``.
         """
+
         if file is None:
-            file = stdio.stdout
+            file = _get_stdout()
+
         return cls(items, file=file)
 
 
@@ -614,7 +690,7 @@ class Spinner(object):
     """
     A class to display a spinner in the terminal.
 
-    It is designed to be used with the `with` statement::
+    It is designed to be used with the ``with`` statement::
 
         with Spinner("Reticulating splines", "green") as s:
             for item in enumerate(items):
@@ -640,8 +716,9 @@ class Spinner(object):
         file : writeable file-like object, optional
             The file to write the spinner to.  Defaults to
             `sys.stdout`.  If `file` is not a tty (as determined by
-            calling its `isatty` member, if any), the scrollbar will
-            be completely silent.
+            calling its `isatty` member, if any, or special case hacks
+            to detect the IPython console), the spinner will be
+            completely silent.
 
         step : int, optional
             Only update the spinner every *step* steps
@@ -649,15 +726,16 @@ class Spinner(object):
         chars : str, optional
             The character sequence to use for the spinner
         """
+
         if file is None:
-            file = stdio.stdout
+            file = _get_stdout()
 
         self._msg = msg
         self._color = color
         self._file = file
         self._step = step
         if chars is None:
-            if UNICODE_OUTPUT():
+            if conf.unicode_output:
                 chars = self._default_unicode_chars
             else:
                 chars = self._default_ascii_chars
@@ -730,7 +808,7 @@ class ProgressBarOrSpinner(object):
     depending on whether the total size of the operation is
     known or not.
 
-    It is designed to be used with the `with` statement::
+    It is designed to be used with the ``with`` statement::
 
         if file.has_length():
             length = file.get_length()
@@ -757,7 +835,7 @@ class ProgressBarOrSpinner(object):
             alongside the `Spinner`.
 
         color : str, optional
-            The color of `msg`, if any.  Must be an ANSI terminal
+            The color of ``msg``, if any.  Must be an ANSI terminal
             color name.  Must be one of: black, red, green, brown,
             blue, magenta, cyan, lightgrey, default, darkgrey,
             lightred, lightgreen, yellow, lightblue, lightmagenta,
@@ -766,12 +844,12 @@ class ProgressBarOrSpinner(object):
         file : writable file-like object, optional
             The file to write the to.  Defaults to `sys.stdout`.  If
             `file` is not a tty (as determined by calling its `isatty`
-            member, if any), only `msg` will be displayed: the
+            member, if any), only ``msg`` will be displayed: the
             `ProgressBar` or `Spinner` will be silent.
         """
 
         if file is None:
-            file = stdio.stdout
+            file = _get_stdout()
 
         if total is None or not isatty(file):
             self._is_spinner = True
@@ -805,7 +883,7 @@ def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
     position in the line.  Useful for displaying the context of error
     messages.
 
-    If the line is more than `width` characters, the line is truncated
+    If the line is more than ``width`` characters, the line is truncated
     accordingly and '…' characters are inserted at the front and/or
     end.
 
@@ -820,8 +898,8 @@ def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
         The line of code to display
 
     col : int, optional
-        The character in the line to highlight.  `col` must be less
-        than `len(line)`.
+        The character in the line to highlight.  ``col`` must be less
+        than ``len(line)``.
 
     file : writeable file-like object, optional
         Where to write to.  Defaults to `sys.stdout`.
@@ -838,7 +916,12 @@ def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
     """
 
     if file is None:
-        file = stdio.stdout
+        file = _get_stdout()
+
+    if conf.unicode_output:
+        ellipsis = '…'
+    else:
+        ellipsis = '...'
 
     write = file.write
 
@@ -853,15 +936,15 @@ def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
     if col is not None and col > width:
         new_col = min(width // 2, len(line) - col)
         offset = col - new_col
-        line = line[offset + 1:]
+        line = line[offset + len(ellipsis):]
+        width -= len(ellipsis)
         new_col = col
         col -= offset
-        width = width - 3
-        color_print('…', 'darkgrey', file=file, end='')
+        color_print(ellipsis, 'darkgrey', file=file, end='')
 
     if len(line) > width:
-        write(line[:width - 1])
-        color_print('…', 'darkgrey', file=file)
+        write(line[:width - len(ellipsis)])
+        color_print(ellipsis, 'darkgrey', file=file)
     else:
         write(line)
         write('\n')
