@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
 A "grab bag" of relatively small general-purpose utilities that don't have
@@ -9,6 +10,7 @@ from __future__ import (absolute_import, division, print_function,
 
 
 import contextlib
+import difflib
 import functools
 import inspect
 import json
@@ -17,9 +19,12 @@ import signal
 import sys
 import textwrap
 import traceback
+import types
+import unicodedata
 import warnings
 
-from .exceptions import AstropyDeprecationWarning, AstropyPendingDeprecationWarning
+from .exceptions import (AstropyDeprecationWarning,
+                         AstropyPendingDeprecationWarning)
 
 from ..extern import six
 from ..extern.six.moves import urllib
@@ -28,7 +33,8 @@ from ..extern.six.moves import urllib
 __all__ = ['find_current_module', 'isiterable', 'deprecated', 'lazyproperty',
            'deprecated_attribute', 'silence', 'format_exception',
            'NumpyRNGContext', 'find_api_page', 'is_path_hidden',
-           'walk_skip_hidden', 'JsonCustomEncoder', 'indent']
+           'walk_skip_hidden', 'JsonCustomEncoder', 'indent',
+           'InheritDocstrings']
 
 __doctest_skip__ = ['find_current_module']
 
@@ -36,9 +42,9 @@ __doctest_skip__ = ['find_current_module']
 def find_current_module(depth=1, finddiff=False):
     """ Determines the module/package from which this function is called.
 
-    This function has two modes, determined by the `finddiff` option. it
+    This function has two modes, determined by the ``finddiff`` option. it
     will either simply go the requested number of frames up the call
-    stack (if `finddiff` is False), or it will go up the call stack until
+    stack (if ``finddiff`` is False), or it will go up the call stack until
     it reaches a module that is *not* in a specified set.
 
     Parameters
@@ -47,13 +53,13 @@ def find_current_module(depth=1, finddiff=False):
         Specifies how far back to go in the call stack (0-indexed, so that
         passing in 0 gives back `astropy.utils.misc`).
     finddiff : bool or list
-        If False, the returned `mod` will just be `depth` frames up from
+        If False, the returned ``mod`` will just be ``depth`` frames up from
         the current frame. Otherwise, the function will start at a frame
-        `depth` up from current, and continue up the call stack to the
+        ``depth`` up from current, and continue up the call stack to the
         first module that is *different* from those in the provided list.
-        In this case, `finddiff` can be a list of modules or modules
+        In this case, ``finddiff`` can be a list of modules or modules
         names. Alternatively, it can be True, which will use the module
-        `depth` call stack frames up as the module the returned module
+        ``depth`` call stack frames up as the module the returned module
         most be different from.
 
     Returns
@@ -66,12 +72,12 @@ def find_current_module(depth=1, finddiff=False):
     Raises
     ------
     ValueError
-        If `finddiff` is a list with an invalid entry.
+        If ``finddiff`` is a list with an invalid entry.
 
     Examples
     --------
     The examples below assume that there are two modules in a package named
-    `pkg`. ``mod1.py``::
+    ``pkg``. ``mod1.py``::
 
         def find1():
             from astropy.utils import find_current_module
@@ -151,28 +157,30 @@ def find_mod_objs(modname, onlylocals=False):
 
     .. note::
         The returned list *not* include subpackages or modules of
-        `modname`,nor does it include private attributes (those that
+        ``modname``, nor does it include private attributes (those that
         beginwith '_' or are not in `__all__`).
 
     Parameters
     ----------
     modname : str
         The name of the module to search.
-    onlylocals : bool
-        If True, only attributes that are either members of `modname` OR one of
-        its modules or subpackages will be included.
+    onlylocals : bool or list of str
+        If `True`, only attributes that are either members of ``modname`` OR
+        one of its modules or subpackages will be included. If it is a list
+        of strings, those specify the possible packages that will be
+        considered "local".
 
     Returns
     -------
     localnames : list of str
         A list of the names of the attributes as they are named in the
-        module `modname` .
+        module ``modname`` .
     fqnames : list of str
         A list of the full qualified names of the attributes (e.g.,
         ``astropy.utils.misc.find_mod_objs``). For attributes that are
         simple variables, this is based on the local name, but for
         functions or classes it can be different if they are actually
-        defined elsewhere and just referenced in `modname`.
+        defined elsewhere and just referenced in ``modname``.
     objs : list of objects
         A list of the actual attributes themselves (in the same order as
         the other arguments)
@@ -201,7 +209,9 @@ def find_mod_objs(modname, onlylocals=False):
             fqnames.append(modname + '.' + lnm)
 
     if onlylocals:
-        valids = [fqn.startswith(modname) for fqn in fqnames]
+        if onlylocals is True:
+            onlylocals = [modname]
+        valids = [any([fqn.startswith(nm) for nm in onlylocals]) for fqn in fqnames]
         localnames = [e for i, e in enumerate(localnames) if valids[i]]
         fqnames = [e for i, e in enumerate(fqnames) if valids[i]]
         objs = [e for i, e in enumerate(objs) if valids[i]]
@@ -267,36 +277,34 @@ class lazyproperty(object):
             self.__doc__ = fget.__doc__
         else:
             self.__doc__ = doc
+        self._key = self._fget.__name__
 
     def __get__(self, obj, owner=None):
         if obj is None:
             return self
-        key = self._fget.__name__
-        if key not in obj.__dict__:
+        try:
+            return obj.__dict__[self._key]
+        except KeyError:
             val = self._fget(obj)
-            obj.__dict__[key] = val
+            obj.__dict__[self._key] = val
             return val
-        else:
-            return obj.__dict__[key]
 
     def __set__(self, obj, val):
         obj_dict = obj.__dict__
-        func_name = self._fget.__name__
         if self._fset:
             ret = self._fset(obj, val)
-            if ret is not None and obj_dict.get(func_name) is ret:
+            if ret is not None and obj_dict.get(self._key) is ret:
                 # By returning the value set the setter signals that it took
                 # over setting the value in obj.__dict__; this mechanism allows
                 # it to override the input value
                 return
-        obj_dict[func_name] = val
+        obj_dict[self._key] = val
 
     def __delete__(self, obj):
         if self._fdel:
             self._fdel(obj)
-        key = self._fget.__name__
-        if key in obj.__dict__:
-            del obj.__dict__[key]
+        if self._key in obj.__dict__:
+            del obj.__dict__[self._key]
 
     def getter(self, fget):
         return self.__ter(fget, 0)
@@ -321,11 +329,10 @@ class lazyproperty(object):
         return cls_ns[property_name]
 
 
-# TODO: Provide a class deprecation marker as well.
 def deprecated(since, message='', name='', alternative='', pending=False,
-               obj_type='function'):
+               obj_type=None):
     """
-    Used to mark a function as deprecated.
+    Used to mark a function or class as deprecated.
 
     To mark an attribute as deprecated, use `deprecated_attribute`.
 
@@ -337,56 +344,171 @@ def deprecated(since, message='', name='', alternative='', pending=False,
 
     message : str, optional
         Override the default deprecation message.  The format
-        specifier `%(func)s` may be used for the name of the function,
-        and `%(alternative)s` may be used in the deprecation message
+        specifier ``func`` may be used for the name of the function,
+        and ``alternative`` may be used in the deprecation message
         to insert the name of an alternative to the deprecated
-        function.  `%(obj_type)` may be used to insert a friendly name
+        function. ``obj_type`` may be used to insert a friendly name
         for the type of object being deprecated.
 
     name : str, optional
-        The name of the deprecated function; if not provided the name
-        is automatically determined from the passed in function,
-        though this is useful in the case of renamed functions, where
-        the new function is just assigned to the name of the
-        deprecated function.  For example::
+        The name of the deprecated function or class; if not provided
+        the name is automatically determined from the passed in
+        function or class, though this is useful in the case of
+        renamed functions, where the new function is just assigned to
+        the name of the deprecated function.  For example::
 
             def new_function():
                 ...
             oldFunction = new_function
 
     alternative : str, optional
-        An alternative function that the user may use in place of the
-        deprecated function.  The deprecation warning will tell the user about
-        this alternative if provided.
+        An alternative function or class name that the user may use in
+        place of the deprecated object.  The deprecation warning will
+        tell the user about this alternative if provided.
 
     pending : bool, optional
         If True, uses a AstropyPendingDeprecationWarning instead of a
         AstropyDeprecationWarning.
+
+    obj_type : str, optional
+        The type of this object, if the automatically determined one
+        needs to be overridden.
     """
 
-    def deprecate(func, message=message, name=name, alternative=alternative,
-                  pending=pending):
-        if isinstance(func, classmethod):
+    method_types = (classmethod, staticmethod)
+
+    def deprecate_doc(old_doc, message):
+        """
+        Returns a given docstring with a deprecation message prepended
+        to it.
+        """
+        if not old_doc:
+            old_doc = ''
+        old_doc = textwrap.dedent(old_doc).strip('\n')
+        new_doc = (('\n.. deprecated:: %(since)s'
+                    '\n    %(message)s\n\n' %
+                    {'since': since, 'message': message.strip()}) + old_doc)
+        if not old_doc:
+            # This is to prevent a spurious 'unexected unindent' warning from
+            # docutils when the original docstring was blank.
+            new_doc += r'\ '
+        return new_doc
+
+    def get_function(func):
+        """
+        Given a function or classmethod (or other function wrapper type), get
+        the function object.
+        """
+        if isinstance(func, method_types):
             try:
                 func = func.__func__
             except AttributeError:
                 # classmethods in Python2.6 and below lack the __func__
                 # attribute so we need to hack around to get it
                 method = func.__get__(None, object)
-                if hasattr(method, '__func__'):
+                if isinstance(method, types.FunctionType):
+                    # For staticmethods anyways the wrapped object is just a
+                    # plain function (not a bound method or anything like that)
+                    func = method
+                elif hasattr(method, '__func__'):
                     func = method.__func__
                 elif hasattr(method, 'im_func'):
                     func = method.im_func
                 else:
                     # Nothing we can do really...  just return the original
-                    # classmethod
+                    # classmethod, etc.
                     return func
-            is_classmethod = True
+        return func
+
+    def deprecate_function(func, message):
+        """
+        Returns a wrapped function that displays an
+        ``AstropyDeprecationWarning`` when it is called.
+        """
+
+        if isinstance(func, method_types):
+            func_wrapper = type(func)
         else:
-            is_classmethod = False
+            func_wrapper = lambda f: f
+
+        func = get_function(func)
+
+        def deprecated_func(*args, **kwargs):
+            if pending:
+                category = AstropyPendingDeprecationWarning
+            else:
+                category = AstropyDeprecationWarning
+
+            warnings.warn(message, category, stacklevel=2)
+
+            return func(*args, **kwargs)
+
+        # If this is an extension function, we can't call
+        # functools.wraps on it, but we normally don't care.
+        # This crazy way to get the type of a wrapper descriptor is
+        # straight out of the Python 3.3 inspect module docs.
+        if type(func) != type(str.__dict__['__add__']):
+            deprecated_func = functools.wraps(func)(deprecated_func)
+
+        deprecated_func.__doc__ = deprecate_doc(
+            deprecated_func.__doc__, message)
+
+        return func_wrapper(deprecated_func)
+
+    def deprecate_class(cls, message):
+        """
+        Returns a wrapper class with the docstrings updated and an
+        __init__ function that will raise an
+        ``AstropyDeprectationWarning`` warning when called.
+        """
+        # Creates a new class with the same name and bases as the
+        # original class, but updates the dictionary with a new
+        # docstring and a wrapped __init__ method.  __module__ needs
+        # to be manually copied over, since otherwise it will be set
+        # to *this* module (astropy.utils.misc).
+
+        # This approach seems to make Sphinx happy (the new class
+        # looks enough like the original class), and works with
+        # extension classes (which functools.wraps does not, since
+        # it tries to modify the original class).
+
+        # We need to add a custom pickler or you'll get
+        #     Can't pickle <class ..>: it's not found as ...
+        # errors. Picklability is required for any class that is
+        # documented by Sphinx.
+
+        def __getstate__(self):
+            return super(cls, self).__getstate__()
+
+        def __setstate__(self, state):
+            return super(cls, self).__setstate__(state)
+
+        d = {
+            '__doc__': deprecate_doc(cls.__doc__, message),
+            '__init__': deprecate_function(cls.__init__, message),
+            '__module__': cls.__module__,
+            '__getstate__': __getstate__,
+            '__setstate__': __setstate__
+        }
+
+        return type(cls.__name__, (cls,), d)
+
+    def deprecate(obj, message=message, name=name, alternative=alternative,
+                  pending=pending):
+        if obj_type is None:
+            if isinstance(obj, type):
+                obj_type_name = 'class'
+            elif inspect.isfunction(obj):
+                obj_type_name = 'function'
+            elif inspect.ismethod(obj) or isinstance(obj, method_types):
+                obj_type_name = 'method'
+            else:
+                obj_type_name = 'object'
+        else:
+            obj_type_name = obj_type
 
         if not name:
-            name = func.__name__
+            name = get_function(obj).__name__
 
         altmessage = ''
         if not message or type(message) == type(deprecate):
@@ -403,40 +525,13 @@ def deprecated(since, message='', name='', alternative='', pending=False,
             'func': name,
             'name': name,
             'alternative': alternative,
-            'obj_type': obj_type}) +
+            'obj_type': obj_type_name}) +
             altmessage)
 
-        @functools.wraps(func)
-        def deprecated_func(*args, **kwargs):
-            if pending:
-                category = AstropyPendingDeprecationWarning
-            else:
-                category = AstropyDeprecationWarning
-
-            warnings.warn(message, category, stacklevel=2)
-
-            return func(*args, **kwargs)
-
-        old_doc = deprecated_func.__doc__
-        if not old_doc:
-            old_doc = ''
-        old_doc = textwrap.dedent(old_doc).strip('\n')
-        altmessage = altmessage.strip()
-        if not altmessage:
-            altmessage = message.strip()
-        new_doc = (('\n.. deprecated:: %(since)s'
-                    '\n    %(message)s\n\n' %
-                    {'since': since, 'message': altmessage.strip()}) + old_doc)
-        if not old_doc:
-            # This is to prevent a spurious 'unexected unindent' warning from
-            # docutils when the original docstring was blank.
-            new_doc += r'\ '
-
-        deprecated_func.__doc__ = new_doc
-
-        if is_classmethod:
-            deprecated_func = classmethod(deprecated_func)
-        return deprecated_func
+        if isinstance(obj, type):
+            return deprecate_class(obj, message)
+        else:
+            return deprecate_function(obj, message)
 
     if type(message) == type(deprecate):
         return deprecate(message)
@@ -451,7 +546,7 @@ def deprecated_attribute(name, since, message=None, alternative=None,
     property that will warn when the given attribute name is accessed.
     To prevent the warning (i.e. for internal code), use the private
     name for the attribute by prepending an underscore
-    (i.e. `self._name`).
+    (i.e. ``self._name``).
 
     Parameters
     ----------
@@ -464,8 +559,8 @@ def deprecated_attribute(name, since, message=None, alternative=None,
 
     message : str, optional
         Override the default deprecation message.  The format
-        specifier `%(name)s` may be used for the name of the attribute,
-        and `%(alternative)s` may be used in the deprecation message
+        specifier ``name`` may be used for the name of the attribute,
+        and ``alternative`` may be used in the deprecation message
         to insert the name of an alternative to the deprecated
         function.
 
@@ -544,12 +639,12 @@ def format_exception(msg, *args, **kwargs):
     also used to format the message.
 
     .. note::
-        This uses `sys.exc_info` to gather up the information needed to
-        fill in the formatting arguments. Python 2.x and 3.x have slightly
-        different behavior regarding `sys.exc_info` (the latter will not carry
-        it outside a handled exception), so it's not wise to use this outside of
-        an `except` clause - if it is, this will substitute '<unkown>' for the 4
-        formatting arguments.
+        This uses `sys.exc_info` to gather up the information needed to fill
+        in the formatting arguments. Python 2.x and 3.x have slightly
+        different behavior regarding `sys.exc_info` (the latter will not
+        carry it outside a handled exception), so it's not wise to use this
+        outside of an ``except`` clause - if it is, this will substitute
+        '<unkown>' for the 4 formatting arguments.
     """
 
     tb = traceback.extract_tb(sys.exc_info()[2], limit=1)
@@ -614,9 +709,9 @@ def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
     optionally open that page in a web browser.
 
     .. note::
-        You must be connected to the internet for this to function even
-        if `openinbrowser` is False, unless you provide a local version of
-        the documentation to `version` (e.g., ``file:///path/to/docs``).
+        You must be connected to the internet for this to function even if
+        ``openinbrowser`` is `False`, unless you provide a local version of
+        the documentation to ``version`` (e.g., ``file:///path/to/docs``).
 
     Parameters
     ----------
@@ -630,12 +725,13 @@ def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
         latest if you are on aren't on a release, otherwise, the version you
         are on.
     openinbrowser : bool
-        If True, the `webbrowser` package will be used to open the doc
+        If `True`, the `webbrowser` package will be used to open the doc
         page in a new web browser window.
     timeout : number, optional
         The number of seconds to wait before timing-out the query to
         the astropy documentation.  If not given, the default python
         stdlib timeout will be used.
+
     Returns
     -------
     url : str
@@ -780,7 +876,7 @@ def walk_skip_hidden(top, onerror=None, followlinks=False):
     """
     A wrapper for `os.walk` that skips hidden files and directories.
 
-    This function does not have the parameter `topdown` from
+    This function does not have the parameter ``topdown`` from
     `os.walk`: the directories must always be recursed top-down when
     using this function.
 
@@ -829,3 +925,117 @@ class JsonCustomEncoder(json.JSONEncoder):
         elif isinstance(obj, bytes):  # pragma: py3
             return obj.decode()
         return json.JSONEncoder.default(self, obj)
+
+
+def strip_accents(s):
+    """
+    Remove accents from a Unicode string.
+
+    This helps with matching "ångström" to "angstrom", for example.
+    """
+    return ''.join(
+        c for c in unicodedata.normalize('NFD', s)
+        if unicodedata.category(c) != 'Mn')
+
+
+def did_you_mean(s, candidates, n=3, cutoff=0.8):
+    """
+    When a string isn't found in a set of candidates, we can be nice
+    to provide a list of alternatives in the exception.  This
+    convenience function helps to format that part of the exception.
+
+    Parameters
+    ----------
+    s : str
+
+    candidates : sequence of str or dict of str keys
+
+    n : int
+        The maximum number of results to include.  See
+        `difflib.get_close_matches`.
+
+    cutoff : float
+        In the range [0, 1]. Possibilities that don't score at least
+        that similar to word are ignored.  See
+        `difflib.get_close_matches`.
+
+    Returns
+    -------
+    message : str
+        Returns the string "Did you mean X, Y, or Z?", or the empty
+        string if no alternatives were found.
+    """
+    if isinstance(s, six.text_type):
+        s = strip_accents(s)
+    s_lower = s.lower()
+
+    # Create a mapping from the lower case name to all capitalization
+    # variants of that name.
+    candidates_lower = {}
+    for candidate in candidates:
+        candidate_lower = candidate.lower()
+        candidates_lower.setdefault(candidate_lower, [])
+        candidates_lower[candidate_lower].append(candidate)
+
+    # The heuristic here is to first try "singularizing" the word.  If
+    # that doesn't match anything use difflib to find close matches in
+    # original, lower and upper case.
+    if s_lower.endswith('s') and s_lower[:-1] in candidates_lower:
+        matches = [s_lower[:-1]]
+    else:
+        matches = difflib.get_close_matches(
+            s_lower, candidates_lower, n=n, cutoff=cutoff)
+
+    if len(matches):
+        capitalized_matches = set()
+        for match in matches:
+            capitalized_matches.update(candidates_lower[match])
+
+        matches = sorted(capitalized_matches)
+
+        if len(matches) == 1:
+            matches = matches[0]
+        else:
+            matches = ', '.join(matches[:-1]) + ' or ' + matches[-1]
+        return 'Did you mean {0}?'.format(matches)
+
+    return ''
+
+
+class InheritDocstrings(type):
+    """
+    This metaclass makes methods of a class automatically have their
+    docstrings filled in from the methods they override in the base
+    class.
+
+    If the class uses multiple inheritance, the docstring will be
+    chosen from the first class in the bases list, in the same way as
+    methods are normally resolved in Python.  If this results in
+    selecting the wrong docstring, the docstring will need to be
+    explicitly included on the method.
+
+    For example::
+
+        >>> from astropy.utils.misc import InheritDocstrings
+        >>> from astropy.extern import six
+        >>> @six.add_metaclass(InheritDocstrings)
+        ... class A(object):
+        ...     def wiggle(self):
+        ...         "Wiggle the thingamajig"
+        ...         pass
+        >>> class B(A):
+        ...     def wiggle(self):
+        ...         pass
+        >>> B.wiggle.__doc__
+        u'Wiggle the thingamajig'
+    """
+    def __init__(cls, name, bases, dct):
+        for key, val in six.iteritems(dct):
+            if (inspect.isfunction(val) and
+                not key.startswith('_') and
+                val.__doc__ is None):
+                for base in cls.__mro__[1:]:
+                    super_method = getattr(base, key, None)
+                    if super_method is not None:
+                        val.__doc__ = super_method.__doc__
+                        break
