@@ -142,6 +142,7 @@ Wcs_init(
         return -1;
       }
 
+      Py_XDECREF(self->py_det2im[i]);
       self->py_det2im[i] = py_det2im[i];
       self->x.det2im[i] = &(((PyDistLookup*)py_det2im[i])->x);
     }
@@ -155,6 +156,7 @@ Wcs_init(
       return -1;
     }
 
+    Py_XDECREF(self->py_sip);
     self->py_sip = py_sip;
     self->x.sip = &(((PySip*)py_sip)->x);
   }
@@ -168,6 +170,7 @@ Wcs_init(
         return -1;
       }
 
+      Py_XDECREF(self->py_distortion_lookup[i]);
       self->py_distortion_lookup[i] = py_distortion_lookup[i];
       self->x.cpdis[i] = &(((PyDistLookup*)py_distortion_lookup[i])->x);
     }
@@ -181,6 +184,7 @@ Wcs_init(
       return -1;
     }
 
+    Py_XDECREF(self->py_wcsprm);
     self->py_wcsprm = py_wcsprm;
     self->x.wcs = &(((PyWcsprm*)py_wcsprm)->x);
   }
