@@ -161,7 +161,7 @@ wcsprm_fix_values(
 
   unsigned int naxis = (unsigned int)x->naxis;
 
-  value_fixer(x->cd, 4);
+  value_fixer(x->cd, naxis * naxis);
   value_fixer(x->cdelt, naxis);
   value_fixer(x->crder, naxis);
   value_fixer(x->crota, naxis);
@@ -945,7 +945,6 @@ parse_unsafe_unit_conversion_spec(
 
   *ctrl = 0;
 
-  p = arg;
   for (p = arg; *p != '\0'; ++p) {
     switch (*p) {
     case 's':
