@@ -45,9 +45,13 @@ from . import _docutil as __
 try:
     from . import _wcs
 except ImportError:
-    _wcs = None
+    if not _ASTROPY_SETUP_:
+        raise
+    else:
+        _wcs = None
 
 from ..utils import deprecated, deprecated_attribute
+from ..utils.compat import possible_filename
 from ..utils.exceptions import AstropyWarning, AstropyUserWarning, AstropyDeprecationWarning
 
 if _wcs is not None:
@@ -57,8 +61,8 @@ if _wcs is not None:
 
 
 __all__ = ['FITSFixedWarning', 'WCS', 'find_all_wcs',
-           'DistortionLookupTable', 'Sip', 'Tabprm', 'UnitConverter',
-           'Wcsprm', 'WCSBase', 'validate', 'WcsError', 'SingularMatrixError',
+           'DistortionLookupTable', 'Sip', 'Tabprm', 'Wcsprm',
+           'WCSBase', 'validate', 'WcsError', 'SingularMatrixError',
            'InconsistentAxisTypesError', 'InvalidTransformError',
            'InvalidCoordinateError', 'NoSolutionError',
            'InvalidSubimageSpecificationError',
@@ -70,7 +74,6 @@ if _wcs is not None:
     WCSBase = _wcs._Wcs
     DistortionLookupTable = _wcs.DistortionLookupTable
     Sip = _wcs.Sip
-    UnitConverter = _wcs.UnitConverter
     Wcsprm = _wcs.Wcsprm
     Tabprm = _wcs.Tabprm
     WcsError = _wcs.WcsError
@@ -91,16 +94,11 @@ if _wcs is not None:
             key.startswith('WCSHDO')):
             locals()[key] = val
             __all__.append(key)
-
-    UnitConverter = deprecated(
-        '0.2', name='UnitConverter', alternative='astropy.units')(
-            UnitConverter)
 else:
     WCSBase = object
     Wcsprm = object
     DistortionLookupTable = object
     Sip = object
-    UnitConverter = object
     Tabprm = object
     WcsError = None
     SingularMatrixError = None
@@ -220,11 +218,17 @@ class WCS(WCSBase):
         specified columns.  If `None`, there is no restriction.
 
     fix : bool, optional
-        When `True` (default), call `~astropy.wcs._wcs.Wcsprm.fix` on
+        When `True` (default), call `~astropy.wcs.Wcsprm.fix` on
         the resulting object to fix any non-standard uses in the
         header.  `FITSFixedWarning` Warnings will be emitted if any
         changes were made.
 
+    translate_units : str, optional
+        Specify which potentially unsafe translations of non-standard
+        unit strings to perform.  By default, performs none.  See
+        `WCS.fix` for more information about this parameter.  Only
+        effective when ``fix`` is `True`.
+
     Raises
     ------
     MemoryError
@@ -243,15 +247,14 @@ class WCS(WCSBase):
     Notes
     -----
 
-    1. astropy.wcs supports arbitrary *n* dimensions for the core WCS
-       (the transformations handled by WCSLIB).  However, the Paper IV
-       lookup table and SIP distortions must be two dimensional.
-       Therefore, if you try to create a WCS object where the core WCS
-       has a different number of dimensions than 2 and that object
-       also contains a Paper IV lookup table or SIP distortion, a
-       `ValueError` exception will be raised.  To avoid this, consider
-       using the *naxis* kwarg to select two dimensions from the core
-       WCS.
+    1. astropy.wcs supports arbitrary *n* dimensions for the core WCS (the
+       transformations handled by WCSLIB).  However, the Paper IV lookup
+       table and SIP distortions must be two dimensional.  Therefore, if you
+       try to create a WCS object where the core WCS has a different number
+       of dimensions than 2 and that object also contains a Paper IV lookup
+       table or SIP distortion, a `~.exceptions.ValueError` exception will
+       be raised.  To avoid this, consider using the *naxis* kwarg to select
+       two dimensions from the core WCS.
 
     2. The number of coordinate axes in the transformation is not
        determined directly from the ``NAXIS`` keyword but instead from
@@ -270,17 +273,22 @@ class WCS(WCSBase):
        coordinate representation, then no coordinate description is
        constructed for it.
 
-       The number of axes, which is set as the `naxis` member, may
+       The number of axes, which is set as the ``naxis`` member, may
        differ for different coordinate representations of the same
        image.
 
     3. When the header includes duplicate keywords, in most cases the
        last encountered is used.
+
+    4. `~astropy.wcs.Wcsprm.set` is called immediately after
+       construction, so any invalid keywords or transformations will
+       be raised by the constructor, not when subsequently calling a
+       transformation method.
     """
 
     def __init__(self, header=None, fobj=None, key=' ', minerr=0.0,
                  relax=True, naxis=None, keysel=None, colsel=None,
-                 fix=True):
+                 fix=True, translate_units='', _do_set=True):
         close_fds = []
 
         if header is None:
@@ -298,7 +306,8 @@ class WCS(WCSBase):
 
             if isinstance(header, (six.text_type, six.binary_type)):
                 try:
-                    is_path = os.path.exists(header)
+                    is_path = (possible_filename(header) and
+                               os.path.exists(header))
                 except (IOError, ValueError):
                     is_path = False
 
@@ -383,7 +392,10 @@ naxis kwarg.
         WCSBase.__init__(self, sip, cpdis, wcsprm, det2im)
 
         if fix:
-            self.fix()
+            self.fix(translate_units=translate_units)
+
+        if _do_set:
+            self.wcs.set()
 
         for fd in close_fds:
             fd.close()
@@ -480,14 +492,47 @@ naxis kwarg.
         warnings.warn("Removed redundant SCAMP distortion parameters " +
             "because SIP parameters are also present", FITSFixedWarning)
 
-    def fix(self):
+    def fix(self, translate_units='', naxis=None):
         """
         Perform the fix operations from wcslib, and warn about any
         changes it has made.
+
+        Parameters
+        ----------
+        translate_units : str, optional
+            Specify which potentially unsafe translations of
+            non-standard unit strings to perform.  By default,
+            performs none.
+
+            Although ``"S"`` is commonly used to represent seconds,
+            its translation to ``"s"`` is potentially unsafe since the
+            standard recognizes ``"S"`` formally as Siemens, however
+            rarely that may be used.  The same applies to ``"H"`` for
+            hours (Henry), and ``"D"`` for days (Debye).
+
+            This string controls what to do in such cases, and is
+            case-insensitive.
+
+            - If the string contains ``"s"``, translate ``"S"`` to
+              ``"s"``.
+
+            - If the string contains ``"h"``, translate ``"H"`` to
+              ``"h"``.
+
+            - If the string contains ``"d"``, translate ``"D"`` to
+              ``"d"``.
+
+            Thus ``''`` doesn't do any unsafe translations, whereas
+            ``'shd'`` does all of them.
+
+        naxis : int array[naxis], optional
+            Image axis lengths.  If this array is set to zero or
+            ``None``, then `~astropy.wcs.Wcsprm.cylfix` will not be
+            invoked.
         """
         if self.wcs is not None:
             self._fix_scamp()
-            fixes = self.wcs.fix()
+            fixes = self.wcs.fix(translate_units, naxis)
             for key, val in six.iteritems(fixes):
                 if val != "No change":
                     warnings.warn(
@@ -495,7 +540,12 @@ naxis kwarg.
                         format(key, val),
                         FITSFixedWarning)
 
+    @deprecated("0.4", name="calcFootprint", alternative="calc_footprint")
     def calcFootprint(self, header=None, undistort=True, axes=None):
+        return self.calc_footprint(header=header, undistort=undistort, axes=axes,
+                                   center=True)
+
+    def calc_footprint(self, header=None, undistort=True, axes=None, center=True):
         """
         Calculates the footprint of the image on the sky.
 
@@ -505,7 +555,7 @@ naxis kwarg.
 
         Parameters
         ----------
-        header : astropy.io.fits header object, optional
+        header : `~astropy.io.fits.Header` object, optional
 
         undistort : bool, optional
             If `True`, take SIP and distortion lookup table into
@@ -517,9 +567,13 @@ naxis kwarg.
             keywords from the header that was used to create this
             `WCS` object.
 
+        center : bool, optional
+            If `True` use the center of the pixel, otherwise use the corner.
+
         Returns
         -------
         coord : (4, 2) array of (*x*, *y*) coordinates.
+            The order is counter-clockwise starting with the bottom left corner.
         """
         if axes is not None:
             naxis1, naxis2 = axes
@@ -537,18 +591,20 @@ naxis kwarg.
                 naxis1 = header.get('NAXIS1', None)
                 naxis2 = header.get('NAXIS2', None)
 
-        corners = np.zeros(shape=(4, 2), dtype=np.float64)
         if naxis1 is None or naxis2 is None:
             return None
 
-        corners[0, 0] = 1.
-        corners[0, 1] = 1.
-        corners[1, 0] = 1.
-        corners[1, 1] = naxis2
-        corners[2, 0] = naxis1
-        corners[2, 1] = naxis2
-        corners[3, 0] = naxis1
-        corners[3, 1] = 1.
+        if center == True:
+            corners = np.array([[1, 1],
+                                [1, naxis2],
+                                [naxis1, naxis2],
+                                [naxis1, 1]], dtype = np.float64)
+        else:
+            corners = np.array([[0.5, 0.5],
+                                [0.5, naxis2 + 0.5],
+                                [naxis1 + 0.5, naxis2 + 0.5],
+                                [naxis1 + 0.5, 0.5]], dtype = np.float64)
+
         if undistort:
             return self.all_pix2world(corners, 1)
         else:
@@ -611,10 +667,8 @@ naxis kwarg.
 
     def _read_d2im_old_format(self, header, fobj, axiscorr):
         warnings.warn("The use of ``AXISCORR`` for D2IM correction has been deprecated."
-                      "The new style of this correction is described at"
-                      ""
-                      "PyWCS will read in files with ``AXISCORR`` but to_fits() will write"
-                      "out files in the new style",
+                      "`~astropy.wcs` will read in files with ``AXISCORR`` but ``to_fits()`` will write "
+                      "out files without it.",
                       AstropyDeprecationWarning)
         cpdis = [None, None]
         crpix = [0., 0.]
@@ -912,7 +966,7 @@ naxis kwarg.
             raise ValueError(
                 "WCS does not have longitude type of 'RA', therefore " +
                 "(ra, dec) data can not be used as input")
-        if self.wcs.lattype != 'DEC':
+        if self.wcs.lattyp != 'DEC':
             raise ValueError(
                 "WCS does not have longitude type of 'DEC', therefore " +
                 "(ra, dec) data can not be used as input")
@@ -942,7 +996,7 @@ naxis kwarg.
             raise ValueError(
                 "WCS does not have longitude type of 'RA', therefore " +
                 "(ra, dec) data can not be returned")
-        if self.wcs.lattype != 'DEC':
+        if self.wcs.lattyp != 'DEC':
             raise ValueError(
                 "WCS does not have longitude type of 'DEC', therefore " +
                 "(ra, dec) data can not be returned")
@@ -989,7 +1043,7 @@ naxis kwarg.
                 xy = self._denormalize_sky(xy)
             output = func(xy, origin)
             if ra_dec_order and sky == 'output':
-                output = self._normalize_sky_output(output)
+                output = self._normalize_sky(output)
                 return (output[:, 0].reshape(axes[0].shape),
                         output[:, 1].reshape(axes[0].shape))
             return [output[:, i].reshape(axes[0].shape)
@@ -1073,7 +1127,7 @@ naxis kwarg.
         Notes
         -----
         The order of the axes for the result is determined by the
-        `CTYPEia` keywords in the FITS header, therefore it may not
+        ``CTYPEia`` keywords in the FITS header, therefore it may not
         always be of the form (*ra*, *dec*).  The
         `~astropy.wcs.Wcsprm.lat`, `~astropy.wcs.Wcsprm.lng`,
         `~astropy.wcs.Wcsprm.lattyp` and `~astropy.wcs.Wcsprm.lngtyp`
@@ -1108,10 +1162,6 @@ naxis kwarg.
                    __.RA_DEC_ORDER(8),
                    __.RETURNS('sky coordinates, in degrees', 8))
 
-    @deprecated("0.0", name="all_pix2sky", alternative="all_pix2world")
-    def all_pix2sky(self, *args, **kwargs):
-        return self.all_pix2world(*args, **kwargs)
-
     def wcs_pix2world(self, *args, **kwargs):
         if self.wcs is None:
             raise ValueError("No basic WCS settings were created.")
@@ -1137,11 +1187,6 @@ naxis kwarg.
 
         {1}
 
-        tolerance : float, optional
-            Tolerance of solution. Iteration terminates when the iterative
-            solver estimates that the true solution is within this many pixels
-            current estimate. Default value is 1e-6 (pixels).
-
         Returns
         -------
 
@@ -1176,7 +1221,7 @@ naxis kwarg.
         Notes
         -----
         The order of the axes for the result is determined by the
-        `CTYPEia` keywords in the FITS header, therefore it may not
+        ``CTYPEia`` keywords in the FITS header, therefore it may not
         always be of the form (*ra*, *dec*).  The
         `~astropy.wcs.Wcsprm.lat`, `~astropy.wcs.Wcsprm.lng`,
         `~astropy.wcs.Wcsprm.lattyp` and `~astropy.wcs.Wcsprm.lngtyp`
@@ -1186,17 +1231,8 @@ naxis kwarg.
                    __.RA_DEC_ORDER(8),
                    __.RETURNS('world coordinates, in degrees', 8))
 
-    @deprecated("0.0", name="wcs_pix2sky", alternative="wcs_pix2world")
-    def wcs_pix2sky(self, *args, **kwargs):
-        return self.wcs_pix2world(*args, **kwargs)
-
     def _all_world2pix(self, world, origin, tolerance, **kwargs):
-        try:
-            import scipy.optimize
-        except ImportError:
-            raise ImportError(
-                "You must have Scipy installed to use this method. " +
-                "See <http://www.scipy.org>.")
+        import scipy.optimize
         pix = []
         for i in range(len(world)):
             x0 = self.wcs_world2pix(np.atleast_2d(world[i]), origin,
@@ -1235,6 +1271,11 @@ naxis kwarg.
 
         {1}
 
+        tolerance : float, optional
+            Tolerance of solution. Iteration terminates when the iterative
+            solver estimates that the true solution is within this many pixels
+            current estimate. Default value is 1e-6 (pixels).
+
         Returns
         -------
 
@@ -1243,7 +1284,7 @@ naxis kwarg.
         Notes
         -----
         The order of the axes for the input world array is determined by
-        the `CTYPEia` keywords in the FITS header, therefore it may
+        the ``CTYPEia`` keywords in the FITS header, therefore it may
         not always be of the form (*ra*, *dec*).  The
         `~astropy.wcs.Wcsprm.lat`, `~astropy.wcs.Wcsprm.lng`,
         `~astropy.wcs.Wcsprm.lattyp` and `~astropy.wcs.Wcsprm.lngtyp`
@@ -1306,7 +1347,7 @@ naxis kwarg.
         Notes
         -----
         The order of the axes for the input world array is determined by
-        the `CTYPEia` keywords in the FITS header, therefore it may
+        the ``CTYPEia`` keywords in the FITS header, therefore it may
         not always be of the form (*ra*, *dec*).  The
         `~astropy.wcs.Wcsprm.lat`, `~astropy.wcs.Wcsprm.lng`,
         `~astropy.wcs.Wcsprm.lattyp` and `~astropy.wcs.Wcsprm.lngtyp`
@@ -1341,10 +1382,6 @@ naxis kwarg.
                    __.RA_DEC_ORDER(8),
                    __.RETURNS('pixel coordinates', 8))
 
-    @deprecated("0.0", name="wcs_sky2pix", alternative="wcs_world2pix")
-    def wcs_sky2pix(self, *args, **kwargs):
-        return self.wcs_world2pix(*args, **kwargs)
-
     def pix2foc(self, *args):
         return self._array_converter(self._pix2foc, None, *args)
     pix2foc.__doc__ = """
@@ -1352,6 +1389,9 @@ naxis kwarg.
         `SIP`_ polynomial distortion convention and `Paper IV`_
         table-lookup distortion correction.
 
+        The output is in absolute pixel coordinates, not relative to
+        ``CRPIX``.
+
         Parameters
         ----------
 
@@ -1378,6 +1418,9 @@ naxis kwarg.
         Convert pixel coordinates to focal plane coordinates using
         `Paper IV`_ table-lookup distortion correction.
 
+        The output is in absolute pixel coordinates, not relative to
+        ``CRPIX``.
+
         Parameters
         ----------
 
@@ -1404,6 +1447,9 @@ naxis kwarg.
         Convert detector coordinates to image plane coordinates using
         `Paper IV`_ table-lookup distortion correction.
 
+        The output is in absolute pixel coordinates, not relative to
+        ``CRPIX``.
+
         Parameters
         ----------
 
@@ -1437,6 +1483,8 @@ naxis kwarg.
         Convert pixel coordinates to focal plane coordinates using the
         `SIP`_ polynomial distortion convention.
 
+        The output is in pixel coordinates, relative to ``CRPIX``.
+
         `Paper IV`_ table lookup distortion correction is not applied,
         even if that information existed in the FITS file that
         initialized this :class:`~astropy.wcs.WCS` object.  To correct
@@ -1591,7 +1639,7 @@ naxis kwarg.
 
           2. Deprecated (e.g. ``CROTAn``) or non-standard usage will
              be translated to standard (this is partially dependent on
-             whether `fix` was applied).
+             whether ``fix`` was applied).
 
           3. Quantities will be converted to the units used internally,
              basically SI with the addition of degrees.
@@ -1668,17 +1716,10 @@ naxis kwarg.
         f.write(comments)
         f.write('linear\n')
         f.write('polygon(')
-        self.footprint.tofile(f, sep=',')
+        self.calc_footprint().tofile(f, sep=',')
         f.write(') # color={0}, width={1:d} \n'.format(color, width))
         f.close()
 
-    naxis1 = deprecated_attribute('naxis1', '0.2')
-    naxis2 = deprecated_attribute('naxis2', '0.2')
-
-    @deprecated('0.2', message='This method should not be public')
-    def get_naxis(self, header=None):
-        return self._get_naxis(header=header)
-
     def _get_naxis(self, header=None):
         self._naxis1 = 0
         self._naxis2 = 0
@@ -1696,32 +1737,41 @@ naxis kwarg.
         self.wcs.cd = new_cd
 
     def printwcs(self):
-        """
-        Temporary function for internal use.
-        """
-        print('WCS Keywords\n')
-        if hasattr(self.wcs, 'cd'):
-            print('CD_11  CD_12: {!r} {!r}'.format(
-                self.wcs.cd[0, 0], self.wcs.cd[0, 1]))
-            print('CD_21  CD_22: {!r} {!r}'.format(
-                self.wcs.cd[1, 0], self.wcs.cd[1, 1]))
-        else:
-            print('PC_11  PC_12: {!r} {!r}'.format(
-                self.wcs.pc[0, 0], self.wcs.pc[0, 1]))
-            print('PC_21  PC_22: {!r} {!r}'.format(
-                self.wcs.pc[1, 0], self.wcs.pc[1, 1]))
-        print('CRVAL    : {!r} {!r}'.format(
-            self.wcs.crval[0], self.wcs.crval[1]))
-        print('CRPIX    : {!r} {!r}'.format(
-            self.wcs.crpix[0], self.wcs.crpix[1]))
-        print('CDELT    : {!r} {!r}'.format(
-            self.wcs.cdelt[0], self.wcs.cdelt[1]))
-        print('NAXIS    : {!r} {!r}'.format(
-            self.naxis1, self.naxis2))
+        print("WCS Keywords\n")
+        print("Number of WCS axes: {0!r}".format(self.naxis))
+        sfmt = ': ' +  "".join(["{"+"{0}".format(i)+"!r}  " for i in range(self.naxis)])
+
+        s = 'CTYPE ' + sfmt
+        print(s.format(*self.wcs.ctype))
+
+        s = 'CRVAL ' + sfmt
+        print(s.format(*self.wcs.crval))
+
+        s = 'CRPIX ' + sfmt
+        print(s.format(*self.wcs.crpix))
+
+        if hasattr(self.wcs, 'pc'):
+            for i in range(self.naxis):
+                s = ''
+                for j in range(self.naxis):
+                    s += ''.join(['PC', str(i+1), '_', str(j+1), ' '])
+                s += sfmt
+                print(s.format(*self.wcs.pc[i]))
+            s = 'CDELT ' + sfmt
+            print(s.format(*self.wcs.cdelt))
+        elif hasattr(self.wcs, 'cd'):
+            for i in range(self.naxis):
+                s = ''
+                for j in range(self.naxis):
+                    s += "".join(['CD', str(i+1), '_', str(j+1), ' '])
+                s += sfmt
+                print(s.format(*self.wcs.cd[i]))
+
+        print('NAXIS    : {0!r} {1!r}'.format(self._naxis1, self._naxis2))
 
     def get_axis_types(self):
         """
-        Similar to `self.wcsprm.axis_types <_wcs.Wcsprm.axis_types>`
+        Similar to `self.wcsprm.axis_types <astropy.wcs.Wcsprm.axis_types>`
         but provides the information in a more Python-friendly format.
 
         Returns
@@ -1830,6 +1880,140 @@ naxis kwarg.
         return (__WCS_unpickle__,
                 (self.__class__, self.__dict__, buffer.getvalue(),))
 
+    def dropaxis(self, dropax):
+        """
+        Remove an axis from the WCS.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The WCS with naxis to be chopped to naxis-1
+        dropax : int
+            The index of the WCS to drop, counting from 0 (i.e., python convention,
+            not FITS convention)
+
+        Returns
+        -------
+        A new `~astropy.wcs.WCS` instance with one axis fewer
+        """
+        inds = list(range(self.wcs.naxis))
+        inds.pop(dropax)
+
+        # axis 0 has special meaning to sub
+        # if wcs.wcs.ctype == ['RA','DEC','VLSR'], you want
+        # wcs.sub([1,2]) to get 'RA','DEC' back
+        return self.sub([i+1 for i in inds])
+
+    def swapaxes(self, ax0, ax1):
+        """
+        Swap axes in a WCS.
+
+        Parameters
+        ----------
+        wcs: `~astropy.wcs.WCS`
+            The WCS to have its axes swapped
+        ax0: int
+        ax1: int
+            The indices of the WCS to be swapped, counting from 0 (i.e., python
+            convention, not FITS convention)
+
+        Returns
+        -------
+        A new `~astropy.wcs.WCS` instance with the same number of axes, but two
+        swapped
+        """
+        inds = list(range(self.wcs.naxis))
+        inds[ax0],inds[ax1] = inds[ax1],inds[ax0]
+
+        return self.sub([i+1 for i in inds])
+
+    def reorient_celestial_first(self):
+        """
+        Reorient the WCS such that the celestial axes are first, followed by
+        the spectral axis, followed by any others.
+        Assumes at least celestial axes are present.
+        """
+        return self.sub([WCSSUB_CELESTIAL, WCSSUB_SPECTRAL, WCSSUB_STOKES])
+
+    def slice(self, view, numpy_order=True):
+        """
+        Slice a WCS instance using a Numpy slice. The order of the slice should
+        be reversed (as for the data) compared to the natural WCS order.
+
+        Parameters
+        ----------
+        view : tuple
+            A tuple containing the same number of slices as the WCS system.
+            The ``step`` method, the third argument to a slice, is not
+            presently supported.
+        numpy_order : bool
+            Use numpy order, i.e. slice the WCS so that an identical slice
+            applied to a numpy array will slice the array and WCS in the same
+            way. If set to `False`, the WCS will be sliced in FITS order,
+            meaning the first slice will be applied to the *last* numpy index
+            but the *first* WCS axis.
+
+        Returns
+        -------
+        wcs_new : `~astropy.wcs.WCS`
+            A new resampled WCS axis
+        """
+        if hasattr(view, '__len__') and len(view) > self.wcs.naxis:
+            raise ValueError("Must have # of slices <= # of WCS axes")
+        elif not hasattr(view, '__len__'): # view MUST be an iterable
+            view = [view]
+
+        if not all([isinstance(x, slice) for x in view]):
+            raise ValueError("Cannot downsample a WCS with indexing.  Use "
+                             "wcs.sub or wcs.dropaxis if you want to remove "
+                             "axes.")
+
+        wcs_new = self.deepcopy()
+        for i, iview in enumerate(view):
+            if iview.start is not None:
+                if numpy_order:
+                    wcs_index = self.wcs.naxis - 1 - i
+                else:
+                    wcs_index = i
+
+                if iview.step not in (None, 1):
+                    crpix = self.wcs.crpix[wcs_index]
+                    cdelt = self.wcs.cdelt[wcs_index]
+                    # equivalently (keep this comment so you can compare eqns):
+                    # wcs_new.wcs.crpix[wcs_index] =
+                    # (crpix - iview.start)*iview.step + 0.5 - iview.step/2.
+                    crp = ((crpix - iview.start - 1.)/iview.step
+                           + 0.5 + 1./iview.step/2.)
+                    wcs_new.wcs.crpix[wcs_index] = crp
+                    wcs_new.wcs.cdelt[wcs_index] = cdelt * iview.step
+                else:
+                    wcs_new.wcs.crpix[wcs_index] -= iview.start
+        return wcs_new
+
+    def __getitem__(self, item):
+        # "getitem" is a shortcut for self.slice; it is very limited
+        # there is no obvious and unambiguous interpretation of wcs[1,2,3]
+        # We COULD allow wcs[1] to link to wcs.sub([2])
+        # (wcs[i] -> wcs.sub([i+1])
+        return self.slice(item)
+
+    @property
+    def axis_type_names(self):
+        """
+        World names for each coordinate axis
+
+        Returns
+        -------
+        A list of names along each axis
+        """
+        names = list(self.wcs.cname)
+        types = self.wcs.ctype
+        for i in range(len(names)):
+            if len(names[i]) > 0:
+                continue
+            names[i] = types[i].split('-')[0]
+        return names
+
 
 def __WCS_unpickle__(cls, dct, fits_data):
     """
@@ -1848,6 +2032,7 @@ def __WCS_unpickle__(cls, dct, fits_data):
 
 
 def find_all_wcs(header, relax=True, keysel=None, fix=True,
+                 translate_units='',
                  _do_set=True):
     """
     Find all the WCS transformations in the given header.
@@ -1889,11 +2074,17 @@ def find_all_wcs(header, relax=True, keysel=None, fix=True,
         'pixel'.
 
     fix : bool, optional
-        When `True` (default), call `~astropy.wcs._wcs.Wcsprm.fix` on
+        When `True` (default), call `~astropy.wcs.Wcsprm.fix` on
         the resulting objects to fix any non-standard uses in the
         header.  `FITSFixedWarning` warnings will be emitted if any
         changes were made.
 
+    translate_units : str, optional
+        Specify which potentially unsafe translations of non-standard
+        unit strings to perform.  By default, performs none.  See
+        `WCS.fix` for more information about this parameter.  Only
+        effective when ``fix`` is `True`.
+
     Returns
     -------
     wcses : list of `WCS` objects
@@ -1918,12 +2109,12 @@ def find_all_wcs(header, relax=True, keysel=None, fix=True,
 
     result = []
     for wcsprm in wcsprms:
-        subresult = WCS(fix=False)
+        subresult = WCS(fix=False, _do_set=False)
         subresult.wcs = wcsprm
         result.append(subresult)
 
         if fix:
-            subresult.fix()
+            subresult.fix(translate_units)
 
         if _do_set:
             subresult.wcs.set()
@@ -2016,10 +2207,13 @@ def validate(source):
                 hdu.header, relax=True, fix=False, _do_set=False)
 
         for wcs in wcses:
-            wcs_results = _WcsValidateWcsResult(wcs.wcs.name)
+            wcs_results = _WcsValidateWcsResult(wcs.wcs.alt)
             hdu_results.append(wcs_results)
 
-            del __warningregistry__
+            try:
+                del __warningregistry__
+            except NameError:
+                pass
 
             with warnings.catch_warnings(record=True) as warning_lines:
                 warnings.resetwarnings()
@@ -2028,8 +2222,8 @@ def validate(source):
 
                 try:
                     WCS(hdu.header,
-                        key=wcs.wcs.name or ' ',
-                        relax=True, fix=True)
+                        key=wcs.wcs.alt or ' ',
+                        relax=True, fix=True, _do_set=False)
                 except WcsError as e:
                     wcs_results.append(str(e))
 
