@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.19 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2013, Mark Calabretta
+  WCSLIB 4.23 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2014, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,7 +22,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: wcs.c,v 4.19 2013/09/29 14:17:51 mcalabre Exp $
+  $Id: wcs.c,v 4.23 2014/05/11 04:09:38 mcalabre Exp $
 *===========================================================================*/
 
 #include <math.h>
@@ -515,6 +515,7 @@ int wcsini(int alloc, int naxis, struct wcsprm *wcs)
   memset(wcs->dateavg, 0, 72);
   wcs->mjdobs     = UNDEFINED;
   wcs->mjdavg     = UNDEFINED;
+  wcs->velangl    = UNDEFINED;
 
   wcs->ntab = 0;
   wcs->tab  = 0x0;
@@ -548,8 +549,9 @@ int wcssub(
   static const char *function = "wcssub";
 
   char *c, ctypei[16];
-  int  axis, cubeface, dealloc, dummy, i, itab, j, k, latitude, longitude, m,
-       *map = 0x0, msub, naxis, npv, nps, other, spectral, status, stokes;
+  int  axis, cubeface, dealloc, dummy, i, itab, *itmp = 0x0, j, k, latitude,
+       longitude, m, *map, msub, naxis, npv, nps, ntmp, other, spectral,
+       status, stokes;
   const double *srcp;
   double *dstp;
   struct tabprm *tabp;
@@ -564,10 +566,6 @@ int wcssub(
       "naxis must be positive (got %d)", naxis);
   }
 
-  if (!(map = calloc(naxis, sizeof(int)))) {
-    return wcserr_set(WCS_ERRMSG(WCSERR_MEMORY));
-  }
-
   if (nsub == 0x0) {
     nsub = &dummy;
     *nsub = naxis;
@@ -575,10 +573,16 @@ int wcssub(
     *nsub = naxis;
   }
 
+  /* Allocate enough temporary storage to hold either axes[] or map[].*/
+  ntmp = (*nsub <= naxis) ? naxis : *nsub;
+  if (!(itmp = calloc(ntmp, sizeof(int)))) {
+    return wcserr_set(WCS_ERRMSG(WCSERR_MEMORY));
+  }
+
   if ((dealloc = (axes == 0x0))) {
     /* Construct an index array. */
     if (!(axes = calloc(naxis, sizeof(int)))) {
-      free(map);
+      free(itmp);
       return wcserr_set(WCS_ERRMSG(WCSERR_MEMORY));
     }
 
@@ -678,25 +682,25 @@ int wcssub(
 
         /* This axis is wanted, but has it already been added? */
         for (k = 0; k < msub; k++) {
-          if (map[k] == i+1) {
+          if (itmp[k] == i+1) {
             break;
           }
         }
-        if (k == msub) map[msub++] = i+1;
+        if (k == msub) itmp[msub++] = i+1;
       }
 
     } else if (0 < axis && axis <= naxis) {
       /* Check that the requested axis has not already been added. */
       for (k = 0; k < msub; k++) {
-        if (map[k] == axis) {
+        if (itmp[k] == axis) {
           break;
         }
       }
-      if (k == msub) map[msub++] = axis;
+      if (k == msub) itmp[msub++] = axis;
 
     } else if (axis == 0) {
       /* Graft on a new axis. */
-      map[msub++] = 0;
+      itmp[msub++] = 0;
 
     } else {
       status = wcserr_set(WCS_ERRMSG(WCSERR_BAD_SUBIMAGE));
@@ -710,7 +714,7 @@ int wcssub(
   }
 
   for (i = 0; i < *nsub; i++) {
-    axes[i] = map[i];
+    axes[i] = itmp[i];
   }
 
 
@@ -719,6 +723,7 @@ int wcssub(
      axes[i] == 0 means to create a new axis,
       map[i] == j means that input axis i+1 goes to output axis j,
       map[i] == 0 means that input axis i+1 is not used. */
+  map = itmp;
   for (i = 0; i < naxis; i++) {
     map[i] = 0;
   }
@@ -730,16 +735,14 @@ int wcssub(
   }
 
   /* Check that the subimage coordinate system is separable. */
-  if (*nsub < naxis) {
-    srcp = wcssrc->pc;
-    for (i = 0; i < naxis; i++) {
-      for (j = 0; j < naxis; j++) {
-        if (*(srcp++) == 0.0 || j == i) continue;
+  srcp = wcssrc->pc;
+  for (i = 0; i < naxis; i++) {
+    for (j = 0; j < naxis; j++) {
+      if (*(srcp++) == 0.0 || j == i) continue;
 
-        if ((map[i] == 0) != (map[j] == 0)) {
-          status = wcserr_set(WCS_ERRMSG(WCSERR_NON_SEPARABLE));
-          goto cleanup;
-        }
+      if ((map[i] == 0) != (map[j] == 0)) {
+        status = wcserr_set(WCS_ERRMSG(WCSERR_NON_SEPARABLE));
+        goto cleanup;
       }
     }
   }
@@ -965,7 +968,7 @@ int wcssub(
 
 
 cleanup:
-  if (map) free(map);
+  if (itmp) free(itmp);
   if (dealloc) {
     free(axes);
   }
@@ -977,6 +980,148 @@ cleanup:
 
 /*--------------------------------------------------------------------------*/
 
+int wcscompare(
+  int cmp,
+  const struct wcsprm *wcs1,
+  const struct wcsprm *wcs2,
+  int *equal)
+
+{
+  int i, j, naxis, naxis2;
+  double diff;
+  int tab_equal;
+  int status;
+
+  if (wcs1  == 0x0) return WCSERR_NULL_POINTER;
+  if (wcs2  == 0x0) return WCSERR_NULL_POINTER;
+  if (equal == 0x0) return WCSERR_NULL_POINTER;
+
+  *equal = 0;
+
+  if (wcs1->naxis != wcs2->naxis) {
+    return 0;
+  }
+
+  naxis = wcs1->naxis;
+  naxis2 = wcs1->naxis*wcs1->naxis;
+
+  if (cmp & WCSCOMPARE_CRPIX) {
+    /* Don't compare crpix. */
+  } else if (cmp & WCSCOMPARE_TILING) {
+    for (i = 0; i < naxis; ++i) {
+      diff = wcs1->crpix[i] - wcs2->crpix[i];
+      if ((double)(int)(diff) != diff) {
+        return 0;
+      }
+    }
+  } else {
+    if (!wcsutil_Eq(naxis, wcs1->crpix, wcs2->crpix)) {
+      return 0;
+    }
+  }
+
+  if (!wcsutil_Eq(naxis2, wcs1->pc, wcs2->pc) ||
+      !wcsutil_Eq(naxis, wcs1->cdelt, wcs2->cdelt) ||
+      !wcsutil_Eq(naxis, wcs1->crval, wcs2->crval) ||
+      !wcsutil_strEq(naxis, wcs1->cunit, wcs2->cunit) ||
+      !wcsutil_strEq(naxis, wcs1->ctype, wcs2->ctype) ||
+      wcs1->lonpole != wcs2->lonpole ||
+      wcs1->latpole != wcs2->latpole ||
+      wcs1->restfrq != wcs2->restfrq ||
+      wcs1->restwav != wcs2->restwav ||
+      wcs1->npv != wcs2->npv ||
+      wcs1->nps != wcs2->nps) {
+    return 0;
+  }
+
+  /* Compare pv cards, which may not be in the same order */
+  for (i = 0; i < wcs1->npv; ++i) {
+    for (j = 0; j < wcs2->npv; ++j) {
+      if (wcs1->pv[i].i == wcs2->pv[j].i &&
+          wcs1->pv[i].m == wcs2->pv[j].m) {
+        if (wcs1->pv[i].value != wcs2->pv[j].value) {
+          return 0;
+        }
+        break;
+      }
+    }
+    /* We didn't find a match, so they are not equal */
+    if (j == wcs2->npv) {
+      return 0;
+    }
+  }
+
+  /* Compare ps cards, which may not be in the same order */
+  for (i = 0; i < wcs1->nps; ++i) {
+    for (j = 0; j < wcs2->nps; ++j) {
+      if (wcs1->ps[i].i == wcs2->ps[j].i &&
+          wcs1->ps[i].m == wcs2->ps[j].m) {
+        if (strncmp(wcs1->ps[i].value, wcs2->ps[j].value, 72)) {
+          return 0;
+        }
+        break;
+      }
+    }
+    /* We didn't find a match, so they are not equal */
+    if (j == wcs2->nps) {
+      return 0;
+    }
+  }
+
+  if (wcs1->flag != WCSSET || wcs2->flag != WCSSET) {
+    if (!wcsutil_Eq(naxis2, wcs1->cd, wcs2->cd) ||
+        !wcsutil_Eq(naxis, wcs1->crota, wcs2->crota) ||
+        wcs1->altlin != wcs2->altlin ||
+        wcs1->velref != wcs2->velref) {
+      return 0;
+    }
+  }
+
+  if (!(cmp & WCSCOMPARE_ANCILLARY)) {
+    if (strncmp(wcs1->alt, wcs2->alt, 4) ||
+        wcs1->colnum != wcs2->colnum ||
+        !wcsutil_intEq(naxis, wcs1->colax, wcs2->colax) ||
+        !wcsutil_strEq(naxis, wcs1->cname, wcs2->cname) ||
+        !wcsutil_Eq(naxis, wcs1->crder, wcs2->crder) ||
+        !wcsutil_Eq(naxis, wcs1->csyer, wcs2->csyer) ||
+        strncmp(wcs1->dateavg, wcs2->dateavg, 72) ||
+        strncmp(wcs1->dateobs, wcs2->dateobs, 72) ||
+        wcs1->equinox != wcs2->equinox ||
+        wcs1->mjdavg != wcs2->mjdavg ||
+        wcs1->mjdobs != wcs2->mjdobs ||
+        !wcsutil_Eq(3, wcs1->obsgeo, wcs2->obsgeo) ||
+        strncmp(wcs1->radesys, wcs2->radesys, 72) ||
+        strncmp(wcs1->specsys, wcs2->specsys, 72) ||
+        strncmp(wcs1->ssysobs, wcs2->ssysobs, 72) ||
+        wcs1->velosys != wcs2->velosys ||
+        wcs1->zsource != wcs2->zsource ||
+        strncmp(wcs1->ssyssrc, wcs2->ssyssrc, 72) ||
+        wcs1->velangl != wcs2->velangl ||
+        strncmp(wcs1->wcsname, wcs2->wcsname, 72)) {
+      return 0;
+    }
+  }
+
+  /* Compare tabular parameters */
+  if (wcs1->ntab != wcs2->ntab) {
+    return 0;
+  }
+
+  for (i = 0; i < wcs1->ntab; ++i) {
+    if ((status = tabcmp(0, &wcs1->tab[i], &wcs2->tab[i], &tab_equal))) {
+      return status;
+    }
+    if (!tab_equal) {
+      return 0;
+    }
+  }
+
+  *equal = 1;
+  return 0;
+}
+
+/*--------------------------------------------------------------------------*/
+
 int wcsfree(struct wcsprm *wcs)
 
 {
@@ -1483,6 +1628,24 @@ int wcsperr(const struct wcsprm *wcs, const char *prefix)
 
 /*--------------------------------------------------------------------------*/
 
+int wcsbchk(struct wcsprm *wcs, int bounds)
+
+{
+  int status;
+
+  if (wcs == 0x0) return WCSERR_NULL_POINTER;
+
+  if (wcs->flag != WCSSET) {
+    if ((status = wcsset(wcs))) return status;
+  }
+
+  wcs->cel.prj.bounds = bounds;
+
+  return 0;
+}
+
+/*--------------------------------------------------------------------------*/
+
 int wcsset(struct wcsprm *wcs)
 
 {
@@ -2037,7 +2200,7 @@ int wcs_units(struct wcsprm *wcs)
       if (wcsunitse(wcs->cunit[i], units, &scale, &offset, &power,
                     &uniterr)) {
         wcserr_set(WCSERR_SET(WCSERR_BAD_COORD_TRANS),
-          "In CUNIT%d%.1s: %s", i, (*wcs->alt)?wcs->alt:"", uniterr->msg);
+          "In CUNIT%d%.1s: %s", i+1, (*wcs->alt)?wcs->alt:"", uniterr->msg);
         free(uniterr);
         return WCSERR_BAD_COORD_TRANS;
       }
