@@ -1,12 +1,11 @@
 # Set the WCS information manually by setting properties of the WCS
 # object.
 
-from __future__ import division # confidence high
+from __future__ import division, print_function
 
 import numpy
 from astropy import wcs
 from astropy.io import fits
-import sys
 
 # Create a new WCS object.  The number of axes must be set
 # from the start
@@ -21,15 +20,15 @@ w.wcs.ctype = ["RA---AIR", "DEC--AIR"]
 w.wcs.set_pv([(2, 1, 45.0)])
 
 # Some pixel coordinates of interest.
-pixcrd = numpy.array([[0,0],[24,38],[45,98]], numpy.float_)
+pixcrd = numpy.array([[0, 0], [24, 38], [45, 98]], numpy.float_)
 
 # Convert pixel coordinates to world coordinates
 world = w.wcs_pix2world(pixcrd, 1)
-print world
+print(world)
 
 # Convert the same coordinates back to pixel coordinates.
 pixcrd2 = w.wcs_world2pix(world, 1)
-print pixcrd2
+print(pixcrd2)
 
 # These should be the same as the original pixel coordinates, modulo
 # some floating-point error.
@@ -41,4 +40,5 @@ header = w.to_header()
 # header is an astropy.io.fits.Header object.  We can use it to create a new
 # PrimaryHDU and write it to a file.
 hdu = fits.PrimaryHDU(header=header)
-hdu.writeto('test.fits')
+# Save to FITS file
+# hdu.writeto('test.fits')
